export default [
    "{ filename: \"src/Adafruit_NeoPixel.cpp\", code: \"/*-------------------------------------------------------------------------\\r\\n  Arduino library to control a wide variety of WS2811- and WS2812-based RGB\\r\\n  LED devices such as Adafruit FLORA RGB Smart Pixels and NeoPixel strips.\\r\\n  Currently handles 400 and 800 KHz bitstreams on 8, 12 and 16 MHz ATmega\\r\\n  MCUs, with LEDs wired for various color orders.  Handles most output pins\\r\\n  (possible exception with upper PORT registers on the Arduino Mega).\\r\\n\\r\\n  Written by Phil Burgess / Paint Your Dragon for Adafruit Industries,\\r\\n  contributions by PJRC, Michael Miller and other members of the open\\r\\n  source community.\\r\\n\\r\\n  Adafruit invests time and resources providing this open source code,\\r\\n  please support Adafruit and open-source hardware by purchasing products\\r\\n  from Adafruit!\\r\\n\\r\\n  -------------------------------------------------------------------------\\r\\n  This file is part of the Adafruit NeoPixel library.\\r\\n\\r\\n  NeoPixel is free software: you can redistribute it and/or modify\\r\\n  it under the terms of the GNU Lesser General Public License as\\r\\n  published by the Free Software Foundation, either version 3 of\\r\\n  the License, or (at your option) any later version.\\r\\n\\r\\n  NeoPixel is distributed in the hope that it will be useful,\\r\\n  but WITHOUT ANY WARRANTY; without even the implied warranty of\\r\\n  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\\r\\n  GNU Lesser General Public License for more details.\\r\\n\\r\\n  You should have received a copy of the GNU Lesser General Public\\r\\n  License along with NeoPixel.  If not, see\\r\\n  <http://www.gnu.org/licenses/>.\\r\\n  -------------------------------------------------------------------------*/\\r\\n\\r\\n#include \\\"Adafruit_NeoPixel.h\\\"\\r\\n\\r\\n#if defined(NRF52)\\r\\n#include \\\"nrf.h\\\"\\r\\n\\r\\n// Interrupt is only disabled if there is no PWM device available\\r\\n// Note: Adafruit Bluefruit nrf52 does not use this option\\r\\n//#define NRF52_DISABLE_INT\\r\\n#endif\\r\\n\\r\\n// Constructor when length, pin and type are known at compile-time:\\r\\nAdafruit_NeoPixel::Adafruit_NeoPixel(uint16_t n, uint8_t p, neoPixelType t) :\\r\\n  begun(false), brightness(0), pixels(NULL), endTime(0)  \\r\\n{\\r\\n  updateType(t);\\r\\n  updateLength(n);\\r\\n  setPin(p);\\r\\n}\\r\\n\\r\\n// via Michael Vogt/neophob: empty constructor is used when strand length\\r\\n// isn't known at compile-time; situations where program config might be\\r\\n// read from internal flash memory or an SD card, or arrive via serial\\r\\n// command.  If using this constructor, MUST follow up with updateType(),\\r\\n// updateLength(), etc. to establish the strand type, length and pin number!\\r\\nAdafruit_NeoPixel::Adafruit_NeoPixel() :\\r\\n#ifdef NEO_KHZ400\\r\\n  is800KHz(true),\\r\\n#endif\\r\\n  begun(false), numLEDs(0), numBytes(0), pin(-1), brightness(0), pixels(NULL),\\r\\n  rOffset(1), gOffset(0), bOffset(2), wOffset(1), endTime(0)\\r\\n{\\r\\n}\\r\\n\\r\\nAdafruit_NeoPixel::~Adafruit_NeoPixel() {\\r\\n  if(pixels)   free(pixels);\\r\\n  if(pin >= 0) pinMode(pin, INPUT);\\r\\n}\\r\\n\\r\\nvoid Adafruit_NeoPixel::begin(void) {\\r\\n  if(pin >= 0) {\\r\\n    pinMode(pin, OUTPUT);\\r\\n    digitalWrite(pin, LOW);\\r\\n  }\\r\\n  begun = true;\\r\\n\\r\\n}\\r\\n\\r\\nvoid Adafruit_NeoPixel::updateLength(uint16_t n) {\\r\\n  if(pixels) free(pixels); // Free existing data (if any)\\r\\n\\r\\n  // Allocate new data -- note: ALL PIXELS ARE CLEARED\\r\\n  numBytes = n * ((wOffset == rOffset) ? 3 : 4);\\r\\n  if((pixels = (uint8_t *)malloc(numBytes))) {\\r\\n    memset(pixels, 0, numBytes);\\r\\n    numLEDs = n;\\r\\n  } else {\\r\\n    numLEDs = numBytes = 0;\\r\\n  }\\r\\n}\\r\\n\\r\\nvoid Adafruit_NeoPixel::updateType(neoPixelType t) {\\r\\n  boolean oldThreeBytesPerPixel = (wOffset == rOffset); // false if RGBW\\r\\n\\r\\n  wOffset = (t >> 6) & 0b11; // See notes in header file\\r\\n  rOffset = (t >> 4) & 0b11; // regarding R/G/B/W offsets\\r\\n  gOffset = (t >> 2) & 0b11;\\r\\n  bOffset =  t       & 0b11;\\r\\n#ifdef NEO_KHZ400\\r\\n  is800KHz = (t < 256);      // 400 KHz flag is 1<<8\\r\\n#endif\\r\\n\\r\\n  // If bytes-per-pixel has changed (and pixel data was previously\\r\\n  // allocated), re-allocate to new size.  Will clear any data.\\r\\n  if(pixels) {\\r\\n    boolean newThreeBytesPerPixel = (wOffset == rOffset);\\r\\n    if(newThreeBytesPerPixel != oldThreeBytesPerPixel) updateLength(numLEDs);\\r\\n  }\\r\\n}\\r\\n\\r\\n#if defined(ESP8266) \\r\\n// ESP8266 show() is external to enforce ICACHE_RAM_ATTR execution\\r\\nextern \\\"C\\\" void ICACHE_RAM_ATTR espShow(\\r\\n  uint8_t pin, uint8_t *pixels, uint32_t numBytes, uint8_t type);\\r\\n#elif defined(ESP32)\\r\\nextern \\\"C\\\" void espShow(\\r\\n  uint8_t pin, uint8_t *pixels, uint32_t numBytes, uint8_t type);\\r\\n#endif // ESP8266\\r\\n\\r\\nvoid Adafruit_NeoPixel::show(void) {\\r\\n\\r\\n  if(!pixels) return;\\r\\n\\r\\n  // Data latch = 300+ microsecond pause in the output stream.  Rather than\\r\\n  // put a delay at the end of the function, the ending time is noted and\\r\\n  // the function will simply hold off (if needed) on issuing the\\r\\n  // subsequent round of data until the latch time has elapsed.  This\\r\\n  // allows the mainline code to start generating the next frame of data\\r\\n  // rather than stalling for the latch.\\r\\n  while(!canShow());\\r\\n  // endTime is a private member (rather than global var) so that multiple\\r\\n  // instances on different pins can be quickly issued in succession (each\\r\\n  // instance doesn't delay the next).\\r\\n\\r\\n  // In order to make this code runtime-configurable to work with any pin,\\r\\n  // SBI/CBI instructions are eschewed in favor of full PORT writes via the\\r\\n  // OUT or ST instructions.  It relies on two facts: that peripheral\\r\\n  // functions (such as PWM) take precedence on output pins, so our PORT-\\r\\n  // wide writes won't interfere, and that interrupts are globally disabled\\r\\n  // while data is being issued to the LEDs, so no other code will be\\r\\n  // accessing the PORT.  The code takes an initial 'snapshot' of the PORT\\r\\n  // state, computes 'pin high' and 'pin low' values, and writes these back\\r\\n  // to the PORT register as needed.\\r\\n\\r\\n  // NRF52 may use PWM + DMA (if available), may not need to disable interrupt\\r\\n#ifndef NRF52\\r\\n  noInterrupts(); // Need 100% focus on instruction timing\\r\\n#endif\\r\\n\\r\\n#ifdef __AVR__\\r\\n// AVR MCUs -- ATmega & ATtiny (no XMEGA) ---------------------------------\\r\\n\\r\\n  volatile uint16_t\\r\\n    i   = numBytes; // Loop counter\\r\\n  volatile uint8_t\\r\\n   *ptr = pixels,   // Pointer to next byte\\r\\n    b   = *ptr++,   // Current byte value\\r\\n    hi,             // PORT w/output bit set high\\r\\n    lo;             // PORT w/output bit set low\\r\\n\\r\\n  // Hand-tuned assembly code issues data to the LED drivers at a specific\\r\\n  // rate.  There's separate code for different CPU speeds (8, 12, 16 MHz)\\r\\n  // for both the WS2811 (400 KHz) and WS2812 (800 KHz) drivers.  The\\r\\n  // datastream timing for the LED drivers allows a little wiggle room each\\r\\n  // way (listed in the datasheets), so the conditions for compiling each\\r\\n  // case are set up for a range of frequencies rather than just the exact\\r\\n  // 8, 12 or 16 MHz values, permitting use with some close-but-not-spot-on\\r\\n  // devices (e.g. 16.5 MHz DigiSpark).  The ranges were arrived at based\\r\\n  // on the datasheet figures and have not been extensively tested outside\\r\\n  // the canonical 8/12/16 MHz speeds; there's no guarantee these will work\\r\\n  // close to the extremes (or possibly they could be pushed further).\\r\\n  // Keep in mind only one CPU speed case actually gets compiled; the\\r\\n  // resulting program isn't as massive as it might look from source here.\\r\\n\\r\\n// 8 MHz(ish) AVR ---------------------------------------------------------\\r\\n#if (F_CPU >= 7400000UL) && (F_CPU <= 9500000UL)\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n\\r\\n    volatile uint8_t n1, n2 = 0;  // First, next bits out\\r\\n\\r\\n    // Squeezing an 800 KHz stream out of an 8 MHz chip requires code\\r\\n    // specific to each PORT register.\\r\\n\\r\\n    // 10 instruction clocks per bit: HHxxxxxLLL\\r\\n    // OUT instructions:              ^ ^    ^   (T=0,2,7)\\r\\n\\r\\n    // PORTD OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTD)\\r\\n #if defined(PORTB) || defined(PORTC) || defined(PORTF)\\r\\n    if(port == &PORTD) {\\r\\n #endif // defined(PORTB/C/F)\\r\\n\\r\\n      hi = PORTD |  pinMask;\\r\\n      lo = PORTD & ~pinMask;\\r\\n      n1 = lo;\\r\\n      if(b & 0x80) n1 = hi;\\r\\n\\r\\n      // Dirty trick: RJMPs proceeding to the next instruction are used\\r\\n      // to delay two clock cycles in one instruction word (rather than\\r\\n      // using two NOPs).  This was necessary in order to squeeze the\\r\\n      // loop down to exactly 64 words -- the maximum possible for a\\r\\n      // relative branch.\\r\\n\\r\\n      asm volatile(\\r\\n       \\\"headD:\\\"                   \\\"\\\\n\\\\t\\\" // Clk  Pseudocode\\r\\n        // Bit 7:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n2   = lo\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n1\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 6\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x40)\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n2 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        // Bit 6:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n1   = lo\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n2\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 5\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x20)\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n1 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        // Bit 5:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n2   = lo\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n1\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 4\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x10)\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n2 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        // Bit 4:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n1   = lo\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n2\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 3\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x08)\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n1 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        // Bit 3:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n2   = lo\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n1\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 2\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x04)\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n2 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        // Bit 2:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n1   = lo\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n2\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 1\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x02)\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n1 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        // Bit 1:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n2   = lo\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n1\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop\\r\\n        \\\"sbrc %[byte] , 0\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x01)\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n2 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\" // 2    i-- (don't act on Z flag yet)\\r\\n        // Bit 0:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    n1   = lo\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = n2\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\" // 2    b = *ptr++\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x80)\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   n1 = hi\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo\\r\\n        \\\"brne headD\\\"              \\\"\\\\n\\\"   // 2    while(i) (Z flag set above)\\r\\n      : [byte]  \\\"+r\\\" (b),\\r\\n        [n1]    \\\"+r\\\" (n1),\\r\\n        [n2]    \\\"+r\\\" (n2),\\r\\n        [count] \\\"+w\\\" (i)\\r\\n      : [port]   \\\"I\\\" (_SFR_IO_ADDR(PORTD)),\\r\\n        [ptr]    \\\"e\\\" (ptr),\\r\\n        [hi]     \\\"r\\\" (hi),\\r\\n        [lo]     \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTB) || defined(PORTC) || defined(PORTF)\\r\\n    } else // other PORT(s)\\r\\n #endif // defined(PORTB/C/F)\\r\\n#endif // defined(PORTD)\\r\\n\\r\\n    // PORTB OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTB)\\r\\n #if defined(PORTD) || defined(PORTC) || defined(PORTF)\\r\\n    if(port == &PORTB) {\\r\\n #endif // defined(PORTD/C/F)\\r\\n\\r\\n      // Same as above, just switched to PORTB and stripped of comments.\\r\\n      hi = PORTB |  pinMask;\\r\\n      lo = PORTB & ~pinMask;\\r\\n      n1 = lo;\\r\\n      if(b & 0x80) n1 = hi;\\r\\n\\r\\n      asm volatile(\\r\\n       \\\"headB:\\\"                   \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 6\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 5\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 4\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 3\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 2\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 0\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"brne headB\\\"              \\\"\\\\n\\\"\\r\\n      : [byte] \\\"+r\\\" (b), [n1] \\\"+r\\\" (n1), [n2] \\\"+r\\\" (n2), [count] \\\"+w\\\" (i)\\r\\n      : [port] \\\"I\\\" (_SFR_IO_ADDR(PORTB)), [ptr] \\\"e\\\" (ptr), [hi] \\\"r\\\" (hi),\\r\\n        [lo] \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTD) || defined(PORTC) || defined(PORTF)\\r\\n    }\\r\\n #endif\\r\\n #if defined(PORTC) || defined(PORTF)\\r\\n    else\\r\\n #endif // defined(PORTC/F)\\r\\n#endif // defined(PORTB)\\r\\n\\r\\n    // PORTC OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTC)\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTF)\\r\\n    if(port == &PORTC) {\\r\\n #endif // defined(PORTD/B/F)\\r\\n\\r\\n      // Same as above, just switched to PORTC and stripped of comments.\\r\\n      hi = PORTC |  pinMask;\\r\\n      lo = PORTC & ~pinMask;\\r\\n      n1 = lo;\\r\\n      if(b & 0x80) n1 = hi;\\r\\n\\r\\n      asm volatile(\\r\\n       \\\"headC:\\\"                   \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 6\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 5\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 4\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 3\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 2\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 0\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"brne headC\\\"              \\\"\\\\n\\\"\\r\\n      : [byte] \\\"+r\\\" (b), [n1] \\\"+r\\\" (n1), [n2] \\\"+r\\\" (n2), [count] \\\"+w\\\" (i)\\r\\n      : [port] \\\"I\\\" (_SFR_IO_ADDR(PORTC)), [ptr] \\\"e\\\" (ptr), [hi] \\\"r\\\" (hi),\\r\\n        [lo] \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTF)\\r\\n    }\\r\\n #endif // defined(PORTD/B/F)\\r\\n #if defined(PORTF)\\r\\n    else\\r\\n #endif\\r\\n#endif // defined(PORTC)\\r\\n\\r\\n    // PORTF OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTF)\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTC)\\r\\n    if(port == &PORTF) {\\r\\n #endif // defined(PORTD/B/C)\\r\\n\\r\\n      hi = PORTF |  pinMask;\\r\\n      lo = PORTF & ~pinMask;\\r\\n      n1 = lo;\\r\\n      if(b & 0x80) n1 = hi;\\r\\n\\r\\n      asm volatile(\\r\\n       \\\"headF:\\\"                   \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 6\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 5\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 4\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 3\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 2\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n2]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n1]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 0\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n2]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[n1]   , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[n2]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[n1]   , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"brne headF\\\"              \\\"\\\\n\\\"\\r\\n      : [byte] \\\"+r\\\" (b), [n1] \\\"+r\\\" (n1), [n2] \\\"+r\\\" (n2), [count] \\\"+w\\\" (i)\\r\\n      : [port] \\\"I\\\" (_SFR_IO_ADDR(PORTF)), [ptr] \\\"e\\\" (ptr), [hi] \\\"r\\\" (hi),\\r\\n        [lo] \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTC)\\r\\n    }\\r\\n #endif // defined(PORTD/B/C)\\r\\n#endif // defined(PORTF)\\r\\n\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // end 800 KHz, do 400 KHz\\r\\n\\r\\n    // Timing is more relaxed; unrolling the inner loop for each bit is\\r\\n    // not necessary.  Still using the peculiar RJMPs as 2X NOPs, not out\\r\\n    // of need but just to trim the code size down a little.\\r\\n    // This 400-KHz-datastream-on-8-MHz-CPU code is not quite identical\\r\\n    // to the 800-on-16 code later -- the hi/lo timing between WS2811 and\\r\\n    // WS2812 is not simply a 2:1 scale!\\r\\n\\r\\n    // 20 inst. clocks per bit: HHHHxxxxxxLLLLLLLLLL\\r\\n    // ST instructions:         ^   ^     ^          (T=0,4,10)\\r\\n\\r\\n    volatile uint8_t next, bit;\\r\\n\\r\\n    hi   = *port |  pinMask;\\r\\n    lo   = *port & ~pinMask;\\r\\n    next = lo;\\r\\n    bit  = 8;\\r\\n\\r\\n    asm volatile(\\r\\n     \\\"head20:\\\"                  \\\"\\\\n\\\\t\\\" // Clk  Pseudocode    (T =  0)\\r\\n      \\\"st   %a[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = hi     (T =  2)\\r\\n      \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 128)\\r\\n       \\\"mov  %[next], %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   next = hi    (T =  4)\\r\\n      \\\"st   %a[port], %[next]\\\"  \\\"\\\\n\\\\t\\\" // 2    PORT = next   (T =  6)\\r\\n      \\\"mov  %[next] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    next = lo     (T =  7)\\r\\n      \\\"dec  %[bit]\\\"             \\\"\\\\n\\\\t\\\" // 1    bit--         (T =  8)\\r\\n      \\\"breq nextbyte20\\\"         \\\"\\\\n\\\\t\\\" // 1-2  if(bit == 0)\\r\\n      \\\"rol  %[byte]\\\"            \\\"\\\\n\\\\t\\\" // 1    b <<= 1       (T = 10)\\r\\n      \\\"st   %a[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 12)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 14)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 16)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 18)\\r\\n      \\\"rjmp head20\\\"             \\\"\\\\n\\\\t\\\" // 2    -> head20 (next bit out)\\r\\n     \\\"nextbyte20:\\\"              \\\"\\\\n\\\\t\\\" //                    (T = 10)\\r\\n      \\\"st   %a[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 12)\\r\\n      \\\"nop\\\"                     \\\"\\\\n\\\\t\\\" // 1    nop           (T = 13)\\r\\n      \\\"ldi  %[bit]  , 8\\\"        \\\"\\\\n\\\\t\\\" // 1    bit = 8       (T = 14)\\r\\n      \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\" // 2    b = *ptr++    (T = 16)\\r\\n      \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\" // 2    i--           (T = 18)\\r\\n      \\\"brne head20\\\"             \\\"\\\\n\\\"   // 2    if(i != 0) -> (next byte)\\r\\n      : [port]  \\\"+e\\\" (port),\\r\\n        [byte]  \\\"+r\\\" (b),\\r\\n        [bit]   \\\"+r\\\" (bit),\\r\\n        [next]  \\\"+r\\\" (next),\\r\\n        [count] \\\"+w\\\" (i)\\r\\n      : [hi]    \\\"r\\\" (hi),\\r\\n        [lo]    \\\"r\\\" (lo),\\r\\n        [ptr]   \\\"e\\\" (ptr));\\r\\n  }\\r\\n#endif // NEO_KHZ400\\r\\n\\r\\n// 12 MHz(ish) AVR --------------------------------------------------------\\r\\n#elif (F_CPU >= 11100000UL) && (F_CPU <= 14300000UL)\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n\\r\\n    // In the 12 MHz case, an optimized 800 KHz datastream (no dead time\\r\\n    // between bytes) requires a PORT-specific loop similar to the 8 MHz\\r\\n    // code (but a little more relaxed in this case).\\r\\n\\r\\n    // 15 instruction clocks per bit: HHHHxxxxxxLLLLL\\r\\n    // OUT instructions:              ^   ^     ^     (T=0,4,10)\\r\\n\\r\\n    volatile uint8_t next;\\r\\n\\r\\n    // PORTD OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTD)\\r\\n #if defined(PORTB) || defined(PORTC) || defined(PORTF)\\r\\n    if(port == &PORTD) {\\r\\n #endif // defined(PORTB/C/F)\\r\\n\\r\\n      hi   = PORTD |  pinMask;\\r\\n      lo   = PORTD & ~pinMask;\\r\\n      next = lo;\\r\\n      if(b & 0x80) next = hi;\\r\\n\\r\\n      // Don't \\\"optimize\\\" the OUT calls into the bitTime subroutine;\\r\\n      // we're exploiting the RCALL and RET as 3- and 4-cycle NOPs!\\r\\n      asm volatile(\\r\\n       \\\"headD:\\\"                   \\\"\\\\n\\\\t\\\" //        (T =  0)\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\" //        (T =  1)\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 7  (T = 15)\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 6\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 5\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 4\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 3\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 2\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeD\\\"          \\\"\\\\n\\\\t\\\" // Bit 1\\r\\n        // Bit 0:\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = hi    (T =  1)\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop      (T =  3)\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\" // 2    b = *ptr++   (T =  5)\\r\\n        \\\"out  %[port] , %[next]\\\"  \\\"\\\\n\\\\t\\\" // 1    PORT = next  (T =  6)\\r\\n        \\\"mov  %[next] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    next = lo    (T =  7)\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x80) (T =  8)\\r\\n         \\\"mov %[next] , %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1    next = hi  (T =  9)\\r\\n        \\\"nop\\\"                     \\\"\\\\n\\\\t\\\" // 1                 (T = 10)\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo    (T = 11)\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\" // 2    i--          (T = 13)\\r\\n        \\\"brne headD\\\"              \\\"\\\\n\\\\t\\\" // 2    if(i != 0) -> (next byte)\\r\\n         \\\"rjmp doneD\\\"             \\\"\\\\n\\\\t\\\"\\r\\n        \\\"bitTimeD:\\\"               \\\"\\\\n\\\\t\\\" //      nop nop nop     (T =  4)\\r\\n         \\\"out  %[port], %[next]\\\"  \\\"\\\\n\\\\t\\\" // 1    PORT = next     (T =  5)\\r\\n         \\\"mov  %[next], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    next = lo       (T =  6)\\r\\n         \\\"rol  %[byte]\\\"           \\\"\\\\n\\\\t\\\" // 1    b <<= 1         (T =  7)\\r\\n         \\\"sbrc %[byte], 7\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 0x80)    (T =  8)\\r\\n          \\\"mov %[next], %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   next = hi      (T =  9)\\r\\n         \\\"nop\\\"                    \\\"\\\\n\\\\t\\\" // 1                    (T = 10)\\r\\n         \\\"out  %[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    PORT = lo       (T = 11)\\r\\n         \\\"ret\\\"                    \\\"\\\\n\\\\t\\\" // 4    nop nop nop nop (T = 15)\\r\\n         \\\"doneD:\\\"                 \\\"\\\\n\\\"\\r\\n        : [byte]  \\\"+r\\\" (b),\\r\\n          [next]  \\\"+r\\\" (next),\\r\\n          [count] \\\"+w\\\" (i)\\r\\n        : [port]   \\\"I\\\" (_SFR_IO_ADDR(PORTD)),\\r\\n          [ptr]    \\\"e\\\" (ptr),\\r\\n          [hi]     \\\"r\\\" (hi),\\r\\n          [lo]     \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTB) || defined(PORTC) || defined(PORTF)\\r\\n    } else // other PORT(s)\\r\\n #endif // defined(PORTB/C/F)\\r\\n#endif // defined(PORTD)\\r\\n\\r\\n    // PORTB OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTB)\\r\\n #if defined(PORTD) || defined(PORTC) || defined(PORTF)\\r\\n    if(port == &PORTB) {\\r\\n #endif // defined(PORTD/C/F)\\r\\n\\r\\n      hi   = PORTB |  pinMask;\\r\\n      lo   = PORTB & ~pinMask;\\r\\n      next = lo;\\r\\n      if(b & 0x80) next = hi;\\r\\n\\r\\n      // Same as above, just set for PORTB & stripped of comments\\r\\n      asm volatile(\\r\\n       \\\"headB:\\\"                   \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeB\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[next]\\\"  \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[next] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[next] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"nop\\\"                     \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n        \\\"brne headB\\\"              \\\"\\\\n\\\\t\\\"\\r\\n         \\\"rjmp doneB\\\"             \\\"\\\\n\\\\t\\\"\\r\\n        \\\"bitTimeB:\\\"               \\\"\\\\n\\\\t\\\"\\r\\n         \\\"out  %[port], %[next]\\\"  \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov  %[next], %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"rol  %[byte]\\\"           \\\"\\\\n\\\\t\\\"\\r\\n         \\\"sbrc %[byte], 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n          \\\"mov %[next], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"nop\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"out  %[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"ret\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"doneB:\\\"                 \\\"\\\\n\\\"\\r\\n        : [byte] \\\"+r\\\" (b), [next] \\\"+r\\\" (next), [count] \\\"+w\\\" (i)\\r\\n        : [port] \\\"I\\\" (_SFR_IO_ADDR(PORTB)), [ptr] \\\"e\\\" (ptr), [hi] \\\"r\\\" (hi),\\r\\n          [lo] \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTD) || defined(PORTC) || defined(PORTF)\\r\\n    }\\r\\n #endif\\r\\n #if defined(PORTC) || defined(PORTF)\\r\\n    else\\r\\n #endif // defined(PORTC/F)\\r\\n#endif // defined(PORTB)\\r\\n\\r\\n    // PORTC OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTC)\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTF)\\r\\n    if(port == &PORTC) {\\r\\n #endif // defined(PORTD/B/F)\\r\\n\\r\\n      hi   = PORTC |  pinMask;\\r\\n      lo   = PORTC & ~pinMask;\\r\\n      next = lo;\\r\\n      if(b & 0x80) next = hi;\\r\\n\\r\\n      // Same as above, just set for PORTC & stripped of comments\\r\\n      asm volatile(\\r\\n       \\\"headC:\\\"                   \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[next]\\\"  \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[next] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[next] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"nop\\\"                     \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n        \\\"brne headC\\\"              \\\"\\\\n\\\\t\\\"\\r\\n         \\\"rjmp doneC\\\"             \\\"\\\\n\\\\t\\\"\\r\\n        \\\"bitTimeC:\\\"               \\\"\\\\n\\\\t\\\"\\r\\n         \\\"out  %[port], %[next]\\\"  \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov  %[next], %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"rol  %[byte]\\\"           \\\"\\\\n\\\\t\\\"\\r\\n         \\\"sbrc %[byte], 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n          \\\"mov %[next], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"nop\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"out  %[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"ret\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"doneC:\\\"                 \\\"\\\\n\\\"\\r\\n        : [byte] \\\"+r\\\" (b), [next] \\\"+r\\\" (next), [count] \\\"+w\\\" (i)\\r\\n        : [port] \\\"I\\\" (_SFR_IO_ADDR(PORTC)), [ptr] \\\"e\\\" (ptr), [hi] \\\"r\\\" (hi),\\r\\n          [lo] \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTF)\\r\\n    }\\r\\n #endif // defined(PORTD/B/F)\\r\\n #if defined(PORTF)\\r\\n    else\\r\\n #endif\\r\\n#endif // defined(PORTC)\\r\\n\\r\\n    // PORTF OUTPUT ----------------------------------------------------\\r\\n\\r\\n#if defined(PORTF)\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTC)\\r\\n    if(port == &PORTF) {\\r\\n #endif // defined(PORTD/B/C)\\r\\n\\r\\n      hi   = PORTF |  pinMask;\\r\\n      lo   = PORTF & ~pinMask;\\r\\n      next = lo;\\r\\n      if(b & 0x80) next = hi;\\r\\n\\r\\n      // Same as above, just set for PORTF & stripped of comments\\r\\n      asm volatile(\\r\\n       \\\"headF:\\\"                   \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out   %[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rcall bitTimeC\\\"          \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\"\\r\\n        \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[next]\\\"  \\\"\\\\n\\\\t\\\"\\r\\n        \\\"mov  %[next] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov %[next] , %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"nop\\\"                     \\\"\\\\n\\\\t\\\"\\r\\n        \\\"out  %[port] , %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n        \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\"\\r\\n        \\\"brne headF\\\"              \\\"\\\\n\\\\t\\\"\\r\\n         \\\"rjmp doneC\\\"             \\\"\\\\n\\\\t\\\"\\r\\n        \\\"bitTimeC:\\\"               \\\"\\\\n\\\\t\\\"\\r\\n         \\\"out  %[port], %[next]\\\"  \\\"\\\\n\\\\t\\\"\\r\\n         \\\"mov  %[next], %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"rol  %[byte]\\\"           \\\"\\\\n\\\\t\\\"\\r\\n         \\\"sbrc %[byte], 7\\\"        \\\"\\\\n\\\\t\\\"\\r\\n          \\\"mov %[next], %[hi]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"nop\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"out  %[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"ret\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n         \\\"doneC:\\\"                 \\\"\\\\n\\\"\\r\\n        : [byte] \\\"+r\\\" (b), [next] \\\"+r\\\" (next), [count] \\\"+w\\\" (i)\\r\\n        : [port] \\\"I\\\" (_SFR_IO_ADDR(PORTF)), [ptr] \\\"e\\\" (ptr), [hi] \\\"r\\\" (hi),\\r\\n          [lo] \\\"r\\\" (lo));\\r\\n\\r\\n #if defined(PORTD) || defined(PORTB) || defined(PORTC)\\r\\n    }\\r\\n #endif // defined(PORTD/B/C)\\r\\n#endif // defined(PORTF)\\r\\n\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 KHz\\r\\n\\r\\n    // 30 instruction clocks per bit: HHHHHHxxxxxxxxxLLLLLLLLLLLLLLL\\r\\n    // ST instructions:               ^     ^        ^    (T=0,6,15)\\r\\n\\r\\n    volatile uint8_t next, bit;\\r\\n\\r\\n    hi   = *port |  pinMask;\\r\\n    lo   = *port & ~pinMask;\\r\\n    next = lo;\\r\\n    bit  = 8;\\r\\n\\r\\n    asm volatile(\\r\\n     \\\"head30:\\\"                  \\\"\\\\n\\\\t\\\" // Clk  Pseudocode    (T =  0)\\r\\n      \\\"st   %a[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = hi     (T =  2)\\r\\n      \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 128)\\r\\n       \\\"mov  %[next], %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 0-1   next = hi    (T =  4)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T =  6)\\r\\n      \\\"st   %a[port], %[next]\\\"  \\\"\\\\n\\\\t\\\" // 2    PORT = next   (T =  8)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 10)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 12)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 14)\\r\\n      \\\"nop\\\"                     \\\"\\\\n\\\\t\\\" // 1    nop           (T = 15)\\r\\n      \\\"st   %a[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 17)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 19)\\r\\n      \\\"dec  %[bit]\\\"             \\\"\\\\n\\\\t\\\" // 1    bit--         (T = 20)\\r\\n      \\\"breq nextbyte30\\\"         \\\"\\\\n\\\\t\\\" // 1-2  if(bit == 0)\\r\\n      \\\"rol  %[byte]\\\"            \\\"\\\\n\\\\t\\\" // 1    b <<= 1       (T = 22)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 24)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 26)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 28)\\r\\n      \\\"rjmp head30\\\"             \\\"\\\\n\\\\t\\\" // 2    -> head30 (next bit out)\\r\\n     \\\"nextbyte30:\\\"              \\\"\\\\n\\\\t\\\" //                    (T = 22)\\r\\n      \\\"nop\\\"                     \\\"\\\\n\\\\t\\\" // 1    nop           (T = 23)\\r\\n      \\\"ldi  %[bit]  , 8\\\"        \\\"\\\\n\\\\t\\\" // 1    bit = 8       (T = 24)\\r\\n      \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\" // 2    b = *ptr++    (T = 26)\\r\\n      \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\" // 2    i--           (T = 28)\\r\\n      \\\"brne head30\\\"             \\\"\\\\n\\\"   // 1-2  if(i != 0) -> (next byte)\\r\\n      : [port]  \\\"+e\\\" (port),\\r\\n        [byte]  \\\"+r\\\" (b),\\r\\n        [bit]   \\\"+r\\\" (bit),\\r\\n        [next]  \\\"+r\\\" (next),\\r\\n        [count] \\\"+w\\\" (i)\\r\\n      : [hi]     \\\"r\\\" (hi),\\r\\n        [lo]     \\\"r\\\" (lo),\\r\\n        [ptr]    \\\"e\\\" (ptr));\\r\\n  }\\r\\n#endif // NEO_KHZ400\\r\\n\\r\\n// 16 MHz(ish) AVR --------------------------------------------------------\\r\\n#elif (F_CPU >= 15400000UL) && (F_CPU <= 19000000L)\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n\\r\\n    // WS2811 and WS2812 have different hi/lo duty cycles; this is\\r\\n    // similar but NOT an exact copy of the prior 400-on-8 code.\\r\\n\\r\\n    // 20 inst. clocks per bit: HHHHHxxxxxxxxLLLLLLL\\r\\n    // ST instructions:         ^   ^        ^       (T=0,5,13)\\r\\n\\r\\n    volatile uint8_t next, bit;\\r\\n\\r\\n    hi   = *port |  pinMask;\\r\\n    lo   = *port & ~pinMask;\\r\\n    next = lo;\\r\\n    bit  = 8;\\r\\n\\r\\n    asm volatile(\\r\\n     \\\"head20:\\\"                   \\\"\\\\n\\\\t\\\" // Clk  Pseudocode    (T =  0)\\r\\n      \\\"st   %a[port],  %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = hi     (T =  2)\\r\\n      \\\"sbrc %[byte],  7\\\"         \\\"\\\\n\\\\t\\\" // 1-2  if(b & 128)\\r\\n       \\\"mov  %[next], %[hi]\\\"     \\\"\\\\n\\\\t\\\" // 0-1   next = hi    (T =  4)\\r\\n      \\\"dec  %[bit]\\\"              \\\"\\\\n\\\\t\\\" // 1    bit--         (T =  5)\\r\\n      \\\"st   %a[port],  %[next]\\\"  \\\"\\\\n\\\\t\\\" // 2    PORT = next   (T =  7)\\r\\n      \\\"mov  %[next] ,  %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    next = lo     (T =  8)\\r\\n      \\\"breq nextbyte20\\\"          \\\"\\\\n\\\\t\\\" // 1-2  if(bit == 0) (from dec above)\\r\\n      \\\"rol  %[byte]\\\"             \\\"\\\\n\\\\t\\\" // 1    b <<= 1       (T = 10)\\r\\n      \\\"rjmp .+0\\\"                 \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 12)\\r\\n      \\\"nop\\\"                      \\\"\\\\n\\\\t\\\" // 1    nop           (T = 13)\\r\\n      \\\"st   %a[port],  %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 15)\\r\\n      \\\"nop\\\"                      \\\"\\\\n\\\\t\\\" // 1    nop           (T = 16)\\r\\n      \\\"rjmp .+0\\\"                 \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 18)\\r\\n      \\\"rjmp head20\\\"              \\\"\\\\n\\\\t\\\" // 2    -> head20 (next bit out)\\r\\n     \\\"nextbyte20:\\\"               \\\"\\\\n\\\\t\\\" //                    (T = 10)\\r\\n      \\\"ldi  %[bit]  ,  8\\\"        \\\"\\\\n\\\\t\\\" // 1    bit = 8       (T = 11)\\r\\n      \\\"ld   %[byte] ,  %a[ptr]+\\\" \\\"\\\\n\\\\t\\\" // 2    b = *ptr++    (T = 13)\\r\\n      \\\"st   %a[port], %[lo]\\\"     \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 15)\\r\\n      \\\"nop\\\"                      \\\"\\\\n\\\\t\\\" // 1    nop           (T = 16)\\r\\n      \\\"sbiw %[count], 1\\\"         \\\"\\\\n\\\\t\\\" // 2    i--           (T = 18)\\r\\n       \\\"brne head20\\\"             \\\"\\\\n\\\"   // 2    if(i != 0) -> (next byte)\\r\\n      : [port]  \\\"+e\\\" (port),\\r\\n        [byte]  \\\"+r\\\" (b),\\r\\n        [bit]   \\\"+r\\\" (bit),\\r\\n        [next]  \\\"+r\\\" (next),\\r\\n        [count] \\\"+w\\\" (i)\\r\\n      : [ptr]    \\\"e\\\" (ptr),\\r\\n        [hi]     \\\"r\\\" (hi),\\r\\n        [lo]     \\\"r\\\" (lo));\\r\\n\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 KHz\\r\\n\\r\\n    // The 400 KHz clock on 16 MHz MCU is the most 'relaxed' version.\\r\\n\\r\\n    // 40 inst. clocks per bit: HHHHHHHHxxxxxxxxxxxxLLLLLLLLLLLLLLLLLLLL\\r\\n    // ST instructions:         ^       ^           ^         (T=0,8,20)\\r\\n\\r\\n    volatile uint8_t next, bit;\\r\\n\\r\\n    hi   = *port |  pinMask;\\r\\n    lo   = *port & ~pinMask;\\r\\n    next = lo;\\r\\n    bit  = 8;\\r\\n\\r\\n    asm volatile(\\r\\n     \\\"head40:\\\"                  \\\"\\\\n\\\\t\\\" // Clk  Pseudocode    (T =  0)\\r\\n      \\\"st   %a[port], %[hi]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = hi     (T =  2)\\r\\n      \\\"sbrc %[byte] , 7\\\"        \\\"\\\\n\\\\t\\\" // 1-2  if(b & 128)\\r\\n       \\\"mov  %[next] , %[hi]\\\"   \\\"\\\\n\\\\t\\\" // 0-1   next = hi    (T =  4)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T =  6)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T =  8)\\r\\n      \\\"st   %a[port], %[next]\\\"  \\\"\\\\n\\\\t\\\" // 2    PORT = next   (T = 10)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 12)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 14)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 16)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 18)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 20)\\r\\n      \\\"st   %a[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 22)\\r\\n      \\\"nop\\\"                     \\\"\\\\n\\\\t\\\" // 1    nop           (T = 23)\\r\\n      \\\"mov  %[next] , %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 1    next = lo     (T = 24)\\r\\n      \\\"dec  %[bit]\\\"             \\\"\\\\n\\\\t\\\" // 1    bit--         (T = 25)\\r\\n      \\\"breq nextbyte40\\\"         \\\"\\\\n\\\\t\\\" // 1-2  if(bit == 0)\\r\\n      \\\"rol  %[byte]\\\"            \\\"\\\\n\\\\t\\\" // 1    b <<= 1       (T = 27)\\r\\n      \\\"nop\\\"                     \\\"\\\\n\\\\t\\\" // 1    nop           (T = 28)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 30)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 32)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 34)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 36)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 38)\\r\\n      \\\"rjmp head40\\\"             \\\"\\\\n\\\\t\\\" // 2    -> head40 (next bit out)\\r\\n     \\\"nextbyte40:\\\"              \\\"\\\\n\\\\t\\\" //                    (T = 27)\\r\\n      \\\"ldi  %[bit]  , 8\\\"        \\\"\\\\n\\\\t\\\" // 1    bit = 8       (T = 28)\\r\\n      \\\"ld   %[byte] , %a[ptr]+\\\" \\\"\\\\n\\\\t\\\" // 2    b = *ptr++    (T = 30)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 32)\\r\\n      \\\"st   %a[port], %[lo]\\\"    \\\"\\\\n\\\\t\\\" // 2    PORT = lo     (T = 34)\\r\\n      \\\"rjmp .+0\\\"                \\\"\\\\n\\\\t\\\" // 2    nop nop       (T = 36)\\r\\n      \\\"sbiw %[count], 1\\\"        \\\"\\\\n\\\\t\\\" // 2    i--           (T = 38)\\r\\n      \\\"brne head40\\\"             \\\"\\\\n\\\"   // 1-2  if(i != 0) -> (next byte)\\r\\n      : [port]  \\\"+e\\\" (port),\\r\\n        [byte]  \\\"+r\\\" (b),\\r\\n        [bit]   \\\"+r\\\" (bit),\\r\\n        [next]  \\\"+r\\\" (next),\\r\\n        [count] \\\"+w\\\" (i)\\r\\n      : [ptr]    \\\"e\\\" (ptr),\\r\\n        [hi]     \\\"r\\\" (hi),\\r\\n        [lo]     \\\"r\\\" (lo));\\r\\n  }\\r\\n#endif // NEO_KHZ400\\r\\n\\r\\n#else\\r\\n #error \\\"CPU SPEED NOT SUPPORTED\\\"\\r\\n#endif // end F_CPU ifdefs on __AVR__\\r\\n\\r\\n// END AVR ----------------------------------------------------------------\\r\\n\\r\\n\\r\\n#elif defined(__arm__)\\r\\n\\r\\n// ARM MCUs -- Teensy 3.0, 3.1, LC, Arduino Due ---------------------------\\r\\n\\r\\n#if defined(TEENSYDUINO) && defined(KINETISK) // Teensy 3.0, 3.1, 3.2, 3.5, 3.6\\r\\n#define CYCLES_800_T0H  (F_CPU / 4000000)\\r\\n#define CYCLES_800_T1H  (F_CPU / 1250000)\\r\\n#define CYCLES_800      (F_CPU /  800000)\\r\\n#define CYCLES_400_T0H  (F_CPU / 2000000)\\r\\n#define CYCLES_400_T1H  (F_CPU /  833333)\\r\\n#define CYCLES_400      (F_CPU /  400000)\\r\\n\\r\\n  uint8_t          *p   = pixels,\\r\\n                   *end = p + numBytes, pix, mask;\\r\\n  volatile uint8_t *set = portSetRegister(pin),\\r\\n                   *clr = portClearRegister(pin);\\r\\n  uint32_t          cyc;\\r\\n\\r\\n  ARM_DEMCR    |= ARM_DEMCR_TRCENA;\\r\\n  ARM_DWT_CTRL |= ARM_DWT_CTRL_CYCCNTENA;\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n    cyc = ARM_DWT_CYCCNT + CYCLES_800;\\r\\n    while(p < end) {\\r\\n      pix = *p++;\\r\\n      for(mask = 0x80; mask; mask >>= 1) {\\r\\n        while(ARM_DWT_CYCCNT - cyc < CYCLES_800);\\r\\n        cyc  = ARM_DWT_CYCCNT;\\r\\n        *set = 1;\\r\\n        if(pix & mask) {\\r\\n          while(ARM_DWT_CYCCNT - cyc < CYCLES_800_T1H);\\r\\n        } else {\\r\\n          while(ARM_DWT_CYCCNT - cyc < CYCLES_800_T0H);\\r\\n        }\\r\\n        *clr = 1;\\r\\n      }\\r\\n    }\\r\\n    while(ARM_DWT_CYCCNT - cyc < CYCLES_800);\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 kHz bitstream\\r\\n    cyc = ARM_DWT_CYCCNT + CYCLES_400;\\r\\n    while(p < end) {\\r\\n      pix = *p++;\\r\\n      for(mask = 0x80; mask; mask >>= 1) {\\r\\n        while(ARM_DWT_CYCCNT - cyc < CYCLES_400);\\r\\n        cyc  = ARM_DWT_CYCCNT;\\r\\n        *set = 1;\\r\\n        if(pix & mask) {\\r\\n          while(ARM_DWT_CYCCNT - cyc < CYCLES_400_T1H);\\r\\n        } else {\\r\\n          while(ARM_DWT_CYCCNT - cyc < CYCLES_400_T0H);\\r\\n        }\\r\\n        *clr = 1;\\r\\n      }\\r\\n    }\\r\\n    while(ARM_DWT_CYCCNT - cyc < CYCLES_400);\\r\\n  }\\r\\n#endif // NEO_KHZ400\\r\\n\\r\\n#elif defined(TEENSYDUINO) && defined(__MKL26Z64__) // Teensy-LC\\r\\n\\r\\n#if F_CPU == 48000000\\r\\n  uint8_t          *p   = pixels,\\r\\n\\t\\t   pix, count, dly,\\r\\n                   bitmask = digitalPinToBitMask(pin);\\r\\n  volatile uint8_t *reg = portSetRegister(pin);\\r\\n  uint32_t         num = numBytes;\\r\\n  asm volatile(\\r\\n\\t\\\"L%=_begin:\\\"\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"ldrb\\t%[pix], [%[p], #0]\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"lsl\\t%[pix], #24\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[count], #7\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop:\\\"\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"lsl\\t%[pix], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bcs\\tL%=_loop_one\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop_zero:\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #0]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #4\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop_delay_T0H:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_loop_delay_T0H\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #4]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #13\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop_delay_T0L:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_loop_delay_T0L\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"b\\tL%=_next\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop_one:\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #0]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #13\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop_delay_T1H:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_loop_delay_T1H\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #4]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #4\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_loop_delay_T1L:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_loop_delay_T1L\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"nop\\\"\\t\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_next:\\\"\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[count], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_loop\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"lsl\\t%[pix], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bcs\\tL%=_last_one\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_last_zero:\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #0]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #4\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_last_delay_T0H:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_last_delay_T0H\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #4]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #10\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_last_delay_T0L:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_last_delay_T0L\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"b\\tL%=_repeat\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_last_one:\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #0]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #13\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_last_delay_T1H:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_last_delay_T1H\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"strb\\t%[bitmask], [%[reg], #4]\\\"\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"movs\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_last_delay_T1L:\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[dly], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_last_delay_T1L\\\"\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"nop\\\"\\t\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_repeat:\\\"\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"add\\t%[p], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"sub\\t%[num], #1\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"bne\\tL%=_begin\\\"\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t\\\"L%=_done:\\\"\\t\\t\\t\\t\\\"\\\\n\\\\t\\\"\\r\\n\\t: [p] \\\"+r\\\" (p),\\r\\n\\t  [pix] \\\"=&r\\\" (pix),\\r\\n\\t  [count] \\\"=&r\\\" (count),\\r\\n\\t  [dly] \\\"=&r\\\" (dly),\\r\\n\\t  [num] \\\"+r\\\" (num)\\r\\n\\t: [bitmask] \\\"r\\\" (bitmask),\\r\\n\\t  [reg] \\\"r\\\" (reg)\\r\\n  );\\r\\n#else\\r\\n#error \\\"Sorry, only 48 MHz is supported, please set Tools > CPU Speed to 48 MHz\\\"\\r\\n#endif // F_CPU == 48000000\\r\\n\\r\\n// Begin of support for NRF52832 based boards  -------------------------\\r\\n\\r\\n#elif defined(NRF52)\\r\\n// [[[Begin of the Neopixel NRF52 EasyDMA implementation\\r\\n//                                    by the Hackerspace San Salvador]]]\\r\\n// This technique uses the PWM peripheral on the NRF52. The PWM uses the\\r\\n// EasyDMA feature included on the chip. This technique loads the duty \\r\\n// cycle configuration for each cycle when the PWM is enabled. For this \\r\\n// to work we need to store a 16 bit configuration for each bit of the\\r\\n// RGB(W) values in the pixel buffer.\\r\\n// Comparator values for the PWM were hand picked and are guaranteed to\\r\\n// be 100% organic to preserve freshness and high accuracy. Current \\r\\n// parameters are:\\r\\n//   * PWM Clock: 16Mhz\\r\\n//   * Minimum step time: 62.5ns\\r\\n//   * Time for zero in high (T0H): 0.31ms\\r\\n//   * Time for one in high (T1H): 0.75ms\\r\\n//   * Cycle time:  1.25us\\r\\n//   * Frequency: 800Khz\\r\\n// For 400Khz we just double the calculated times.\\r\\n// ---------- BEGIN Constants for the EasyDMA implementation -----------\\r\\n// The PWM starts the duty cycle in LOW. To start with HIGH we\\r\\n// need to set the 15th bit on each register.\\r\\n\\r\\n// WS2812 (rev A) timing is 0.35 and 0.7us\\r\\n//#define MAGIC_T0H               5UL | (0x8000) // 0.3125us\\r\\n//#define MAGIC_T1H              12UL | (0x8000) // 0.75us\\r\\n\\r\\n// WS2812B (rev B) timing is 0.4 and 0.8 us\\r\\n#define MAGIC_T0H               6UL | (0x8000) // 0.375us\\r\\n#define MAGIC_T1H              13UL | (0x8000) // 0.8125us\\r\\n\\r\\n// WS2811 (400 khz) timing is 0.5 and 1.2\\r\\n#define MAGIC_T0H_400KHz        8UL  | (0x8000) // 0.5us\\r\\n#define MAGIC_T1H_400KHz        19UL | (0x8000) // 1.1875us\\r\\n\\r\\n// For 400Khz, we double value of CTOPVAL\\r\\n#define CTOPVAL                20UL            // 1.25us\\r\\n#define CTOPVAL_400KHz         40UL            // 2.5us\\r\\n\\r\\n// ---------- END Constants for the EasyDMA implementation -------------\\r\\n// \\r\\n// If there is no device available an alternative cycle-counter\\r\\n// implementation is tried.\\r\\n// The nRF52832 runs with a fixed clock of 64Mhz. The alternative\\r\\n// implementation is the same as the one used for the Teensy 3.0/1/2 but\\r\\n// with the Nordic SDK HAL & registers syntax.\\r\\n// The number of cycles was hand picked and is guaranteed to be 100% \\r\\n// organic to preserve freshness and high accuracy.\\r\\n// ---------- BEGIN Constants for cycle counter implementation ---------\\r\\n#define CYCLES_800_T0H  18  // ~0.36 uS\\r\\n#define CYCLES_800_T1H  41  // ~0.76 uS\\r\\n#define CYCLES_800      71  // ~1.25 uS\\r\\n\\r\\n#define CYCLES_400_T0H  26  // ~0.50 uS\\r\\n#define CYCLES_400_T1H  70  // ~1.26 uS\\r\\n#define CYCLES_400      156 // ~2.50 uS\\r\\n// ---------- END of Constants for cycle counter implementation --------\\r\\n\\r\\n  // To support both the SoftDevice + Neopixels we use the EasyDMA\\r\\n  // feature from the NRF25. However this technique implies to\\r\\n  // generate a pattern and store it on the memory. The actual\\r\\n  // memory used in bytes corresponds to the following formula:\\r\\n  //              totalMem = numBytes*8*2+(2*2)\\r\\n  // The two additional bytes at the end are needed to reset the\\r\\n  // sequence.\\r\\n  //\\r\\n  // If there is not enough memory, we will fall back to cycle counter\\r\\n  // using DWT\\r\\n  uint32_t  pattern_size   = numBytes*8*sizeof(uint16_t)+2*sizeof(uint16_t);\\r\\n  uint16_t* pixels_pattern = NULL;\\r\\n\\r\\n  NRF_PWM_Type* pwm = NULL;\\r\\n\\r\\n  // Try to find a free PWM device, which is not enabled\\r\\n  // and has no connected pins\\r\\n  NRF_PWM_Type* PWM[3] = {NRF_PWM0, NRF_PWM1, NRF_PWM2};\\r\\n  for(int device = 0; device<3; device++) {\\r\\n    if( (PWM[device]->ENABLE == 0)                            &&\\r\\n        (PWM[device]->PSEL.OUT[0] & PWM_PSEL_OUT_CONNECT_Msk) &&\\r\\n        (PWM[device]->PSEL.OUT[1] & PWM_PSEL_OUT_CONNECT_Msk) &&\\r\\n        (PWM[device]->PSEL.OUT[2] & PWM_PSEL_OUT_CONNECT_Msk) &&\\r\\n        (PWM[device]->PSEL.OUT[3] & PWM_PSEL_OUT_CONNECT_Msk)\\r\\n    ) {\\r\\n      pwm = PWM[device];\\r\\n      break;\\r\\n    }\\r\\n  }\\r\\n  \\r\\n  // only malloc if there is PWM device available\\r\\n  if ( pwm != NULL ) {\\r\\n    #ifdef ARDUINO_FEATHER52 // use thread-safe malloc\\r\\n      pixels_pattern = (uint16_t *) rtos_malloc(pattern_size);\\r\\n    #else\\r\\n      pixels_pattern = (uint16_t *) malloc(pattern_size);\\r\\n    #endif\\r\\n  }\\r\\n\\r\\n  // Use the identified device to choose the implementation\\r\\n  // If a PWM device is available use DMA\\r\\n  if( (pixels_pattern != NULL) && (pwm != NULL) ) {\\r\\n    uint16_t pos = 0; // bit position\\r\\n\\r\\n    for(uint16_t n=0; n<numBytes; n++) {\\r\\n      uint8_t pix = pixels[n];\\r\\n\\r\\n      for(uint8_t mask=0x80, i=0; mask>0; mask >>= 1, i++) {\\r\\n        #ifdef NEO_KHZ400\\r\\n        if( !is800KHz ) {\\r\\n          pixels_pattern[pos] = (pix & mask) ? MAGIC_T1H_400KHz : MAGIC_T0H_400KHz;\\r\\n        }else\\r\\n        #endif\\r\\n        {\\r\\n          pixels_pattern[pos] = (pix & mask) ? MAGIC_T1H : MAGIC_T0H;\\r\\n        }\\r\\n\\r\\n        pos++;\\r\\n      }\\r\\n    }\\r\\n\\r\\n    // Zero padding to indicate the end of que sequence\\r\\n    pixels_pattern[++pos] = 0 | (0x8000); // Seq end\\r\\n    pixels_pattern[++pos] = 0 | (0x8000); // Seq end\\r\\n\\r\\n    // Set the wave mode to count UP\\r\\n    pwm->MODE = (PWM_MODE_UPDOWN_Up << PWM_MODE_UPDOWN_Pos);\\r\\n\\r\\n    // Set the PWM to use the 16MHz clock\\r\\n    pwm->PRESCALER = (PWM_PRESCALER_PRESCALER_DIV_1 << PWM_PRESCALER_PRESCALER_Pos);\\r\\n\\r\\n    // Setting of the maximum count\\r\\n    // but keeping it on 16Mhz allows for more granularity just\\r\\n    // in case someone wants to do more fine-tuning of the timing.\\r\\n#ifdef NEO_KHZ400\\r\\n    if( !is800KHz ) {\\r\\n      pwm->COUNTERTOP = (CTOPVAL_400KHz << PWM_COUNTERTOP_COUNTERTOP_Pos);\\r\\n    }else\\r\\n#endif\\r\\n    {\\r\\n      pwm->COUNTERTOP = (CTOPVAL << PWM_COUNTERTOP_COUNTERTOP_Pos);\\r\\n    }\\r\\n\\r\\n    // Disable loops, we want the sequence to repeat only once\\r\\n    pwm->LOOP = (PWM_LOOP_CNT_Disabled << PWM_LOOP_CNT_Pos);\\r\\n\\r\\n    // On the \\\"Common\\\" setting the PWM uses the same pattern for the\\r\\n    // for supported sequences. The pattern is stored on half-word\\r\\n    // of 16bits\\r\\n    pwm->DECODER = (PWM_DECODER_LOAD_Common << PWM_DECODER_LOAD_Pos) |\\r\\n                   (PWM_DECODER_MODE_RefreshCount << PWM_DECODER_MODE_Pos);\\r\\n\\r\\n    // Pointer to the memory storing the patter\\r\\n    pwm->SEQ[0].PTR = (uint32_t)(pixels_pattern) << PWM_SEQ_PTR_PTR_Pos;\\r\\n\\r\\n    // Calculation of the number of steps loaded from memory.\\r\\n    pwm->SEQ[0].CNT = (pattern_size/sizeof(uint16_t)) << PWM_SEQ_CNT_CNT_Pos;\\r\\n\\r\\n    // The following settings are ignored with the current config.\\r\\n    pwm->SEQ[0].REFRESH  = 0;\\r\\n    pwm->SEQ[0].ENDDELAY = 0;\\r\\n\\r\\n    // The Neopixel implementation is a blocking algorithm. DMA\\r\\n    // allows for non-blocking operation. To \\\"simulate\\\" a blocking\\r\\n    // operation we enable the interruption for the end of sequence\\r\\n    // and block the execution thread until the event flag is set by\\r\\n    // the peripheral.\\r\\n//    pwm->INTEN |= (PWM_INTEN_SEQEND0_Enabled<<PWM_INTEN_SEQEND0_Pos);\\r\\n\\r\\n    // PSEL must be configured before enabling PWM\\r\\n    pwm->PSEL.OUT[0] = g_ADigitalPinMap[pin];\\r\\n\\r\\n    // Enable the PWM\\r\\n    pwm->ENABLE = 1;\\r\\n\\r\\n    // After all of this and many hours of reading the documentation\\r\\n    // we are ready to start the sequence...\\r\\n    pwm->EVENTS_SEQEND[0]  = 0;\\r\\n    pwm->TASKS_SEQSTART[0] = 1;\\r\\n\\r\\n    // But we have to wait for the flag to be set.\\r\\n    while(!pwm->EVENTS_SEQEND[0])\\r\\n    {\\r\\n      #ifdef ARDUINO_FEATHER52\\r\\n      yield();\\r\\n      #endif\\r\\n    }\\r\\n\\r\\n    // Before leave we clear the flag for the event.\\r\\n    pwm->EVENTS_SEQEND[0] = 0;\\r\\n\\r\\n    // We need to disable the device and disconnect\\r\\n    // all the outputs before leave or the device will not\\r\\n    // be selected on the next call.\\r\\n    // TODO: Check if disabling the device causes performance issues.\\r\\n    pwm->ENABLE = 0;\\r\\n\\r\\n    pwm->PSEL.OUT[0] = 0xFFFFFFFFUL;\\r\\n\\r\\n    #ifdef ARDUINO_FEATHER52  // use thread-safe free\\r\\n      rtos_free(pixels_pattern);\\r\\n    #else\\r\\n      free(pixels_pattern);\\r\\n    #endif\\r\\n  }// End of DMA implementation\\r\\n  // ---------------------------------------------------------------------\\r\\n  else{\\r\\n    // Fall back to DWT\\r\\n    #ifdef ARDUINO_FEATHER52\\r\\n      // Bluefruit Feather 52 uses freeRTOS\\r\\n      // Critical Section is used since it does not block SoftDevice execution\\r\\n      taskENTER_CRITICAL();\\r\\n    #elif defined(NRF52_DISABLE_INT)\\r\\n      // If you are using the Bluetooth SoftDevice we advise you to not disable\\r\\n      // the interrupts. Disabling the interrupts even for short periods of time\\r\\n      // causes the SoftDevice to stop working.\\r\\n      // Disable the interrupts only in cases where you need high performance for\\r\\n      // the LEDs and if you are not using the EasyDMA feature.\\r\\n      __disable_irq();\\r\\n    #endif\\r\\n\\r\\n    uint32_t pinMask = 1UL << g_ADigitalPinMap[pin];\\r\\n\\r\\n    uint32_t CYCLES_X00     = CYCLES_800;\\r\\n    uint32_t CYCLES_X00_T1H = CYCLES_800_T1H;\\r\\n    uint32_t CYCLES_X00_T0H = CYCLES_800_T0H;\\r\\n\\r\\n#ifdef NEO_KHZ400\\r\\n    if( !is800KHz )\\r\\n    {\\r\\n      CYCLES_X00     = CYCLES_400;\\r\\n      CYCLES_X00_T1H = CYCLES_400_T1H;\\r\\n      CYCLES_X00_T0H = CYCLES_400_T0H;\\r\\n    }\\r\\n#endif\\r\\n\\r\\n    // Enable DWT in debug core\\r\\n    CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;\\r\\n    DWT->CTRL |= DWT_CTRL_CYCCNTENA_Msk;\\r\\n\\r\\n    // Tries to re-send the frame if is interrupted by the SoftDevice.\\r\\n    while(1) {\\r\\n      uint8_t *p = pixels;\\r\\n\\r\\n      uint32_t cycStart = DWT->CYCCNT;\\r\\n      uint32_t cyc = 0;\\r\\n\\r\\n      for(uint16_t n=0; n<numBytes; n++) {\\r\\n        uint8_t pix = *p++;\\r\\n\\r\\n        for(uint8_t mask = 0x80; mask; mask >>= 1) {\\r\\n          while(DWT->CYCCNT - cyc < CYCLES_X00);\\r\\n          cyc  = DWT->CYCCNT;\\r\\n\\r\\n          NRF_GPIO->OUTSET |= pinMask;\\r\\n\\r\\n          if(pix & mask) {\\r\\n            while(DWT->CYCCNT - cyc < CYCLES_X00_T1H);\\r\\n          } else {\\r\\n            while(DWT->CYCCNT - cyc < CYCLES_X00_T0H);\\r\\n          }\\r\\n\\r\\n          NRF_GPIO->OUTCLR |= pinMask;\\r\\n        }\\r\\n      }\\r\\n      while(DWT->CYCCNT - cyc < CYCLES_X00);\\r\\n\\r\\n\\r\\n      // If total time longer than 25%, resend the whole data.\\r\\n      // Since we are likely to be interrupted by SoftDevice\\r\\n      if ( (DWT->CYCCNT - cycStart) < ( 8*numBytes*((CYCLES_X00*5)/4) ) ) {\\r\\n        break;\\r\\n      }\\r\\n\\r\\n      // re-send need 300us delay\\r\\n      delayMicroseconds(300);\\r\\n    }\\r\\n\\r\\n    // Enable interrupts again\\r\\n    #ifdef ARDUINO_FEATHER52\\r\\n      taskEXIT_CRITICAL();\\r\\n    #elif defined(NRF52_DISABLE_INT)\\r\\n      __enable_irq();\\r\\n    #endif\\r\\n  }\\r\\n// END of NRF52 implementation\\r\\n\\r\\n#elif defined (__SAMD21E17A__) || defined(__SAMD21G18A__)  || defined(__SAMD21E18A__) || defined(__SAMD21J18A__) // Arduino Zero, Gemma/Trinket M0, SODAQ Autonomo and others\\r\\n  // Tried this with a timer/counter, couldn't quite get adequate\\r\\n  // resolution.  So yay, you get a load of goofball NOPs...\\r\\n\\r\\n  uint8_t  *ptr, *end, p, bitMask, portNum;\\r\\n  uint32_t  pinMask;\\r\\n\\r\\n  portNum =  g_APinDescription[pin].ulPort;\\r\\n  pinMask =  1ul << g_APinDescription[pin].ulPin;\\r\\n  ptr     =  pixels;\\r\\n  end     =  ptr + numBytes;\\r\\n  p       = *ptr++;\\r\\n  bitMask =  0x80;\\r\\n\\r\\n  volatile uint32_t *set = &(PORT->Group[portNum].OUTSET.reg),\\r\\n                    *clr = &(PORT->Group[portNum].OUTCLR.reg);\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n    for(;;) {\\r\\n      *set = pinMask;\\r\\n      asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n      if(p & bitMask) {\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop;\\\");\\r\\n        *clr = pinMask;\\r\\n      } else {\\r\\n        *clr = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop;\\\");\\r\\n      }\\r\\n      if(bitMask >>= 1) {\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n      } else {\\r\\n        if(ptr >= end) break;\\r\\n        p       = *ptr++;\\r\\n        bitMask = 0x80;\\r\\n      }\\r\\n    }\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 KHz bitstream\\r\\n    for(;;) {\\r\\n      *set = pinMask;\\r\\n      asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n      if(p & bitMask) {\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop;\\\");\\r\\n        *clr = pinMask;\\r\\n      } else {\\r\\n        *clr = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop;\\\");\\r\\n      }\\r\\n      asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n          \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n          \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n          \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n      if(bitMask >>= 1) {\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n      } else {\\r\\n        if(ptr >= end) break;\\r\\n        p       = *ptr++;\\r\\n        bitMask = 0x80;\\r\\n      }\\r\\n    }\\r\\n  }\\r\\n#endif\\r\\n\\r\\n#elif defined (__SAMD51__) // M4 @ 120mhz\\r\\n  // Tried this with a timer/counter, couldn't quite get adequate\\r\\n  // resolution.  So yay, you get a load of goofball NOPs...\\r\\n\\r\\n  uint8_t  *ptr, *end, p, bitMask, portNum;\\r\\n  uint32_t  pinMask;\\r\\n\\r\\n  portNum =  g_APinDescription[pin].ulPort;\\r\\n  pinMask =  1ul << g_APinDescription[pin].ulPin;\\r\\n  ptr     =  pixels;\\r\\n  end     =  ptr + numBytes;\\r\\n  p       = *ptr++;\\r\\n  bitMask =  0x80;\\r\\n\\r\\n  volatile uint32_t *set = &(PORT->Group[portNum].OUTSET.reg),\\r\\n                    *clr = &(PORT->Group[portNum].OUTCLR.reg);\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n    for(;;) {\\r\\n      if(p & bitMask) { // ONE\\r\\n        // High 800ns\\r\\n        *set = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n        // Low 450ns\\r\\n        *clr = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop;\\\");\\r\\n      } else { // ZERO\\r\\n        // High 400ns\\r\\n        *set = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop;\\\");\\r\\n        // Low 850ns\\r\\n        *clr = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\");\\r\\n      }\\r\\n      if(bitMask >>= 1) {\\r\\n        // Move on to the next pixel\\r\\n        asm(\\\"nop;\\\");\\r\\n      } else {\\r\\n        if(ptr >= end) break;\\r\\n        p       = *ptr++;\\r\\n        bitMask = 0x80;\\r\\n      }\\r\\n    }\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 KHz bitstream\\r\\n    // ToDo!\\r\\n  }\\r\\n#endif\\r\\n\\r\\n#elif defined (ARDUINO_STM32_FEATHER) // FEATHER WICED (120MHz)\\r\\n\\r\\n  // Tried this with a timer/counter, couldn't quite get adequate\\r\\n  // resolution.  So yay, you get a load of goofball NOPs...\\r\\n\\r\\n  uint8_t  *ptr, *end, p, bitMask;\\r\\n  uint32_t  pinMask;\\r\\n\\r\\n  pinMask =  BIT(PIN_MAP[pin].gpio_bit);\\r\\n  ptr     =  pixels;\\r\\n  end     =  ptr + numBytes;\\r\\n  p       = *ptr++;\\r\\n  bitMask =  0x80;\\r\\n\\r\\n  volatile uint16_t *set = &(PIN_MAP[pin].gpio_device->regs->BSRRL);\\r\\n  volatile uint16_t *clr = &(PIN_MAP[pin].gpio_device->regs->BSRRH);\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n    for(;;) {\\r\\n      if(p & bitMask) { // ONE\\r\\n        // High 800ns\\r\\n        *set = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop;\\\");\\r\\n        // Low 450ns\\r\\n        *clr = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop;\\\");\\r\\n      } else { // ZERO\\r\\n        // High 400ns\\r\\n        *set = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop;\\\");\\r\\n        // Low 850ns\\r\\n        *clr = pinMask;\\r\\n        asm(\\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop; nop; nop; nop; nop;\\\"\\r\\n            \\\"nop; nop; nop; nop;\\\");\\r\\n      }\\r\\n      if(bitMask >>= 1) {\\r\\n        // Move on to the next pixel\\r\\n        asm(\\\"nop;\\\");\\r\\n      } else {\\r\\n        if(ptr >= end) break;\\r\\n        p       = *ptr++;\\r\\n        bitMask = 0x80;\\r\\n      }\\r\\n    }\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 KHz bitstream\\r\\n    // ToDo!\\r\\n  }\\r\\n#endif\\r\\n\\r\\n#elif defined (NRF51)\\r\\n  uint8_t          *p   = pixels,\\r\\n                    pix, count, mask;\\r\\n  int32_t         num = numBytes;\\r\\n  unsigned int bitmask = ( 1 << g_ADigitalPinMap[pin] );\\r\\n// https://github.com/sandeepmistry/arduino-nRF5/blob/dc53980c8bac27898fca90d8ecb268e11111edc1/variants/BBCmicrobit/variant.cpp\\r\\n\\r\\n  volatile unsigned int *reg = (unsigned int *) (0x50000000UL + 0x508);\\r\\n\\r\\n// https://github.com/sandeepmistry/arduino-nRF5/blob/dc53980c8bac27898fca90d8ecb268e11111edc1/cores/nRF5/SDK/components/device/nrf51.h\\r\\n// http://www.iot-programmer.com/index.php/books/27-micro-bit-iot-in-c/chapters-micro-bit-iot-in-c/47-micro-bit-iot-in-c-fast-memory-mapped-gpio?showall=1\\r\\n// https://github.com/Microsoft/pxt-neopixel/blob/master/sendbuffer.asm\\r\\n\\r\\n  asm volatile(\\r\\n    // \\\"cpsid i\\\" ; disable irq\\r\\n\\r\\n    //    b .start\\r\\n    \\\"b  L%=_start\\\"                    \\\"\\\\n\\\\t\\\"\\r\\n    // .nextbit:               ;            C0\\r\\n    \\\"L%=_nextbit:\\\"                    \\\"\\\\n\\\\t\\\"          //;            C0\\r\\n    //    str r1, [r3, #0]    ; pin := hi  C2\\r\\n    \\\"strb %[bitmask], [%[reg], #0]\\\"   \\\"\\\\n\\\\t\\\"          //; pin := hi  C2\\r\\n    //    tst r6, r0          ;            C3\\r\\n    \\\"tst %[mask], %[pix]\\\"             \\\"\\\\n\\\\t\\\"//          ;            C3\\r\\n    //    bne .islate         ;            C4\\r\\n    \\\"bne L%=_islate\\\"                  \\\"\\\\n\\\\t\\\"          //;            C4\\r\\n    //    str r1, [r2, #0]    ; pin := lo  C6\\r\\n    \\\"strb %[bitmask], [%[reg], #4]\\\"   \\\"\\\\n\\\\t\\\"          //; pin := lo  C6\\r\\n    // .islate:\\r\\n    \\\"L%=_islate:\\\"                     \\\"\\\\n\\\\t\\\"\\r\\n    //    lsrs r6, r6, #1     ; r6 >>= 1   C7\\r\\n    \\\"lsr %[mask], %[mask], #1\\\"       \\\"\\\\n\\\\t\\\"          //; r6 >>= 1   C7\\r\\n    //    bne .justbit        ;            C8\\r\\n    \\\"bne L%=_justbit\\\"                 \\\"\\\\n\\\\t\\\"          //;            C8\\r\\n\\r\\n    //    ; not just a bit - need new byte\\r\\n    //    adds r4, #1         ; r4++       C9\\r\\n    \\\"add %[p], #1\\\"                   \\\"\\\\n\\\\t\\\"          //; r4++       C9\\r\\n    //    subs r5, #1         ; r5--       C10\\r\\n    \\\"sub %[num], #1\\\"                 \\\"\\\\n\\\\t\\\"          //; r5--       C10\\r\\n    //    bcc .stop           ; if (r5<0) goto .stop  C11\\r\\n    \\\"bcc L%=_stop\\\"                    \\\"\\\\n\\\\t\\\"          //; if (r5<0) goto .stop  C11\\r\\n    // .start:\\r\\n    \\\"L%=_start:\\\"\\r\\n    //    movs r6, #0x80      ; reset mask C12\\r\\n    \\\"movs %[mask], #0x80\\\"             \\\"\\\\n\\\\t\\\"          //; reset mask C12\\r\\n    //    nop                 ;            C13\\r\\n    \\\"nop\\\"                             \\\"\\\\n\\\\t\\\"          //;            C13\\r\\n\\r\\n    // .common:               ;             C13\\r\\n    \\\"L%=_common:\\\"                     \\\"\\\\n\\\\t\\\"          //;            C13\\r\\n    //    str r1, [r2, #0]   ; pin := lo   C15\\r\\n    \\\"strb %[bitmask], [%[reg], #4]\\\"   \\\"\\\\n\\\\t\\\"          //; pin := lo  C15\\r\\n    //    ; always re-load byte - it just fits with the cycles better this way\\r\\n    //    ldrb r0, [r4, #0]  ; r0 := *r4   C17\\r\\n    \\\"ldrb  %[pix], [%[p], #0]\\\"        \\\"\\\\n\\\\t\\\"          //; r0 := *r4   C17\\r\\n    //    b .nextbit         ;             C20\\r\\n    \\\"b L%=_nextbit\\\"                   \\\"\\\\n\\\\t\\\"          //;             C20\\r\\n\\r\\n    // .justbit: ; C10\\r\\n    \\\"L%=_justbit:\\\"                    \\\"\\\\n\\\\t\\\"          //; C10\\r\\n    //    ; no nops, branch taken is already 3 cycles\\r\\n    //    b .common ; C13\\r\\n    \\\"b L%=_common\\\"                    \\\"\\\\n\\\\t\\\"          //; C13\\r\\n\\r\\n    // .stop:\\r\\n    \\\"L%=_stop:\\\"                       \\\"\\\\n\\\\t\\\"\\r\\n    //    str r1, [r2, #0]   ; pin := lo\\r\\n    \\\"strb %[bitmask], [%[reg], #4]\\\"   \\\"\\\\n\\\\t\\\"          //; pin := lo\\r\\n    //    cpsie i            ; enable irq\\r\\n\\r\\n    : [p] \\\"+r\\\" (p),\\r\\n    [pix] \\\"=&r\\\" (pix),\\r\\n    [count] \\\"=&r\\\" (count),\\r\\n    [mask] \\\"=&r\\\" (mask),\\r\\n    [num] \\\"+r\\\" (num)\\r\\n    : [bitmask] \\\"r\\\" (bitmask),\\r\\n    [reg] \\\"r\\\" (reg)\\r\\n  );\\r\\n\\r\\n#elif defined(__SAM3X8E__) // Arduino Due\\r\\n\\r\\n  #define SCALE      VARIANT_MCK / 2UL / 1000000UL\\r\\n  #define INST       (2UL * F_CPU / VARIANT_MCK)\\r\\n  #define TIME_800_0 ((int)(0.40 * SCALE + 0.5) - (5 * INST))\\r\\n  #define TIME_800_1 ((int)(0.80 * SCALE + 0.5) - (5 * INST))\\r\\n  #define PERIOD_800 ((int)(1.25 * SCALE + 0.5) - (5 * INST))\\r\\n  #define TIME_400_0 ((int)(0.50 * SCALE + 0.5) - (5 * INST))\\r\\n  #define TIME_400_1 ((int)(1.20 * SCALE + 0.5) - (5 * INST))\\r\\n  #define PERIOD_400 ((int)(2.50 * SCALE + 0.5) - (5 * INST))\\r\\n\\r\\n  int             pinMask, time0, time1, period, t;\\r\\n  Pio            *port;\\r\\n  volatile WoReg *portSet, *portClear, *timeValue, *timeReset;\\r\\n  uint8_t        *p, *end, pix, mask;\\r\\n\\r\\n  pmc_set_writeprotect(false);\\r\\n  pmc_enable_periph_clk((uint32_t)TC3_IRQn);\\r\\n  TC_Configure(TC1, 0,\\r\\n    TC_CMR_WAVE | TC_CMR_WAVSEL_UP | TC_CMR_TCCLKS_TIMER_CLOCK1);\\r\\n  TC_Start(TC1, 0);\\r\\n\\r\\n  pinMask   = g_APinDescription[pin].ulPin; // Don't 'optimize' these into\\r\\n  port      = g_APinDescription[pin].pPort; // declarations above.  Want to\\r\\n  portSet   = &(port->PIO_SODR);            // burn a few cycles after\\r\\n  portClear = &(port->PIO_CODR);            // starting timer to minimize\\r\\n  timeValue = &(TC1->TC_CHANNEL[0].TC_CV);  // the initial 'while'.\\r\\n  timeReset = &(TC1->TC_CHANNEL[0].TC_CCR);\\r\\n  p         =  pixels;\\r\\n  end       =  p + numBytes;\\r\\n  pix       = *p++;\\r\\n  mask      = 0x80;\\r\\n\\r\\n#ifdef NEO_KHZ400 // 800 KHz check needed only if 400 KHz support enabled\\r\\n  if(is800KHz) {\\r\\n#endif\\r\\n    time0  = TIME_800_0;\\r\\n    time1  = TIME_800_1;\\r\\n    period = PERIOD_800;\\r\\n#ifdef NEO_KHZ400\\r\\n  } else { // 400 KHz bitstream\\r\\n    time0  = TIME_400_0;\\r\\n    time1  = TIME_400_1;\\r\\n    period = PERIOD_400;\\r\\n  }\\r\\n#endif\\r\\n\\r\\n  for(t = time0;; t = time0) {\\r\\n    if(pix & mask) t = time1;\\r\\n    while(*timeValue < period);\\r\\n    *portSet   = pinMask;\\r\\n    *timeReset = TC_CCR_CLKEN | TC_CCR_SWTRG;\\r\\n    while(*timeValue < t);\\r\\n    *portClear = pinMask;\\r\\n    if(!(mask >>= 1)) {   // This 'inside-out' loop logic utilizes\\r\\n      if(p >= end) break; // idle time to minimize inter-byte delays.\\r\\n      pix = *p++;\\r\\n      mask = 0x80;\\r\\n    }\\r\\n  }\\r\\n  while(*timeValue < period); // Wait for last bit\\r\\n  TC_Stop(TC1, 0);\\r\\n\\r\\n#endif // end Due\\r\\n\\r\\n// END ARM ----------------------------------------------------------------\\r\\n\\r\\n\\r\\n#elif defined(ESP8266) || defined(ESP32)\\r\\n\\r\\n// ESP8266 ----------------------------------------------------------------\\r\\n\\r\\n  // ESP8266 show() is external to enforce ICACHE_RAM_ATTR execution\\r\\n  espShow(pin, pixels, numBytes, is800KHz);\\r\\n\\r\\n#elif defined(__ARDUINO_ARC__)\\r\\n\\r\\n// Arduino 101  -----------------------------------------------------------\\r\\n\\r\\n#define NOPx7 { __builtin_arc_nop(); \\\\\\r\\n  __builtin_arc_nop(); __builtin_arc_nop(); \\\\\\r\\n  __builtin_arc_nop(); __builtin_arc_nop(); \\\\\\r\\n  __builtin_arc_nop(); __builtin_arc_nop(); }\\r\\n\\r\\n  PinDescription *pindesc = &g_APinDescription[pin];\\r\\n  register uint32_t loop = 8 * numBytes; // one loop to handle all bytes and all bits\\r\\n  register uint8_t *p = pixels;\\r\\n  register uint32_t currByte = (uint32_t) (*p);\\r\\n  register uint32_t currBit = 0x80 & currByte;\\r\\n  register uint32_t bitCounter = 0;\\r\\n  register uint32_t first = 1;\\r\\n\\r\\n  // The loop is unusual. Very first iteration puts all the way LOW to the wire -\\r\\n  // constant LOW does not affect NEOPIXEL, so there is no visible effect displayed.\\r\\n  // During that very first iteration CPU caches instructions in the loop.\\r\\n  // Because of the caching process, \\\"CPU slows down\\\". NEOPIXEL pulse is very time sensitive\\r\\n  // that's why we let the CPU cache first and we start regular pulse from 2nd iteration\\r\\n  if (pindesc->ulGPIOType == SS_GPIO) {\\r\\n    register uint32_t reg = pindesc->ulGPIOBase + SS_GPIO_SWPORTA_DR;\\r\\n    uint32_t reg_val = __builtin_arc_lr((volatile uint32_t)reg);\\r\\n    register uint32_t reg_bit_high = reg_val | (1 << pindesc->ulGPIOId);\\r\\n    register uint32_t reg_bit_low  = reg_val & ~(1 << pindesc->ulGPIOId);\\r\\n\\r\\n    loop += 1; // include first, special iteration\\r\\n    while(loop--) {\\r\\n      if(!first) {\\r\\n        currByte <<= 1;\\r\\n        bitCounter++;\\r\\n      }\\r\\n\\r\\n      // 1 is >550ns high and >450ns low; 0 is 200..500ns high and >450ns low\\r\\n      __builtin_arc_sr(first ? reg_bit_low : reg_bit_high, (volatile uint32_t)reg);\\r\\n      if(currBit) { // ~400ns HIGH (740ns overall)\\r\\n        NOPx7\\r\\n        NOPx7\\r\\n      }\\r\\n      // ~340ns HIGH\\r\\n      NOPx7\\r\\n     __builtin_arc_nop();\\r\\n\\r\\n      // 820ns LOW; per spec, max allowed low here is 5000ns */\\r\\n      __builtin_arc_sr(reg_bit_low, (volatile uint32_t)reg);\\r\\n      NOPx7\\r\\n      NOPx7\\r\\n\\r\\n      if(bitCounter >= 8) {\\r\\n        bitCounter = 0;\\r\\n        currByte = (uint32_t) (*++p);\\r\\n      }\\r\\n\\r\\n      currBit = 0x80 & currByte;\\r\\n      first = 0;\\r\\n    }\\r\\n  } else if(pindesc->ulGPIOType == SOC_GPIO) {\\r\\n    register uint32_t reg = pindesc->ulGPIOBase + SOC_GPIO_SWPORTA_DR;\\r\\n    uint32_t reg_val = MMIO_REG_VAL(reg);\\r\\n    register uint32_t reg_bit_high = reg_val | (1 << pindesc->ulGPIOId);\\r\\n    register uint32_t reg_bit_low  = reg_val & ~(1 << pindesc->ulGPIOId);\\r\\n\\r\\n    loop += 1; // include first, special iteration\\r\\n    while(loop--) {\\r\\n      if(!first) {\\r\\n        currByte <<= 1;\\r\\n        bitCounter++;\\r\\n      }\\r\\n      MMIO_REG_VAL(reg) = first ? reg_bit_low : reg_bit_high;\\r\\n      if(currBit) { // ~430ns HIGH (740ns overall)\\r\\n        NOPx7\\r\\n        NOPx7\\r\\n        __builtin_arc_nop();\\r\\n      }\\r\\n      // ~310ns HIGH\\r\\n      NOPx7\\r\\n\\r\\n      // 850ns LOW; per spec, max allowed low here is 5000ns */\\r\\n      MMIO_REG_VAL(reg) = reg_bit_low;\\r\\n      NOPx7\\r\\n      NOPx7\\r\\n\\r\\n      if(bitCounter >= 8) {\\r\\n        bitCounter = 0;\\r\\n        currByte = (uint32_t) (*++p);\\r\\n      }\\r\\n\\r\\n      currBit = 0x80 & currByte;\\r\\n      first = 0;\\r\\n    }\\r\\n  }\\r\\n\\r\\n#else \\r\\n#error Architecture not supported\\r\\n#endif\\r\\n\\r\\n\\r\\n// END ARCHITECTURE SELECT ------------------------------------------------\\r\\n\\r\\n#ifndef NRF52\\r\\n  interrupts();\\r\\n#endif\\r\\n\\r\\n  endTime = micros(); // Save EOD time for latch on next call\\r\\n}\\r\\n\\r\\n// Set the output pin number\\r\\nvoid Adafruit_NeoPixel::setPin(uint8_t p) {\\r\\n  if(begun && (pin >= 0)) pinMode(pin, INPUT);\\r\\n    pin = p;\\r\\n    if(begun) {\\r\\n      pinMode(p, OUTPUT);\\r\\n      digitalWrite(p, LOW);\\r\\n    }\\r\\n#ifdef __AVR__\\r\\n    port    = portOutputRegister(digitalPinToPort(p));\\r\\n    pinMask = digitalPinToBitMask(p);\\r\\n#endif\\r\\n}\\r\\n\\r\\n// Set pixel color from separate R,G,B components:\\r\\nvoid Adafruit_NeoPixel::setPixelColor(\\r\\n uint16_t n, uint8_t r, uint8_t g, uint8_t b) {\\r\\n\\r\\n  if(n < numLEDs) {\\r\\n    if(brightness) { // See notes in setBrightness()\\r\\n      r = (r * brightness) >> 8;\\r\\n      g = (g * brightness) >> 8;\\r\\n      b = (b * brightness) >> 8;\\r\\n    }\\r\\n    uint8_t *p;\\r\\n    if(wOffset == rOffset) { // Is an RGB-type strip\\r\\n      p = &pixels[n * 3];    // 3 bytes per pixel\\r\\n    } else {                 // Is a WRGB-type strip\\r\\n      p = &pixels[n * 4];    // 4 bytes per pixel\\r\\n      p[wOffset] = 0;        // But only R,G,B passed -- set W to 0\\r\\n    }\\r\\n    p[rOffset] = r;          // R,G,B always stored\\r\\n    p[gOffset] = g;\\r\\n    p[bOffset] = b;\\r\\n  }\\r\\n}\\r\\n\\r\\nvoid Adafruit_NeoPixel::setPixelColor(\\r\\n uint16_t n, uint8_t r, uint8_t g, uint8_t b, uint8_t w) {\\r\\n\\r\\n  if(n < numLEDs) {\\r\\n    if(brightness) { // See notes in setBrightness()\\r\\n      r = (r * brightness) >> 8;\\r\\n      g = (g * brightness) >> 8;\\r\\n      b = (b * brightness) >> 8;\\r\\n      w = (w * brightness) >> 8;\\r\\n    }\\r\\n    uint8_t *p;\\r\\n    if(wOffset == rOffset) { // Is an RGB-type strip\\r\\n      p = &pixels[n * 3];    // 3 bytes per pixel (ignore W)\\r\\n    } else {                 // Is a WRGB-type strip\\r\\n      p = &pixels[n * 4];    // 4 bytes per pixel\\r\\n      p[wOffset] = w;        // Store W\\r\\n    }\\r\\n    p[rOffset] = r;          // Store R,G,B\\r\\n    p[gOffset] = g;\\r\\n    p[bOffset] = b;\\r\\n  }\\r\\n}\\r\\n\\r\\n// Set pixel color from 'packed' 32-bit RGB color:\\r\\nvoid Adafruit_NeoPixel::setPixelColor(uint16_t n, uint32_t c) {\\r\\n  if(n < numLEDs) {\\r\\n    uint8_t *p,\\r\\n      r = (uint8_t)(c >> 16),\\r\\n      g = (uint8_t)(c >>  8),\\r\\n      b = (uint8_t)c;\\r\\n    if(brightness) { // See notes in setBrightness()\\r\\n      r = (r * brightness) >> 8;\\r\\n      g = (g * brightness) >> 8;\\r\\n      b = (b * brightness) >> 8;\\r\\n    }\\r\\n    if(wOffset == rOffset) {\\r\\n      p = &pixels[n * 3];\\r\\n    } else {\\r\\n      p = &pixels[n * 4];\\r\\n      uint8_t w = (uint8_t)(c >> 24);\\r\\n      p[wOffset] = brightness ? ((w * brightness) >> 8) : w;\\r\\n    }\\r\\n    p[rOffset] = r;\\r\\n    p[gOffset] = g;\\r\\n    p[bOffset] = b;\\r\\n  }\\r\\n}\\r\\n\\r\\n// Convert separate R,G,B into packed 32-bit RGB color.\\r\\n// Packed format is always RGB, regardless of LED strand color order.\\r\\nuint32_t Adafruit_NeoPixel::Color(uint8_t r, uint8_t g, uint8_t b) {\\r\\n  return ((uint32_t)r << 16) | ((uint32_t)g <<  8) | b;\\r\\n}\\r\\n\\r\\n// Convert separate R,G,B,W into packed 32-bit WRGB color.\\r\\n// Packed format is always WRGB, regardless of LED strand color order.\\r\\nuint32_t Adafruit_NeoPixel::Color(uint8_t r, uint8_t g, uint8_t b, uint8_t w) {\\r\\n  return ((uint32_t)w << 24) | ((uint32_t)r << 16) | ((uint32_t)g <<  8) | b;\\r\\n}\\r\\n\\r\\n// Query color from previously-set pixel (returns packed 32-bit RGB value)\\r\\nuint32_t Adafruit_NeoPixel::getPixelColor(uint16_t n) const {\\r\\n  if(n >= numLEDs) return 0; // Out of bounds, return no color.\\r\\n\\r\\n  uint8_t *p;\\r\\n\\r\\n  if(wOffset == rOffset) { // Is RGB-type device\\r\\n    p = &pixels[n * 3];\\r\\n    if(brightness) {\\r\\n      // Stored color was decimated by setBrightness().  Returned value\\r\\n      // attempts to scale back to an approximation of the original 24-bit\\r\\n      // value used when setting the pixel color, but there will always be\\r\\n      // some error -- those bits are simply gone.  Issue is most\\r\\n      // pronounced at low brightness levels.\\r\\n      return (((uint32_t)(p[rOffset] << 8) / brightness) << 16) |\\r\\n             (((uint32_t)(p[gOffset] << 8) / brightness) <<  8) |\\r\\n             ( (uint32_t)(p[bOffset] << 8) / brightness       );\\r\\n    } else {\\r\\n      // No brightness adjustment has been made -- return 'raw' color\\r\\n      return ((uint32_t)p[rOffset] << 16) |\\r\\n             ((uint32_t)p[gOffset] <<  8) |\\r\\n              (uint32_t)p[bOffset];\\r\\n    }\\r\\n  } else {                 // Is RGBW-type device\\r\\n    p = &pixels[n * 4];\\r\\n    if(brightness) { // Return scaled color\\r\\n      return (((uint32_t)(p[wOffset] << 8) / brightness) << 24) |\\r\\n             (((uint32_t)(p[rOffset] << 8) / brightness) << 16) |\\r\\n             (((uint32_t)(p[gOffset] << 8) / brightness) <<  8) |\\r\\n             ( (uint32_t)(p[bOffset] << 8) / brightness       );\\r\\n    } else { // Return raw color\\r\\n      return ((uint32_t)p[wOffset] << 24) |\\r\\n             ((uint32_t)p[rOffset] << 16) |\\r\\n             ((uint32_t)p[gOffset] <<  8) |\\r\\n              (uint32_t)p[bOffset];\\r\\n    }\\r\\n  }\\r\\n}\\r\\n\\r\\n// Returns pointer to pixels[] array.  Pixel data is stored in device-\\r\\n// native format and is not translated here.  Application will need to be\\r\\n// aware of specific pixel data format and handle colors appropriately.\\r\\nuint8_t *Adafruit_NeoPixel::getPixels(void) const {\\r\\n  return pixels;\\r\\n}\\r\\n\\r\\nuint16_t Adafruit_NeoPixel::numPixels(void) const {\\r\\n  return numLEDs;\\r\\n}\\r\\n\\r\\n// Adjust output brightness; 0=darkest (off), 255=brightest.  This does\\r\\n// NOT immediately affect what's currently displayed on the LEDs.  The\\r\\n// next call to show() will refresh the LEDs at this level.  However,\\r\\n// this process is potentially \\\"lossy,\\\" especially when increasing\\r\\n// brightness.  The tight timing in the WS2811/WS2812 code means there\\r\\n// aren't enough free cycles to perform this scaling on the fly as data\\r\\n// is issued.  So we make a pass through the existing color data in RAM\\r\\n// and scale it (subsequent graphics commands also work at this\\r\\n// brightness level).  If there's a significant step up in brightness,\\r\\n// the limited number of steps (quantization) in the old data will be\\r\\n// quite visible in the re-scaled version.  For a non-destructive\\r\\n// change, you'll need to re-render the full strip data.  C'est la vie.\\r\\nvoid Adafruit_NeoPixel::setBrightness(uint8_t b) {\\r\\n  // Stored brightness value is different than what's passed.\\r\\n  // This simplifies the actual scaling math later, allowing a fast\\r\\n  // 8x8-bit multiply and taking the MSB.  'brightness' is a uint8_t,\\r\\n  // adding 1 here may (intentionally) roll over...so 0 = max brightness\\r\\n  // (color values are interpreted literally; no scaling), 1 = min\\r\\n  // brightness (off), 255 = just below max brightness.\\r\\n  uint8_t newBrightness = b + 1;\\r\\n  if(newBrightness != brightness) { // Compare against prior value\\r\\n    // Brightness has changed -- re-scale existing data in RAM\\r\\n    uint8_t  c,\\r\\n            *ptr           = pixels,\\r\\n             oldBrightness = brightness - 1; // De-wrap old brightness value\\r\\n    uint16_t scale;\\r\\n    if(oldBrightness == 0) scale = 0; // Avoid /0\\r\\n    else if(b == 255) scale = 65535 / oldBrightness;\\r\\n    else scale = (((uint16_t)newBrightness << 8) - 1) / oldBrightness;\\r\\n    for(uint16_t i=0; i<numBytes; i++) {\\r\\n      c      = *ptr;\\r\\n      *ptr++ = (c * scale) >> 8;\\r\\n    }\\r\\n    brightness = newBrightness;\\r\\n  }\\r\\n}\\r\\n\\r\\n//Return the brightness value\\r\\nuint8_t Adafruit_NeoPixel::getBrightness(void) const {\\r\\n  return brightness - 1;\\r\\n}\\r\\n\\r\\nvoid Adafruit_NeoPixel::clear() {\\r\\n  memset(pixels, 0, numBytes);\\r\\n}\\r\\n\\r\\n/* A PROGMEM (flash mem) table containing 8-bit unsigned sine wave (0-255).\\r\\n   Copy & paste this snippet into a Python REPL to regenerate:\\r\\nimport math\\r\\nfor x in range(256):\\r\\n    print(\\\"{:3},\\\".format(int((math.sin(x/128.0*math.pi)+1.0)*127.5+0.5))),\\r\\n    if x&15 == 15: print\\r\\n*/\\r\\nstatic const uint8_t PROGMEM _sineTable[256] = {\\r\\n  128,131,134,137,140,143,146,149,152,155,158,162,165,167,170,173,\\r\\n  176,179,182,185,188,190,193,196,198,201,203,206,208,211,213,215,\\r\\n  218,220,222,224,226,228,230,232,234,235,237,238,240,241,243,244,\\r\\n  245,246,248,249,250,250,251,252,253,253,254,254,254,255,255,255,\\r\\n  255,255,255,255,254,254,254,253,253,252,251,250,250,249,248,246,\\r\\n  245,244,243,241,240,238,237,235,234,232,230,228,226,224,222,220,\\r\\n  218,215,213,211,208,206,203,201,198,196,193,190,188,185,182,179,\\r\\n  176,173,170,167,165,162,158,155,152,149,146,143,140,137,134,131,\\r\\n  128,124,121,118,115,112,109,106,103,100, 97, 93, 90, 88, 85, 82,\\r\\n   79, 76, 73, 70, 67, 65, 62, 59, 57, 54, 52, 49, 47, 44, 42, 40,\\r\\n   37, 35, 33, 31, 29, 27, 25, 23, 21, 20, 18, 17, 15, 14, 12, 11,\\r\\n   10,  9,  7,  6,  5,  5,  4,  3,  2,  2,  1,  1,  1,  0,  0,  0,\\r\\n    0,  0,  0,  0,  1,  1,  1,  2,  2,  3,  4,  5,  5,  6,  7,  9,\\r\\n   10, 11, 12, 14, 15, 17, 18, 20, 21, 23, 25, 27, 29, 31, 33, 35,\\r\\n   37, 40, 42, 44, 47, 49, 52, 54, 57, 59, 62, 65, 67, 70, 73, 76,\\r\\n   79, 82, 85, 88, 90, 93, 97,100,103,106,109,112,115,118,121,124};\\r\\n\\r\\n/* Similar to above, but for an 8-bit gamma-correction table.\\r\\n   Copy & paste this snippet into a Python REPL to regenerate:\\r\\nimport math\\r\\ngamma=2.6\\r\\nfor x in range(256):\\r\\n    print(\\\"{:3},\\\".format(int(math.pow((x)/255.0,gamma)*255.0+0.5))),\\r\\n    if x&15 == 15: print\\r\\n*/\\r\\nstatic const uint8_t PROGMEM _gammaTable[256] = {\\r\\n    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,\\r\\n    0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  1,  1,  1,\\r\\n    1,  1,  1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  3,  3,  3,  3,\\r\\n    3,  3,  4,  4,  4,  4,  5,  5,  5,  5,  5,  6,  6,  6,  6,  7,\\r\\n    7,  7,  8,  8,  8,  9,  9,  9, 10, 10, 10, 11, 11, 11, 12, 12,\\r\\n   13, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17, 18, 18, 19, 19, 20,\\r\\n   20, 21, 21, 22, 22, 23, 24, 24, 25, 25, 26, 27, 27, 28, 29, 29,\\r\\n   30, 31, 31, 32, 33, 34, 34, 35, 36, 37, 38, 38, 39, 40, 41, 42,\\r\\n   42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57,\\r\\n   58, 59, 60, 61, 62, 63, 64, 65, 66, 68, 69, 70, 71, 72, 73, 75,\\r\\n   76, 77, 78, 80, 81, 82, 84, 85, 86, 88, 89, 90, 92, 93, 94, 96,\\r\\n   97, 99,100,102,103,105,106,108,109,111,112,114,115,117,119,120,\\r\\n  122,124,125,127,129,130,132,134,136,137,139,141,143,145,146,148,\\r\\n  150,152,154,156,158,160,162,164,166,168,170,172,174,176,178,180,\\r\\n  182,184,186,188,191,193,195,197,199,202,204,206,209,211,213,215,\\r\\n  218,220,223,225,227,230,232,235,237,240,242,245,247,250,252,255};\\r\\n\\r\\nuint8_t Adafruit_NeoPixel::sine8(uint8_t x) const {\\r\\n  return pgm_read_byte(&_sineTable[x]); // 0-255 in, 0-255 out\\r\\n}\\r\\n\\r\\nuint8_t Adafruit_NeoPixel::gamma8(uint8_t x) const {\\r\\n  return pgm_read_byte(&_gammaTable[x]); // 0-255 in, 0-255 out\\r\\n}\\r\\n\" }",
    "{ filename: \"src/Adafruit_NeoPixel.h\", code: \"/*--------------------------------------------------------------------\\r\\n  This file is part of the Adafruit NeoPixel library.\\r\\n\\r\\n  NeoPixel is free software: you can redistribute it and/or modify\\r\\n  it under the terms of the GNU Lesser General Public License as\\r\\n  published by the Free Software Foundation, either version 3 of\\r\\n  the License, or (at your option) any later version.\\r\\n\\r\\n  NeoPixel is distributed in the hope that it will be useful,\\r\\n  but WITHOUT ANY WARRANTY; without even the implied warranty of\\r\\n  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\\r\\n  GNU Lesser General Public License for more details.\\r\\n\\r\\n  You should have received a copy of the GNU Lesser General Public\\r\\n  License along with NeoPixel.  If not, see\\r\\n  <http://www.gnu.org/licenses/>.\\r\\n  --------------------------------------------------------------------*/\\r\\n\\r\\n#ifndef ADAFRUIT_NEOPIXEL_H\\r\\n#define ADAFRUIT_NEOPIXEL_H\\r\\n\\r\\n#if (ARDUINO >= 100)\\r\\n #include <Arduino.h>\\r\\n#else\\r\\n #include <WProgram.h>\\r\\n #include <pins_arduino.h>\\r\\n#endif\\r\\n\\r\\n// The order of primary colors in the NeoPixel data stream can vary\\r\\n// among device types, manufacturers and even different revisions of\\r\\n// the same item.  The third parameter to the Adafruit_NeoPixel\\r\\n// constructor encodes the per-pixel byte offsets of the red, green\\r\\n// and blue primaries (plus white, if present) in the data stream --\\r\\n// the following #defines provide an easier-to-use named version for\\r\\n// each permutation.  e.g. NEO_GRB indicates a NeoPixel-compatible\\r\\n// device expecting three bytes per pixel, with the first byte\\r\\n// containing the green value, second containing red and third\\r\\n// containing blue.  The in-memory representation of a chain of\\r\\n// NeoPixels is the same as the data-stream order; no re-ordering of\\r\\n// bytes is required when issuing data to the chain.\\r\\n\\r\\n// Bits 5,4 of this value are the offset (0-3) from the first byte of\\r\\n// a pixel to the location of the red color byte.  Bits 3,2 are the\\r\\n// green offset and 1,0 are the blue offset.  If it is an RGBW-type\\r\\n// device (supporting a white primary in addition to R,G,B), bits 7,6\\r\\n// are the offset to the white byte...otherwise, bits 7,6 are set to\\r\\n// the same value as 5,4 (red) to indicate an RGB (not RGBW) device.\\r\\n// i.e. binary representation:\\r\\n// 0bWWRRGGBB for RGBW devices\\r\\n// 0bRRRRGGBB for RGB\\r\\n\\r\\n// RGB NeoPixel permutations; white and red offsets are always same\\r\\n// Offset:         W          R          G          B\\r\\n#define NEO_RGB  ((0 << 6) | (0 << 4) | (1 << 2) | (2))\\r\\n#define NEO_RBG  ((0 << 6) | (0 << 4) | (2 << 2) | (1))\\r\\n#define NEO_GRB  ((1 << 6) | (1 << 4) | (0 << 2) | (2))\\r\\n#define NEO_GBR  ((2 << 6) | (2 << 4) | (0 << 2) | (1))\\r\\n#define NEO_BRG  ((1 << 6) | (1 << 4) | (2 << 2) | (0))\\r\\n#define NEO_BGR  ((2 << 6) | (2 << 4) | (1 << 2) | (0))\\r\\n\\r\\n// RGBW NeoPixel permutations; all 4 offsets are distinct\\r\\n// Offset:         W          R          G          B\\r\\n#define NEO_WRGB ((0 << 6) | (1 << 4) | (2 << 2) | (3))\\r\\n#define NEO_WRBG ((0 << 6) | (1 << 4) | (3 << 2) | (2))\\r\\n#define NEO_WGRB ((0 << 6) | (2 << 4) | (1 << 2) | (3))\\r\\n#define NEO_WGBR ((0 << 6) | (3 << 4) | (1 << 2) | (2))\\r\\n#define NEO_WBRG ((0 << 6) | (2 << 4) | (3 << 2) | (1))\\r\\n#define NEO_WBGR ((0 << 6) | (3 << 4) | (2 << 2) | (1))\\r\\n\\r\\n#define NEO_RWGB ((1 << 6) | (0 << 4) | (2 << 2) | (3))\\r\\n#define NEO_RWBG ((1 << 6) | (0 << 4) | (3 << 2) | (2))\\r\\n#define NEO_RGWB ((2 << 6) | (0 << 4) | (1 << 2) | (3))\\r\\n#define NEO_RGBW ((3 << 6) | (0 << 4) | (1 << 2) | (2))\\r\\n#define NEO_RBWG ((2 << 6) | (0 << 4) | (3 << 2) | (1))\\r\\n#define NEO_RBGW ((3 << 6) | (0 << 4) | (2 << 2) | (1))\\r\\n\\r\\n#define NEO_GWRB ((1 << 6) | (2 << 4) | (0 << 2) | (3))\\r\\n#define NEO_GWBR ((1 << 6) | (3 << 4) | (0 << 2) | (2))\\r\\n#define NEO_GRWB ((2 << 6) | (1 << 4) | (0 << 2) | (3))\\r\\n#define NEO_GRBW ((3 << 6) | (1 << 4) | (0 << 2) | (2))\\r\\n#define NEO_GBWR ((2 << 6) | (3 << 4) | (0 << 2) | (1))\\r\\n#define NEO_GBRW ((3 << 6) | (2 << 4) | (0 << 2) | (1))\\r\\n\\r\\n#define NEO_BWRG ((1 << 6) | (2 << 4) | (3 << 2) | (0))\\r\\n#define NEO_BWGR ((1 << 6) | (3 << 4) | (2 << 2) | (0))\\r\\n#define NEO_BRWG ((2 << 6) | (1 << 4) | (3 << 2) | (0))\\r\\n#define NEO_BRGW ((3 << 6) | (1 << 4) | (2 << 2) | (0))\\r\\n#define NEO_BGWR ((2 << 6) | (3 << 4) | (1 << 2) | (0))\\r\\n#define NEO_BGRW ((3 << 6) | (2 << 4) | (1 << 2) | (0))\\r\\n\\r\\n// Add NEO_KHZ400 to the color order value to indicate a 400 KHz\\r\\n// device.  All but the earliest v1 NeoPixels expect an 800 KHz data\\r\\n// stream, this is the default if unspecified.  Because flash space\\r\\n// is very limited on ATtiny devices (e.g. Trinket, Gemma), v1\\r\\n// NeoPixels aren't handled by default on those chips, though it can\\r\\n// be enabled by removing the ifndef/endif below -- but code will be\\r\\n// bigger.  Conversely, can disable the NEO_KHZ400 line on other MCUs\\r\\n// to remove v1 support and save a little space.\\r\\n\\r\\n#define NEO_KHZ800 0x0000 // 800 KHz datastream\\r\\n#ifndef __AVR_ATtiny85__\\r\\n#define NEO_KHZ400 0x0100 // 400 KHz datastream\\r\\n#endif\\r\\n\\r\\n// If 400 KHz support is enabled, the third parameter to the constructor\\r\\n// requires a 16-bit value (in order to select 400 vs 800 KHz speed).\\r\\n// If only 800 KHz is enabled (as is default on ATtiny), an 8-bit value\\r\\n// is sufficient to encode pixel color order, saving some space.\\r\\n\\r\\n#ifdef NEO_KHZ400\\r\\ntypedef uint16_t neoPixelType;\\r\\n#else\\r\\ntypedef uint8_t  neoPixelType;\\r\\n#endif\\r\\n\\r\\nclass Adafruit_NeoPixel {\\r\\n\\r\\n public:\\r\\n\\r\\n  // Constructor: number of LEDs, pin number, LED type\\r\\n  Adafruit_NeoPixel(uint16_t n, uint8_t p=6, neoPixelType t=NEO_GRB + NEO_KHZ800);\\r\\n  Adafruit_NeoPixel(void);\\r\\n  ~Adafruit_NeoPixel();\\r\\n\\r\\n  void\\r\\n    begin(void),\\r\\n    show(void),\\r\\n    setPin(uint8_t p),\\r\\n    setPixelColor(uint16_t n, uint8_t r, uint8_t g, uint8_t b),\\r\\n    setPixelColor(uint16_t n, uint8_t r, uint8_t g, uint8_t b, uint8_t w),\\r\\n    setPixelColor(uint16_t n, uint32_t c),\\r\\n    setBrightness(uint8_t),\\r\\n    clear(),\\r\\n    updateLength(uint16_t n),\\r\\n    updateType(neoPixelType t);\\r\\n  uint8_t\\r\\n   *getPixels(void) const,\\r\\n    getBrightness(void) const,\\r\\n    sine8(uint8_t) const,\\r\\n    gamma8(uint8_t) const;\\r\\n  int8_t\\r\\n    getPin(void) { return pin; };\\r\\n  uint16_t\\r\\n    numPixels(void) const;\\r\\n  static uint32_t\\r\\n    Color(uint8_t r, uint8_t g, uint8_t b),\\r\\n    Color(uint8_t r, uint8_t g, uint8_t b, uint8_t w);\\r\\n  uint32_t\\r\\n    getPixelColor(uint16_t n) const;\\r\\n  inline bool\\r\\n    canShow(void) { return (micros() - endTime) >= 300L; }\\r\\n\\r\\n protected:\\r\\n\\r\\n  boolean\\r\\n#ifdef NEO_KHZ400  // If 400 KHz NeoPixel support enabled...\\r\\n    is800KHz,      // ...true if 800 KHz pixels\\r\\n#endif\\r\\n    begun;         // true if begin() previously called\\r\\n  uint16_t\\r\\n    numLEDs,       // Number of RGB LEDs in strip\\r\\n    numBytes;      // Size of 'pixels' buffer below (3 or 4 bytes/pixel)\\r\\n  int8_t\\r\\n    pin;           // Output pin number (-1 if not yet set)\\r\\n  uint8_t\\r\\n    brightness,\\r\\n   *pixels,        // Holds LED color values (3 or 4 bytes each)\\r\\n    rOffset,       // Index of red byte within each 3- or 4-byte pixel\\r\\n    gOffset,       // Index of green byte\\r\\n    bOffset,       // Index of blue byte\\r\\n    wOffset;       // Index of white byte (same as rOffset if no white)\\r\\n  uint32_t\\r\\n    endTime;       // Latch timing reference\\r\\n#ifdef __AVR__\\r\\n  volatile uint8_t\\r\\n    *port;         // Output PORT register\\r\\n  uint8_t\\r\\n    pinMask;       // Output PORT bitmask\\r\\n#endif\\r\\n};\\r\\n\\r\\n#endif // ADAFRUIT_NEOPIXEL_H\\r\\n\" }",
    "{ filename: \"src/arbot_dht11.cpp\", code: \"/*####################################################################\\r\\n FILE: dht11.cpp\\r\\n VERSION: 0.4.5\\r\\n PURPOSE: DHT11 Temperature & Humidity Sensor library for Arduino\\r\\n LICENSE: GPL v3 (http://www.gnu.org/licenses/gpl.html)\\r\\n\\r\\n DATASHEET: http://feed.virtuabotix.com/wp-content/uploads/2011/11/DHT11.pdf\\r\\n GET UPDATES: http://feed.virtuabotix.com/?239\\r\\n\\r\\n     --##--##--##--##--##--##--##--##--##--##--  \\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ## DHT11 SENSOR ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##FRONT ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     --##--##--##--##--##--##--##--##--##--##--\\r\\n         ||       ||          || (Not    || \\r\\n         ||       ||          || Used)   || \\r\\n       VDD(5V)   Readout(I/O)          Ground\\r\\n                 \\r\\n HISTORY:\\r\\n Mod by Joseph Dattilo (Virtuabotix LLC) - Version 0.4.5 (11/11/11)\\r\\n Mod by Joseph Dattilo (Virtuabotix LLC) - Version 0.4.0 (06/11/11)\\r\\n Mod by Rob Tillaart - Version 0.3 (28/03/2011)\\r\\n Mod by SimKard - Version 0.2 (24/11/2010)\\r\\n George Hadjikyriacou - Original version (??)\\r\\n#######################################################################*/\\r\\n\\r\\n#include \\\"arbot_dht11.h\\\"\\r\\n\\r\\n// returnvalues:\\r\\n//  0 : OK\\r\\n// -1 : checksum error\\r\\n// -2 : timeout\\r\\n\\r\\nvoid dht11::attach(int pin)\\r\\n{\\r\\n\\tsensorPin = pin; //and there was much rejoicing! Pin attached.\\r\\n}\\r\\n\\r\\nint dht11::read()\\r\\n{\\r\\n\\tif(sensorPin<0){attach(2);}//not a pin change to default pin 2\\r\\n\\treturn read(sensorPin);//reads from attached sensorPin (attach command must be run first or this will use the default pin 2)\\r\\n}\\r\\n\\r\\nint dht11::read(int pin)\\r\\n{\\r\\n\\t// BUFFER TO RECEIVE\\r\\n\\tuint8_t bits[5];\\r\\n\\tuint8_t cnt = 7;\\r\\n\\tuint8_t idx = 0;\\r\\n\\r\\n\\t// EMPTY BUFFER\\r\\n\\tfor (int i=0; i< 5; i++) bits[i] = 0;\\r\\n\\r\\n\\t// REQUEST SAMPLE\\r\\n\\tpinMode(pin, OUTPUT);\\r\\n\\tdigitalWrite(pin, LOW);\\r\\n\\tdelay(18);\\r\\n\\tdigitalWrite(pin, HIGH);\\r\\n\\tdelayMicroseconds(40);\\r\\n\\tpinMode(pin, INPUT);\\r\\n\\r\\n\\t// ACKNOWLEDGE or TIMEOUT\\r\\n\\tunsigned int loopCnt = 10000;\\r\\n\\twhile(digitalRead(pin) == LOW)\\r\\n\\t\\tif (loopCnt-- == 0) return -2;\\r\\n\\r\\n\\tloopCnt = 10000;\\r\\n\\twhile(digitalRead(pin) == HIGH)\\r\\n\\t\\tif (loopCnt-- == 0) return -2;\\r\\n\\r\\n\\t// READ OUTPUT - 40 BITS => 5 BYTES or TIMEOUT\\r\\n\\tfor (int i=0; i<40; i++)\\r\\n\\t{\\r\\n\\t\\tloopCnt = 10000;\\r\\n\\t\\twhile(digitalRead(pin) == LOW)\\r\\n\\t\\t\\tif (loopCnt-- == 0) return -2;\\r\\n\\r\\n\\t\\tunsigned long t = micros();\\r\\n\\r\\n\\t\\tloopCnt = 10000;\\r\\n\\t\\twhile(digitalRead(pin) == HIGH)\\r\\n\\t\\t\\tif (loopCnt-- == 0) return -2;\\r\\n\\r\\n\\t\\tif ((micros() - t) > 40) bits[idx] |= (1 << cnt);\\r\\n\\t\\tif (cnt == 0)   // next byte?\\r\\n\\t\\t{\\r\\n\\t\\t\\tcnt = 7;    // restart at MSB\\r\\n\\t\\t\\tidx++;      // next byte!\\r\\n\\t\\t}\\r\\n\\t\\telse cnt--;\\r\\n\\t}\\r\\n\\r\\n\\t// WRITE TO RIGHT VARS\\r\\n    // as bits[1] and bits[3] are allways zero they are omitted in formulas.\\r\\n\\thumidity    = bits[0]; \\r\\n\\ttemperature = bits[2]; \\r\\n\\r\\n\\tuint8_t sum = bits[0] + bits[2];  \\r\\n\\r\\n\\tif (bits[4] != sum) return -1;\\r\\n\\treturn 0;\\r\\n}\\r\\n\\r\\nint dht11::celcius()\\r\\n{\\r\\n\\tread();//make sure the temp has been read\\r\\n\\r\\n\\treturn temperature;\\r\\n}\\r\\n\\r\\n\\r\\n//Celsius to Fahrenheit conversion\\r\\ndouble dht11::fahrenheit(double dCelsius)\\r\\n{\\r\\n\\treturn 1.8 * dCelsius + 32;\\r\\n}\\r\\n\\r\\ndouble dht11::fahrenheit()\\r\\n{\\r\\n\\tread();//make sure the temp has been read\\r\\n\\r\\n\\treturn fahrenheit(temperature);\\r\\n}\\r\\n\\r\\n//Celsius to Kelvin conversion\\r\\ndouble dht11::kelvin(double dCelsius)\\r\\n{\\r\\n\\treturn dCelsius + 273.15;\\r\\n}\\r\\n\\r\\ndouble dht11::kelvin()\\r\\n{\\r\\n\\tread();\\r\\n\\treturn kelvin(temperature);\\r\\n}\\r\\n\\r\\n\\r\\n// dewPoint function NOAA\\r\\n// reference: http://wahiduddin.net/calc/density_algorithms.htm \\r\\ndouble dht11::dewPoint()\\r\\n{\\r\\n\\tread();//make sure the temp has been read\\r\\n\\r\\n\\tdouble A0= 373.15/(273.15 + temperature);\\r\\n\\tdouble SUM = -7.90298 * (A0-1);\\r\\n\\tSUM += 5.02808 * log10(A0);\\r\\n\\tSUM += -1.3816e-7 * (pow(10, (11.344*(1-1/A0)))-1) ;\\r\\n\\tSUM += 8.1328e-3 * (pow(10,(-3.49149*(A0-1)))-1) ;\\r\\n\\tSUM += log10(1013.246);\\r\\n\\tdouble VP = pow(10, SUM-3) * humidity;\\r\\n\\tdouble T = log(VP/0.61078);   // temp var\\r\\n\\treturn (241.88 * T) / (17.558-T);\\r\\n}\\r\\n\\r\\n// delta max = 0.6544 wrt dewPoint()\\r\\n// 5x faster than dewPoint()\\r\\n// reference: http://en.wikipedia.org/wiki/Dew_point\\r\\ndouble dht11::dewPointFast()\\r\\n{\\r\\n\\tread();//make sure the temp has been read\\r\\n\\r\\n\\tdouble a = 17.271;\\r\\n\\tdouble b = 237.7;\\r\\n\\tdouble temp = (a * temperature) / (b + temperature) + log((double) humidity/100);\\r\\n\\tdouble Td = (b * temp) / (a - temp);\\r\\n\\treturn Td;\\r\\n}\\r\\n\" }",
    "{ filename: \"src/arbot_dht11.h\", code: \"/*####################################################################\\r\\n FILE: dht11.cpp\\r\\n VERSION: 0.4.5\\r\\n PURPOSE: DHT11 Temperature & Humidity Sensor library for Arduino\\r\\n LICENSE: GPL v3 (http://www.gnu.org/licenses/gpl.html)\\r\\n\\r\\n DATASHEET: http://feed.virtuabotix.com/wp-content/uploads/2011/11/DHT11.pdf\\r\\n GET UPDATES: http://feed.virtuabotix.com/?239\\r\\n\\r\\n     --##--##--##--##--##--##--##--##--##--##--  \\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ## DHT11 SENSOR ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##FRONT ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     | ##  ##  ##  ##  ##  ##  ##  ##  ##  ## |\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##\\r\\n     --##--##--##--##--##--##--##--##--##--##--\\r\\n         ||       ||          || (Not    || \\r\\n         ||       ||          || Used)   || \\r\\n       VDD(5V)   Readout(I/O)          Ground\\r\\n                 \\r\\n HISTORY:\\r\\n Mod by Joseph Dattilo (Virtuabotix LLC) - Version 0.4.5 (11/11/11)\\r\\n Mod by Joseph Dattilo (Virtuabotix LLC) - Version 0.4.0 (06/11/11)\\r\\n Mod by Rob Tillaart - Version 0.3 (28/03/2011)\\r\\n Mod by SimKard - Version 0.2 (24/11/2010)\\r\\n George Hadjikyriacou - Original version (??)\\r\\n#######################################################################*/\\r\\n\\r\\n#ifndef dht11_h\\r\\n#define dht11_h\\r\\n\\r\\n#include \\\"Arduino.h\\\"\\r\\n\\r\\n#define DHT11LIB_VERSION \\\"0.4.5\\\"\\r\\n\\r\\nclass dht11\\r\\n{\\r\\nprivate:\\r\\n\\tint sensorPin;//defaults to pin 2\\r\\npublic:\\r\\n    void attach(int pin);//Attaches dht11 object to a particular sensorPin\\r\\n\\tint read();//defaults to the attached pin\\r\\n    int read(int pin);\\r\\n\\tint humidity;\\r\\n\\tint temperature;\\r\\n\\tint celcius();\\r\\n\\tdouble fahrenheit();\\r\\n\\tdouble fahrenheit(double dCelcius);\\r\\n\\tdouble kelvin();\\r\\n\\tdouble kelvin(double dCelcius);\\r\\n\\tdouble dewPoint();\\r\\n\\tdouble dewPointFast();\\r\\n};\\r\\n#endif\\r\\n\" }",
    "{ filename: \"src/arbot_LiquidCrystal_I2C.cpp\", code: \"// Based on the work by DFRobot\\r\\n\\r\\n#include \\\"arbot_LiquidCrystal_I2C.h\\\"\\r\\n#include <inttypes.h>\\r\\n#if defined(ARDUINO) && ARDUINO >= 100\\r\\n\\r\\n#include \\\"Arduino.h\\\"\\r\\n\\r\\n#define printIIC(args)\\tWire.write(args)\\r\\ninline size_t arbot_LiquidCrystal_I2C::write(uint8_t value) {\\r\\n\\tsend(value, Rs);\\r\\n\\treturn 1;\\r\\n}\\r\\n\\r\\n#else\\r\\n#include \\\"WProgram.h\\\"\\r\\n\\r\\n#define printIIC(args)\\tWire.send(args)\\r\\ninline void arbot_LiquidCrystal_I2C::write(uint8_t value) {\\r\\n\\tsend(value, Rs);\\r\\n}\\r\\n\\r\\n#endif\\r\\n#include \\\"Wire.h\\\"\\r\\n\\r\\n\\r\\n\\r\\n// When the display powers up, it is configured as follows:\\r\\n//\\r\\n// 1. Display clear\\r\\n// 2. Function set: \\r\\n//    DL = 1; 8-bit interface data \\r\\n//    N = 0; 1-line display \\r\\n//    F = 0; 5x8 dot character font \\r\\n// 3. Display on/off control: \\r\\n//    D = 0; Display off \\r\\n//    C = 0; Cursor off \\r\\n//    B = 0; Blinking off \\r\\n// 4. Entry mode set: \\r\\n//    I/D = 1; Increment by 1\\r\\n//    S = 0; No shift \\r\\n//\\r\\n// Note, however, that resetting the Arduino doesn't reset the LCD, so we\\r\\n// can't assume that its in that state when a sketch starts (and the\\r\\n// LiquidCrystal constructor is called).\\r\\n\\r\\narbot_LiquidCrystal_I2C::arbot_LiquidCrystal_I2C(uint8_t lcd_Addr,uint8_t lcd_cols,uint8_t lcd_rows)\\r\\n{\\r\\n  _Addr = lcd_Addr;\\r\\n  _cols = lcd_cols;\\r\\n  _rows = lcd_rows;\\r\\n  _backlightval = LCD_NOBACKLIGHT;\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::init(){\\r\\n\\tinit_priv();\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::init_priv()\\r\\n{\\r\\n\\tWire.begin();\\r\\n\\t_displayfunction = LCD_4BITMODE | LCD_1LINE | LCD_5x8DOTS;\\r\\n\\tbegin(_cols, _rows);  \\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::begin(uint8_t cols, uint8_t lines, uint8_t dotsize) {\\r\\n\\tif (lines > 1) {\\r\\n\\t\\t_displayfunction |= LCD_2LINE;\\r\\n\\t}\\r\\n\\t_numlines = lines;\\r\\n\\r\\n\\t// for some 1 line displays you can select a 10 pixel high font\\r\\n\\tif ((dotsize != 0) && (lines == 1)) {\\r\\n\\t\\t_displayfunction |= LCD_5x10DOTS;\\r\\n\\t}\\r\\n\\r\\n\\t// SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION!\\r\\n\\t// according to datasheet, we need at least 40ms after power rises above 2.7V\\r\\n\\t// before sending commands. Arduino can turn on way befer 4.5V so we'll wait 50\\r\\n\\tdelay(50); \\r\\n  \\r\\n\\t// Now we pull both RS and R/W low to begin commands\\r\\n\\texpanderWrite(_backlightval);\\t// reset expanderand turn backlight off (Bit 8 =1)\\r\\n\\tdelay(1000);\\r\\n\\r\\n  \\t//put the LCD into 4 bit mode\\r\\n\\t// this is according to the hitachi HD44780 datasheet\\r\\n\\t// figure 24, pg 46\\r\\n\\t\\r\\n\\t  // we start in 8bit mode, try to set 4 bit mode\\r\\n   write4bits(0x03 << 4);\\r\\n   delayMicroseconds(4500); // wait min 4.1ms\\r\\n   \\r\\n   // second try\\r\\n   write4bits(0x03 << 4);\\r\\n   delayMicroseconds(4500); // wait min 4.1ms\\r\\n   \\r\\n   // third go!\\r\\n   write4bits(0x03 << 4); \\r\\n   delayMicroseconds(150);\\r\\n   \\r\\n   // finally, set to 4-bit interface\\r\\n   write4bits(0x02 << 4); \\r\\n\\r\\n\\r\\n\\t// set # lines, font size, etc.\\r\\n\\tcommand(LCD_FUNCTIONSET | _displayfunction);  \\r\\n\\t\\r\\n\\t// turn the display on with no cursor or blinking default\\r\\n\\t_displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;\\r\\n\\tdisplay();\\r\\n\\t\\r\\n\\t// clear it off\\r\\n\\tclear();\\r\\n\\t\\r\\n\\t// Initialize to default text direction (for roman languages)\\r\\n\\t_displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;\\r\\n\\t\\r\\n\\t// set the entry mode\\r\\n\\tcommand(LCD_ENTRYMODESET | _displaymode);\\r\\n\\t\\r\\n\\thome();\\r\\n  \\r\\n}\\r\\n\\r\\n/********** high level commands, for the user! */\\r\\nvoid arbot_LiquidCrystal_I2C::clear(){\\r\\n\\tcommand(LCD_CLEARDISPLAY);// clear display, set cursor position to zero\\r\\n\\tdelayMicroseconds(2000);  // this command takes a long time!\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::home(){\\r\\n\\tcommand(LCD_RETURNHOME);  // set cursor position to zero\\r\\n\\tdelayMicroseconds(2000);  // this command takes a long time!\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::setCursor(uint8_t col, uint8_t row){\\r\\n\\tint row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };\\r\\n\\tif ( row > _numlines ) {\\r\\n\\t\\trow = _numlines-1;    // we count rows starting w/0\\r\\n\\t}\\r\\n\\tcommand(LCD_SETDDRAMADDR | (col + row_offsets[row]));\\r\\n}\\r\\n\\r\\n// Turn the display on/off (quickly)\\r\\nvoid arbot_LiquidCrystal_I2C::noDisplay() {\\r\\n\\t_displaycontrol &= ~LCD_DISPLAYON;\\r\\n\\tcommand(LCD_DISPLAYCONTROL | _displaycontrol);\\r\\n}\\r\\nvoid arbot_LiquidCrystal_I2C::display() {\\r\\n\\t_displaycontrol |= LCD_DISPLAYON;\\r\\n\\tcommand(LCD_DISPLAYCONTROL | _displaycontrol);\\r\\n}\\r\\n\\r\\n// Turns the underline cursor on/off\\r\\nvoid arbot_LiquidCrystal_I2C::noCursor() {\\r\\n\\t_displaycontrol &= ~LCD_CURSORON;\\r\\n\\tcommand(LCD_DISPLAYCONTROL | _displaycontrol);\\r\\n}\\r\\nvoid arbot_LiquidCrystal_I2C::cursor() {\\r\\n\\t_displaycontrol |= LCD_CURSORON;\\r\\n\\tcommand(LCD_DISPLAYCONTROL | _displaycontrol);\\r\\n}\\r\\n\\r\\n// Turn on and off the blinking cursor\\r\\nvoid arbot_LiquidCrystal_I2C::noBlink() {\\r\\n\\t_displaycontrol &= ~LCD_BLINKON;\\r\\n\\tcommand(LCD_DISPLAYCONTROL | _displaycontrol);\\r\\n}\\r\\nvoid arbot_LiquidCrystal_I2C::blink() {\\r\\n\\t_displaycontrol |= LCD_BLINKON;\\r\\n\\tcommand(LCD_DISPLAYCONTROL | _displaycontrol);\\r\\n}\\r\\n\\r\\n// These commands scroll the display without changing the RAM\\r\\nvoid arbot_LiquidCrystal_I2C::scrollDisplayLeft(void) {\\r\\n\\tcommand(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);\\r\\n}\\r\\nvoid arbot_LiquidCrystal_I2C::scrollDisplayRight(void) {\\r\\n\\tcommand(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);\\r\\n}\\r\\n\\r\\n// This is for text that flows Left to Right\\r\\nvoid arbot_LiquidCrystal_I2C::leftToRight(void) {\\r\\n\\t_displaymode |= LCD_ENTRYLEFT;\\r\\n\\tcommand(LCD_ENTRYMODESET | _displaymode);\\r\\n}\\r\\n\\r\\n// This is for text that flows Right to Left\\r\\nvoid arbot_LiquidCrystal_I2C::rightToLeft(void) {\\r\\n\\t_displaymode &= ~LCD_ENTRYLEFT;\\r\\n\\tcommand(LCD_ENTRYMODESET | _displaymode);\\r\\n}\\r\\n\\r\\n// This will 'right justify' text from the cursor\\r\\nvoid arbot_LiquidCrystal_I2C::autoscroll(void) {\\r\\n\\t_displaymode |= LCD_ENTRYSHIFTINCREMENT;\\r\\n\\tcommand(LCD_ENTRYMODESET | _displaymode);\\r\\n}\\r\\n\\r\\n// This will 'left justify' text from the cursor\\r\\nvoid arbot_LiquidCrystal_I2C::noAutoscroll(void) {\\r\\n\\t_displaymode &= ~LCD_ENTRYSHIFTINCREMENT;\\r\\n\\tcommand(LCD_ENTRYMODESET | _displaymode);\\r\\n}\\r\\n\\r\\n// Allows us to fill the first 8 CGRAM locations\\r\\n// with custom characters\\r\\nvoid arbot_LiquidCrystal_I2C::createChar(uint8_t location, uint8_t charmap[]) {\\r\\n\\tlocation &= 0x7; // we only have 8 locations 0-7\\r\\n\\tcommand(LCD_SETCGRAMADDR | (location << 3));\\r\\n\\tfor (int i=0; i<8; i++) {\\r\\n\\t\\twrite(charmap[i]);\\r\\n\\t}\\r\\n}\\r\\n\\r\\n// Turn the (optional) backlight off/on\\r\\nvoid arbot_LiquidCrystal_I2C::noBacklight(void) {\\r\\n\\t_backlightval=LCD_NOBACKLIGHT;\\r\\n\\texpanderWrite(0);\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::backlight(void) {\\r\\n\\t_backlightval=LCD_BACKLIGHT;\\r\\n\\texpanderWrite(0);\\r\\n}\\r\\n\\r\\n\\r\\n\\r\\n/*********** mid level commands, for sending data/cmds */\\r\\n\\r\\ninline void arbot_LiquidCrystal_I2C::command(uint8_t value) {\\r\\n\\tsend(value, 0);\\r\\n}\\r\\n\\r\\n\\r\\n/************ low level data pushing commands **********/\\r\\n\\r\\n// write either command or data\\r\\nvoid arbot_LiquidCrystal_I2C::send(uint8_t value, uint8_t mode) {\\r\\n\\tuint8_t highnib=value&0xf0;\\r\\n\\tuint8_t lownib=(value<<4)&0xf0;\\r\\n       write4bits((highnib)|mode);\\r\\n\\twrite4bits((lownib)|mode); \\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::write4bits(uint8_t value) {\\r\\n\\texpanderWrite(value);\\r\\n\\tpulseEnable(value);\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::expanderWrite(uint8_t _data){                                        \\r\\n\\tWire.beginTransmission(_Addr);\\r\\n\\tprintIIC((int)(_data) | _backlightval);\\r\\n\\tWire.endTransmission();   \\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::pulseEnable(uint8_t _data){\\r\\n\\texpanderWrite(_data | En);\\t// En high\\r\\n\\tdelayMicroseconds(1);\\t\\t// enable pulse must be >450ns\\r\\n\\t\\r\\n\\texpanderWrite(_data & ~En);\\t// En low\\r\\n\\tdelayMicroseconds(50);\\t\\t// commands need > 37us to settle\\r\\n} \\r\\n\\r\\n\\r\\n// Alias functions\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::cursor_on(){\\r\\n\\tcursor();\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::cursor_off(){\\r\\n\\tnoCursor();\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::blink_on(){\\r\\n\\tblink();\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::blink_off(){\\r\\n\\tnoBlink();\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::load_custom_character(uint8_t char_num, uint8_t *rows){\\r\\n\\t\\tcreateChar(char_num, rows);\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::setBacklight(uint8_t new_val){\\r\\n\\tif(new_val){\\r\\n\\t\\tbacklight();\\t\\t// turn backlight on\\r\\n\\t}else{\\r\\n\\t\\tnoBacklight();\\t\\t// turn backlight off\\r\\n\\t}\\r\\n}\\r\\n\\r\\nvoid arbot_LiquidCrystal_I2C::printstr(const char c[]){\\r\\n\\t//This function is not identical to the function used for \\\"real\\\" I2C displays\\r\\n\\t//it's here so the user sketch doesn't have to be changed \\r\\n\\tprint(c);\\r\\n}\\r\\n\\r\\n\\r\\n// unsupported API functions\\r\\nvoid arbot_LiquidCrystal_I2C::off(){}\\r\\nvoid arbot_LiquidCrystal_I2C::on(){}\\r\\nvoid arbot_LiquidCrystal_I2C::setDelay (int cmdDelay,int charDelay) {}\\r\\nuint8_t arbot_LiquidCrystal_I2C::status(){return 0;}\\r\\nuint8_t arbot_LiquidCrystal_I2C::keypad (){return 0;}\\r\\nuint8_t arbot_LiquidCrystal_I2C::init_bargraph(uint8_t graphtype){return 0;}\\r\\nvoid arbot_LiquidCrystal_I2C::draw_horizontal_graph(uint8_t row, uint8_t column, uint8_t len,  uint8_t pixel_col_end){}\\r\\nvoid arbot_LiquidCrystal_I2C::draw_vertical_graph(uint8_t row, uint8_t column, uint8_t len,  uint8_t pixel_row_end){}\\r\\nvoid arbot_LiquidCrystal_I2C::setContrast(uint8_t new_val){}\\r\\n\\r\\n\\t\\r\\n\" }",
    "{ filename: \"src/arbot_LiquidCrystal_I2C.h\", code: \"//YWROBOT\\r\\n#ifndef arbot_LiquidCrystal_I2C_h\\r\\n#define arbot_LiquidCrystal_I2C_h\\r\\n\\r\\n#include <inttypes.h>\\r\\n#include \\\"Print.h\\\" \\r\\n#include <Wire.h>\\r\\n\\r\\n// commands\\r\\n#define LCD_CLEARDISPLAY 0x01\\r\\n#define LCD_RETURNHOME 0x02\\r\\n#define LCD_ENTRYMODESET 0x04\\r\\n#define LCD_DISPLAYCONTROL 0x08\\r\\n#define LCD_CURSORSHIFT 0x10\\r\\n#define LCD_FUNCTIONSET 0x20\\r\\n#define LCD_SETCGRAMADDR 0x40\\r\\n#define LCD_SETDDRAMADDR 0x80\\r\\n\\r\\n// flags for display entry mode\\r\\n#define LCD_ENTRYRIGHT 0x00\\r\\n#define LCD_ENTRYLEFT 0x02\\r\\n#define LCD_ENTRYSHIFTINCREMENT 0x01\\r\\n#define LCD_ENTRYSHIFTDECREMENT 0x00\\r\\n\\r\\n// flags for display on/off control\\r\\n#define LCD_DISPLAYON 0x04\\r\\n#define LCD_DISPLAYOFF 0x00\\r\\n#define LCD_CURSORON 0x02\\r\\n#define LCD_CURSOROFF 0x00\\r\\n#define LCD_BLINKON 0x01\\r\\n#define LCD_BLINKOFF 0x00\\r\\n\\r\\n// flags for display/cursor shift\\r\\n#define LCD_DISPLAYMOVE 0x08\\r\\n#define LCD_CURSORMOVE 0x00\\r\\n#define LCD_MOVERIGHT 0x04\\r\\n#define LCD_MOVELEFT 0x00\\r\\n\\r\\n// flags for function set\\r\\n#define LCD_8BITMODE 0x10\\r\\n#define LCD_4BITMODE 0x00\\r\\n#define LCD_2LINE 0x08\\r\\n#define LCD_1LINE 0x00\\r\\n#define LCD_5x10DOTS 0x04\\r\\n#define LCD_5x8DOTS 0x00\\r\\n\\r\\n// flags for backlight control\\r\\n#define LCD_BACKLIGHT 0x08\\r\\n#define LCD_NOBACKLIGHT 0x00\\r\\n\\r\\n#define En B00000100  // Enable bit\\r\\n#define Rw B00000010  // Read/Write bit\\r\\n#define Rs B00000001  // Register select bit\\r\\n\\r\\nclass arbot_LiquidCrystal_I2C : public Print {\\r\\npublic:\\r\\n  arbot_LiquidCrystal_I2C(uint8_t lcd_Addr,uint8_t lcd_cols,uint8_t lcd_rows);\\r\\n  void begin(uint8_t cols, uint8_t rows, uint8_t charsize = LCD_5x8DOTS );\\r\\n  void clear();\\r\\n  void home();\\r\\n  void noDisplay();\\r\\n  void display();\\r\\n  void noBlink();\\r\\n  void blink();\\r\\n  void noCursor();\\r\\n  void cursor();\\r\\n  void scrollDisplayLeft();\\r\\n  void scrollDisplayRight();\\r\\n  void printLeft();\\r\\n  void printRight();\\r\\n  void leftToRight();\\r\\n  void rightToLeft();\\r\\n  void shiftIncrement();\\r\\n  void shiftDecrement();\\r\\n  void noBacklight();\\r\\n  void backlight();\\r\\n  void autoscroll();\\r\\n  void noAutoscroll(); \\r\\n  void createChar(uint8_t, uint8_t[]);\\r\\n  void setCursor(uint8_t, uint8_t); \\r\\n#if defined(ARDUINO) && ARDUINO >= 100\\r\\n  virtual size_t write(uint8_t);\\r\\n#else\\r\\n  virtual void write(uint8_t);\\r\\n#endif\\r\\n  void command(uint8_t);\\r\\n  void init();\\r\\n\\r\\n////compatibility API function aliases\\r\\nvoid blink_on();\\t\\t\\t\\t\\t\\t// alias for blink()\\r\\nvoid blink_off();       \\t\\t\\t\\t\\t// alias for noBlink()\\r\\nvoid cursor_on();      \\t \\t\\t\\t\\t\\t// alias for cursor()\\r\\nvoid cursor_off();      \\t\\t\\t\\t\\t// alias for noCursor()\\r\\nvoid setBacklight(uint8_t new_val);\\t\\t\\t\\t// alias for backlight() and nobacklight()\\r\\nvoid load_custom_character(uint8_t char_num, uint8_t *rows);\\t// alias for createChar()\\r\\nvoid printstr(const char[]);\\r\\n\\r\\n////Unsupported API functions (not implemented in this library)\\r\\nuint8_t status();\\r\\nvoid setContrast(uint8_t new_val);\\r\\nuint8_t keypad();\\r\\nvoid setDelay(int,int);\\r\\nvoid on();\\r\\nvoid off();\\r\\nuint8_t init_bargraph(uint8_t graphtype);\\r\\nvoid draw_horizontal_graph(uint8_t row, uint8_t column, uint8_t len,  uint8_t pixel_col_end);\\r\\nvoid draw_vertical_graph(uint8_t row, uint8_t column, uint8_t len,  uint8_t pixel_col_end);\\r\\n\\t \\r\\n\\r\\nprivate:\\r\\n  void init_priv();\\r\\n  void send(uint8_t, uint8_t);\\r\\n  void write4bits(uint8_t);\\r\\n  void expanderWrite(uint8_t);\\r\\n  void pulseEnable(uint8_t);\\r\\n  uint8_t _Addr;\\r\\n  uint8_t _displayfunction;\\r\\n  uint8_t _displaycontrol;\\r\\n  uint8_t _displaymode;\\r\\n  uint8_t _numlines;\\r\\n  uint8_t _cols;\\r\\n  uint8_t _rows;\\r\\n  uint8_t _backlightval;\\r\\n};\\r\\n\\r\\n#endif\\r\\n\" }",
    "{ filename: \"src/oz.h\", code: \"#ifndef arbot_oz\\r\\n\\r\\nint arbotPort[6][2] ={{ 2, 3 }, { 4, 5 }, {  6, 7 }, { A0, A1 },{ A2, A3 },{ A6, A7 }};\\r\\nint arbotMotor[2][3]={{11,12,13},{10,8,9}};\\r\\nint motorAnahtar=0;\\r\\n\\r\\n#endif\\r\\n\" }",
    "{ filename: \"src/Servo.cpp\", code: \"/*\\r\\n Servo.cpp - Interrupt driven Servo library for Arduino using 16 bit timers- Version 2\\r\\n Copyright (c) 2009 Michael Margolis.  All right reserved.\\r\\n\\r\\n This library is free software; you can redistribute it and/or\\r\\n modify it under the terms of the GNU Lesser General Public\\r\\n License as published by the Free Software Foundation; either\\r\\n version 2.1 of the License, or (at your option) any later version.\\r\\n\\r\\n This library is distributed in the hope that it will be useful,\\r\\n but WITHOUT ANY WARRANTY; without even the implied warranty of\\r\\n MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\\r\\n Lesser General Public License for more details.\\r\\n\\r\\n You should have received a copy of the GNU Lesser General Public\\r\\n License along with this library; if not, write to the Free Software\\r\\n Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA\\r\\n */\\r\\n\\r\\n#if defined(ARDUINO_ARCH_AVR)\\r\\n\\r\\n#include <avr/interrupt.h>\\r\\n#include <Arduino.h>\\r\\n\\r\\n#include \\\"Servo.h\\\"\\r\\n\\r\\n#define usToTicks(_us)    (( clockCyclesPerMicrosecond()* _us) / 8)     // converts microseconds to tick (assumes prescale of 8)  // 12 Aug 2009\\r\\n#define ticksToUs(_ticks) (( (unsigned)_ticks * 8)/ clockCyclesPerMicrosecond() ) // converts from ticks back to microseconds\\r\\n\\r\\n\\r\\n#define TRIM_DURATION       2                               // compensation ticks to trim adjust for digitalWrite delays // 12 August 2009\\r\\n\\r\\n//#define NBR_TIMERS        (MAX_SERVOS / SERVOS_PER_TIMER)\\r\\n\\r\\nstatic servo_t servos[MAX_SERVOS];                          // static array of servo structures\\r\\nstatic volatile int8_t Channel[_Nbr_16timers ];             // counter for the servo being pulsed for each timer (or -1 if refresh interval)\\r\\n\\r\\nuint8_t ServoCount = 0;                                     // the total number of attached servos\\r\\n\\r\\n\\r\\n// convenience macros\\r\\n#define SERVO_INDEX_TO_TIMER(_servo_nbr) ((timer16_Sequence_t)(_servo_nbr / SERVOS_PER_TIMER)) // returns the timer controlling this servo\\r\\n#define SERVO_INDEX_TO_CHANNEL(_servo_nbr) (_servo_nbr % SERVOS_PER_TIMER)       // returns the index of the servo on this timer\\r\\n#define SERVO_INDEX(_timer,_channel)  ((_timer*SERVOS_PER_TIMER) + _channel)     // macro to access servo index by timer and channel\\r\\n#define SERVO(_timer,_channel)  (servos[SERVO_INDEX(_timer,_channel)])            // macro to access servo class by timer and channel\\r\\n\\r\\n#define SERVO_MIN() (MIN_PULSE_WIDTH - this->min * 4)  // minimum value in uS for this servo\\r\\n#define SERVO_MAX() (MAX_PULSE_WIDTH - this->max * 4)  // maximum value in uS for this servo\\r\\n\\r\\n/************ static functions common to all instances ***********************/\\r\\n\\r\\nstatic inline void handle_interrupts(timer16_Sequence_t timer, volatile uint16_t *TCNTn, volatile uint16_t* OCRnA)\\r\\n{\\r\\n  if( Channel[timer] < 0 )\\r\\n    *TCNTn = 0; // channel set to -1 indicated that refresh interval completed so reset the timer\\r\\n  else{\\r\\n    if( SERVO_INDEX(timer,Channel[timer]) < ServoCount && SERVO(timer,Channel[timer]).Pin.isActive == true )\\r\\n      digitalWrite( SERVO(timer,Channel[timer]).Pin.nbr,LOW); // pulse this channel low if activated\\r\\n  }\\r\\n\\r\\n  Channel[timer]++;    // increment to the next channel\\r\\n  if( SERVO_INDEX(timer,Channel[timer]) < ServoCount && Channel[timer] < SERVOS_PER_TIMER) {\\r\\n    *OCRnA = *TCNTn + SERVO(timer,Channel[timer]).ticks;\\r\\n    if(SERVO(timer,Channel[timer]).Pin.isActive == true)     // check if activated\\r\\n      digitalWrite( SERVO(timer,Channel[timer]).Pin.nbr,HIGH); // its an active channel so pulse it high\\r\\n  }\\r\\n  else {\\r\\n    // finished all channels so wait for the refresh period to expire before starting over\\r\\n    if( ((unsigned)*TCNTn) + 4 < usToTicks(REFRESH_INTERVAL) )  // allow a few ticks to ensure the next OCR1A not missed\\r\\n      *OCRnA = (unsigned int)usToTicks(REFRESH_INTERVAL);\\r\\n    else\\r\\n      *OCRnA = *TCNTn + 4;  // at least REFRESH_INTERVAL has elapsed\\r\\n    Channel[timer] = -1; // this will get incremented at the end of the refresh period to start again at the first channel\\r\\n  }\\r\\n}\\r\\n\\r\\n#ifndef WIRING // Wiring pre-defines signal handlers so don't define any if compiling for the Wiring platform\\r\\n// Interrupt handlers for Arduino\\r\\n#if defined(_useTimer1)\\r\\nSIGNAL (TIMER1_COMPA_vect)\\r\\n{\\r\\n  handle_interrupts(_timer1, &TCNT1, &OCR1A);\\r\\n}\\r\\n#endif\\r\\n\\r\\n#if defined(_useTimer3)\\r\\nSIGNAL (TIMER3_COMPA_vect)\\r\\n{\\r\\n  handle_interrupts(_timer3, &TCNT3, &OCR3A);\\r\\n}\\r\\n#endif\\r\\n\\r\\n#if defined(_useTimer4)\\r\\nSIGNAL (TIMER4_COMPA_vect)\\r\\n{\\r\\n  handle_interrupts(_timer4, &TCNT4, &OCR4A);\\r\\n}\\r\\n#endif\\r\\n\\r\\n#if defined(_useTimer5)\\r\\nSIGNAL (TIMER5_COMPA_vect)\\r\\n{\\r\\n  handle_interrupts(_timer5, &TCNT5, &OCR5A);\\r\\n}\\r\\n#endif\\r\\n\\r\\n#elif defined WIRING\\r\\n// Interrupt handlers for Wiring\\r\\n#if defined(_useTimer1)\\r\\nvoid Timer1Service()\\r\\n{\\r\\n  handle_interrupts(_timer1, &TCNT1, &OCR1A);\\r\\n}\\r\\n#endif\\r\\n#if defined(_useTimer3)\\r\\nvoid Timer3Service()\\r\\n{\\r\\n  handle_interrupts(_timer3, &TCNT3, &OCR3A);\\r\\n}\\r\\n#endif\\r\\n#endif\\r\\n\\r\\n\\r\\nstatic void initISR(timer16_Sequence_t timer)\\r\\n{\\r\\n#if defined (_useTimer1)\\r\\n  if(timer == _timer1) {\\r\\n    TCCR1A = 0;             // normal counting mode\\r\\n    TCCR1B = _BV(CS11);     // set prescaler of 8\\r\\n    TCNT1 = 0;              // clear the timer count\\r\\n#if defined(__AVR_ATmega8__)|| defined(__AVR_ATmega128__)\\r\\n    TIFR |= _BV(OCF1A);      // clear any pending interrupts;\\r\\n    TIMSK |=  _BV(OCIE1A) ;  // enable the output compare interrupt\\r\\n#else\\r\\n    // here if not ATmega8 or ATmega128\\r\\n    TIFR1 |= _BV(OCF1A);     // clear any pending interrupts;\\r\\n    TIMSK1 |=  _BV(OCIE1A) ; // enable the output compare interrupt\\r\\n#endif\\r\\n#if defined(WIRING)\\r\\n    timerAttach(TIMER1OUTCOMPAREA_INT, Timer1Service);\\r\\n#endif\\r\\n  }\\r\\n#endif\\r\\n\\r\\n#if defined (_useTimer3)\\r\\n  if(timer == _timer3) {\\r\\n    TCCR3A = 0;             // normal counting mode\\r\\n    TCCR3B = _BV(CS31);     // set prescaler of 8\\r\\n    TCNT3 = 0;              // clear the timer count\\r\\n#if defined(__AVR_ATmega128__)\\r\\n    TIFR |= _BV(OCF3A);     // clear any pending interrupts;\\r\\n\\tETIMSK |= _BV(OCIE3A);  // enable the output compare interrupt\\r\\n#else\\r\\n    TIFR3 = _BV(OCF3A);     // clear any pending interrupts;\\r\\n    TIMSK3 =  _BV(OCIE3A) ; // enable the output compare interrupt\\r\\n#endif\\r\\n#if defined(WIRING)\\r\\n    timerAttach(TIMER3OUTCOMPAREA_INT, Timer3Service);  // for Wiring platform only\\r\\n#endif\\r\\n  }\\r\\n#endif\\r\\n\\r\\n#if defined (_useTimer4)\\r\\n  if(timer == _timer4) {\\r\\n    TCCR4A = 0;             // normal counting mode\\r\\n    TCCR4B = _BV(CS41);     // set prescaler of 8\\r\\n    TCNT4 = 0;              // clear the timer count\\r\\n    TIFR4 = _BV(OCF4A);     // clear any pending interrupts;\\r\\n    TIMSK4 =  _BV(OCIE4A) ; // enable the output compare interrupt\\r\\n  }\\r\\n#endif\\r\\n\\r\\n#if defined (_useTimer5)\\r\\n  if(timer == _timer5) {\\r\\n    TCCR5A = 0;             // normal counting mode\\r\\n    TCCR5B = _BV(CS51);     // set prescaler of 8\\r\\n    TCNT5 = 0;              // clear the timer count\\r\\n    TIFR5 = _BV(OCF5A);     // clear any pending interrupts;\\r\\n    TIMSK5 =  _BV(OCIE5A) ; // enable the output compare interrupt\\r\\n  }\\r\\n#endif\\r\\n}\\r\\n\\r\\nstatic void finISR(timer16_Sequence_t timer)\\r\\n{\\r\\n    //disable use of the given timer\\r\\n#if defined WIRING   // Wiring\\r\\n  if(timer == _timer1) {\\r\\n    #if defined(__AVR_ATmega1281__)||defined(__AVR_ATmega2561__)\\r\\n    TIMSK1 &=  ~_BV(OCIE1A) ;  // disable timer 1 output compare interrupt\\r\\n    #else\\r\\n    TIMSK &=  ~_BV(OCIE1A) ;  // disable timer 1 output compare interrupt\\r\\n    #endif\\r\\n    timerDetach(TIMER1OUTCOMPAREA_INT);\\r\\n  }\\r\\n  else if(timer == _timer3) {\\r\\n    #if defined(__AVR_ATmega1281__)||defined(__AVR_ATmega2561__)\\r\\n    TIMSK3 &= ~_BV(OCIE3A);    // disable the timer3 output compare A interrupt\\r\\n    #else\\r\\n    ETIMSK &= ~_BV(OCIE3A);    // disable the timer3 output compare A interrupt\\r\\n    #endif\\r\\n    timerDetach(TIMER3OUTCOMPAREA_INT);\\r\\n  }\\r\\n#else\\r\\n    //For arduino - in future: call here to a currently undefined function to reset the timer\\r\\n#endif\\r\\n}\\r\\n\\r\\nstatic boolean isTimerActive(timer16_Sequence_t timer)\\r\\n{\\r\\n  // returns true if any servo is active on this timer\\r\\n  for(uint8_t channel=0; channel < SERVOS_PER_TIMER; channel++) {\\r\\n    if(SERVO(timer,channel).Pin.isActive == true)\\r\\n      return true;\\r\\n  }\\r\\n  return false;\\r\\n}\\r\\n\\r\\n\\r\\n/****************** end of static functions ******************************/\\r\\n\\r\\nServo::Servo()\\r\\n{\\r\\n  if( ServoCount < MAX_SERVOS) {\\r\\n    this->servoIndex = ServoCount++;                    // assign a servo index to this instance\\r\\n\\tservos[this->servoIndex].ticks = usToTicks(DEFAULT_PULSE_WIDTH);   // store default values  - 12 Aug 2009\\r\\n  }\\r\\n  else\\r\\n    this->servoIndex = INVALID_SERVO ;  // too many servos\\r\\n}\\r\\n\\r\\nuint8_t Servo::attach(int pin)\\r\\n{\\r\\n  return this->attach(pin, MIN_PULSE_WIDTH, MAX_PULSE_WIDTH);\\r\\n}\\r\\n\\r\\nuint8_t Servo::attach(int pin, int min, int max)\\r\\n{\\r\\n  if(this->servoIndex < MAX_SERVOS ) {\\r\\n    pinMode( pin, OUTPUT) ;                                   // set servo pin to output\\r\\n    servos[this->servoIndex].Pin.nbr = pin;\\r\\n    // todo min/max check: abs(min - MIN_PULSE_WIDTH) /4 < 128\\r\\n    this->min  = (MIN_PULSE_WIDTH - min)/4; //resolution of min/max is 4 uS\\r\\n    this->max  = (MAX_PULSE_WIDTH - max)/4;\\r\\n    // initialize the timer if it has not already been initialized\\r\\n    timer16_Sequence_t timer = SERVO_INDEX_TO_TIMER(servoIndex);\\r\\n    if(isTimerActive(timer) == false)\\r\\n      initISR(timer);\\r\\n    servos[this->servoIndex].Pin.isActive = true;  // this must be set after the check for isTimerActive\\r\\n  }\\r\\n  return this->servoIndex ;\\r\\n}\\r\\n\\r\\nvoid Servo::detach()\\r\\n{\\r\\n  servos[this->servoIndex].Pin.isActive = false;\\r\\n  timer16_Sequence_t timer = SERVO_INDEX_TO_TIMER(servoIndex);\\r\\n  if(isTimerActive(timer) == false) {\\r\\n    finISR(timer);\\r\\n  }\\r\\n}\\r\\n\\r\\nvoid Servo::write(int value)\\r\\n{\\r\\n  if(value < MIN_PULSE_WIDTH)\\r\\n  {  // treat values less than 544 as angles in degrees (valid values in microseconds are handled as microseconds)\\r\\n    if(value < 0) value = 0;\\r\\n    if(value > 180) value = 180;\\r\\n    value = map(value, 0, 180, SERVO_MIN(),  SERVO_MAX());\\r\\n  }\\r\\n  this->writeMicroseconds(value);\\r\\n}\\r\\n\\r\\nvoid Servo::writeMicroseconds(int value)\\r\\n{\\r\\n  // calculate and store the values for the given channel\\r\\n  byte channel = this->servoIndex;\\r\\n  if( (channel < MAX_SERVOS) )   // ensure channel is valid\\r\\n  {\\r\\n    if( value < SERVO_MIN() )          // ensure pulse width is valid\\r\\n      value = SERVO_MIN();\\r\\n    else if( value > SERVO_MAX() )\\r\\n      value = SERVO_MAX();\\r\\n\\r\\n    value = value - TRIM_DURATION;\\r\\n    value = usToTicks(value);  // convert to ticks after compensating for interrupt overhead - 12 Aug 2009\\r\\n\\r\\n    uint8_t oldSREG = SREG;\\r\\n    cli();\\r\\n    servos[channel].ticks = value;\\r\\n    SREG = oldSREG;\\r\\n  }\\r\\n}\\r\\n\\r\\nint Servo::read() // return the value as degrees\\r\\n{\\r\\n  return  map( this->readMicroseconds()+1, SERVO_MIN(), SERVO_MAX(), 0, 180);\\r\\n}\\r\\n\\r\\nint Servo::readMicroseconds()\\r\\n{\\r\\n  unsigned int pulsewidth;\\r\\n  if( this->servoIndex != INVALID_SERVO )\\r\\n    pulsewidth = ticksToUs(servos[this->servoIndex].ticks)  + TRIM_DURATION ;   // 12 aug 2009\\r\\n  else\\r\\n    pulsewidth  = 0;\\r\\n\\r\\n  return pulsewidth;\\r\\n}\\r\\n\\r\\nbool Servo::attached()\\r\\n{\\r\\n  return servos[this->servoIndex].Pin.isActive ;\\r\\n}\\r\\n\\r\\n#endif // ARDUINO_ARCH_AVR\\r\\n\" }",
    "{ filename: \"src/Servo.h\", code: \"/*\\r\\n  Servo.h - Interrupt driven Servo library for Arduino using 16 bit timers- Version 2\\r\\n  Copyright (c) 2009 Michael Margolis.  All right reserved.\\r\\n\\r\\n  This library is free software; you can redistribute it and/or\\r\\n  modify it under the terms of the GNU Lesser General Public\\r\\n  License as published by the Free Software Foundation; either\\r\\n  version 2.1 of the License, or (at your option) any later version.\\r\\n\\r\\n  This library is distributed in the hope that it will be useful,\\r\\n  but WITHOUT ANY WARRANTY; without even the implied warranty of\\r\\n  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\\r\\n  Lesser General Public License for more details.\\r\\n\\r\\n  You should have received a copy of the GNU Lesser General Public\\r\\n  License along with this library; if not, write to the Free Software\\r\\n  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA\\r\\n*/\\r\\n\\r\\n/* \\r\\n  A servo is activated by creating an instance of the Servo class passing \\r\\n  the desired pin to the attach() method.\\r\\n  The servos are pulsed in the background using the value most recently \\r\\n  written using the write() method.\\r\\n\\r\\n  Note that analogWrite of PWM on pins associated with the timer are \\r\\n  disabled when the first servo is attached.\\r\\n  Timers are seized as needed in groups of 12 servos - 24 servos use two \\r\\n  timers, 48 servos will use four.\\r\\n  The sequence used to sieze timers is defined in timers.h\\r\\n\\r\\n  The methods are:\\r\\n\\r\\n    Servo - Class for manipulating servo motors connected to Arduino pins.\\r\\n\\r\\n    attach(pin )  - Attaches a servo motor to an i/o pin.\\r\\n    attach(pin, min, max  ) - Attaches to a pin setting min and max values in microseconds\\r\\n    default min is 544, max is 2400  \\r\\n \\r\\n    write()     - Sets the servo angle in degrees.  (invalid angle that is valid as pulse in microseconds is treated as microseconds)\\r\\n    writeMicroseconds() - Sets the servo pulse width in microseconds \\r\\n    read()      - Gets the last written servo pulse width as an angle between 0 and 180. \\r\\n    readMicroseconds()   - Gets the last written servo pulse width in microseconds. (was read_us() in first release)\\r\\n    attached()  - Returns true if there is a servo attached. \\r\\n    detach()    - Stops an attached servos from pulsing its i/o pin. \\r\\n */\\r\\n\\r\\n#ifndef Servo_h\\r\\n#define Servo_h\\r\\n\\r\\n#include <inttypes.h>\\r\\n\\r\\n/* \\r\\n * Defines for 16 bit timers used with  Servo library \\r\\n *\\r\\n * If _useTimerX is defined then TimerX is a 16 bit timer on the current board\\r\\n * timer16_Sequence_t enumerates the sequence that the timers should be allocated\\r\\n * _Nbr_16timers indicates how many 16 bit timers are available.\\r\\n */\\r\\n\\r\\n// Architecture specific include\\r\\n#if defined(ARDUINO_ARCH_AVR)\\r\\n#include \\\"ServoTimers.h\\\"\\r\\n#elif defined(ARDUINO_ARCH_SAM)\\r\\n#include \\\"sam/ServoTimers.h\\\"\\r\\n#else\\r\\n#error \\\"This library only supports boards with an AVR or SAM processor.\\\"\\r\\n#endif\\r\\n\\r\\n#define Servo_VERSION           2     // software version of this library\\r\\n\\r\\n#define MIN_PULSE_WIDTH       544     // the shortest pulse sent to a servo  \\r\\n#define MAX_PULSE_WIDTH      2400     // the longest pulse sent to a servo \\r\\n#define DEFAULT_PULSE_WIDTH  1500     // default pulse width when servo is attached\\r\\n#define REFRESH_INTERVAL    20000     // minumim time to refresh servos in microseconds \\r\\n\\r\\n#define SERVOS_PER_TIMER       12     // the maximum number of servos controlled by one timer \\r\\n#define MAX_SERVOS   (_Nbr_16timers  * SERVOS_PER_TIMER)\\r\\n\\r\\n#define INVALID_SERVO         255     // flag indicating an invalid servo index\\r\\n\\r\\ntypedef struct  {\\r\\n  uint8_t nbr        :6 ;             // a pin number from 0 to 63\\r\\n  uint8_t isActive   :1 ;             // true if this channel is enabled, pin not pulsed if false \\r\\n} ServoPin_t   ;  \\r\\n\\r\\ntypedef struct {\\r\\n  ServoPin_t Pin;\\r\\n  volatile unsigned int ticks;\\r\\n} servo_t;\\r\\n\\r\\nclass Servo\\r\\n{\\r\\npublic:\\r\\n  Servo();\\r\\n  uint8_t attach(int pin);           // attach the given pin to the next free channel, sets pinMode, returns channel number or 0 if failure\\r\\n  uint8_t attach(int pin, int min, int max); // as above but also sets min and max values for writes. \\r\\n  void detach();\\r\\n  void write(int value);             // if value is < 200 its treated as an angle, otherwise as pulse width in microseconds \\r\\n  void writeMicroseconds(int value); // Write pulse width in microseconds \\r\\n  int read();                        // returns current pulse width as an angle between 0 and 180 degrees\\r\\n  int readMicroseconds();            // returns current pulse width in microseconds for this servo (was read_us() in first release)\\r\\n  bool attached();                   // return true if this servo is attached, otherwise false \\r\\nprivate:\\r\\n   uint8_t servoIndex;               // index into the channel data for this servo\\r\\n   int8_t min;                       // minimum is this value times 4 added to MIN_PULSE_WIDTH    \\r\\n   int8_t max;                       // maximum is this value times 4 added to MAX_PULSE_WIDTH   \\r\\n};\\r\\n\\r\\n#endif\\r\\n\" }",
    "{ filename: \"src/ServoTimers.h\", code: \"/*\\r\\n  Servo.h - Interrupt driven Servo library for Arduino using 16 bit timers- Version 2\\r\\n  Copyright (c) 2009 Michael Margolis.  All right reserved.\\r\\n\\r\\n  This library is free software; you can redistribute it and/or\\r\\n  modify it under the terms of the GNU Lesser General Public\\r\\n  License as published by the Free Software Foundation; either\\r\\n  version 2.1 of the License, or (at your option) any later version.\\r\\n\\r\\n  This library is distributed in the hope that it will be useful,\\r\\n  but WITHOUT ANY WARRANTY; without even the implied warranty of\\r\\n  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\\r\\n  Lesser General Public License for more details.\\r\\n\\r\\n  You should have received a copy of the GNU Lesser General Public\\r\\n  License along with this library; if not, write to the Free Software\\r\\n  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA\\r\\n*/\\r\\n\\r\\n/*\\r\\n * Defines for 16 bit timers used with  Servo library\\r\\n *\\r\\n * If _useTimerX is defined then TimerX is a 16 bit timer on the current board\\r\\n * timer16_Sequence_t enumerates the sequence that the timers should be allocated\\r\\n * _Nbr_16timers indicates how many 16 bit timers are available.\\r\\n */\\r\\n\\r\\n/**\\r\\n * AVR Only definitions\\r\\n * --------------------\\r\\n */\\r\\n\\r\\n// Say which 16 bit timers can be used and in what order\\r\\n#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\r\\n#define _useTimer5\\r\\n#define _useTimer1\\r\\n#define _useTimer3\\r\\n#define _useTimer4\\r\\ntypedef enum { _timer5, _timer1, _timer3, _timer4, _Nbr_16timers } timer16_Sequence_t;\\r\\n\\r\\n#elif defined(__AVR_ATmega32U4__)\\r\\n#define _useTimer1\\r\\ntypedef enum { _timer1, _Nbr_16timers } timer16_Sequence_t;\\r\\n\\r\\n#elif defined(__AVR_AT90USB646__) || defined(__AVR_AT90USB1286__)\\r\\n#define _useTimer3\\r\\n#define _useTimer1\\r\\ntypedef enum { _timer3, _timer1, _Nbr_16timers } timer16_Sequence_t;\\r\\n\\r\\n#elif defined(__AVR_ATmega128__) ||defined(__AVR_ATmega1281__)||defined(__AVR_ATmega2561__)\\r\\n#define _useTimer3\\r\\n#define _useTimer1\\r\\ntypedef enum { _timer3, _timer1, _Nbr_16timers } timer16_Sequence_t;\\r\\n\\r\\n#else  // everything else\\r\\n#define _useTimer1\\r\\ntypedef enum { _timer1, _Nbr_16timers } timer16_Sequence_t;\\r\\n#endif\\r\\n\" }",
    "{ filename: \"src/IRremote.cpp\", code: \"/*\\n * IRremote\\n * Version 0.11 August, 2009\\n * Copyright 2009 Ken Shirriff\\n * For details, see http://arcfn.com/2009/08/multi-protocol-infrared-remote-library.html\\n *\\n * Modified by Paul Stoffregen <paul@pjrc.com> to support other boards and timers\\n * Modified  by Mitra Ardron <mitra@mitra.biz> \\n * Added Sanyo and Mitsubishi controllers\\n * Modified Sony to spot the repeat codes that some Sony's send\\n *\\n * Interrupt code based on NECIRrcv by Joe Knapp\\n * http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1210243556\\n * Also influenced by http://zovirl.com/2008/11/12/building-a-universal-remote-with-an-arduino/\\n *\\n * JVC and Panasonic protocol added by Kristian Lauszus (Thanks to zenwheel and other people at the original blog post)\\n */\\n\\n#include \\\"IRremote.h\\\"\\n#include \\\"IRremoteInt.h\\\"\\n\\n// Provides ISR\\n#include <avr/interrupt.h>\\n\\nvolatile irparams_t irparams;\\n\\n// These versions of MATCH, MATCH_MARK, and MATCH_SPACE are only for debugging.\\n// To use them, set DEBUG in IRremoteInt.h\\n// Normally macros are used for efficiency\\n#ifdef DEBUG\\nint MATCH(int measured, int desired) {\\n  Serial.print(\\\"Testing: \\\");\\n  Serial.print(TICKS_LOW(desired), DEC);\\n  Serial.print(\\\" <= \\\");\\n  Serial.print(measured, DEC);\\n  Serial.print(\\\" <= \\\");\\n  Serial.println(TICKS_HIGH(desired), DEC);\\n  return measured >= TICKS_LOW(desired) && measured <= TICKS_HIGH(desired);\\n}\\n\\nint MATCH_MARK(int measured_ticks, int desired_us) {\\n  Serial.print(\\\"Testing mark \\\");\\n  Serial.print(measured_ticks * USECPERTICK, DEC);\\n  Serial.print(\\\" vs \\\");\\n  Serial.print(desired_us, DEC);\\n  Serial.print(\\\": \\\");\\n  Serial.print(TICKS_LOW(desired_us + MARK_EXCESS), DEC);\\n  Serial.print(\\\" <= \\\");\\n  Serial.print(measured_ticks, DEC);\\n  Serial.print(\\\" <= \\\");\\n  Serial.println(TICKS_HIGH(desired_us + MARK_EXCESS), DEC);\\n  return measured_ticks >= TICKS_LOW(desired_us + MARK_EXCESS) && measured_ticks <= TICKS_HIGH(desired_us + MARK_EXCESS);\\n}\\n\\nint MATCH_SPACE(int measured_ticks, int desired_us) {\\n  Serial.print(\\\"Testing space \\\");\\n  Serial.print(measured_ticks * USECPERTICK, DEC);\\n  Serial.print(\\\" vs \\\");\\n  Serial.print(desired_us, DEC);\\n  Serial.print(\\\": \\\");\\n  Serial.print(TICKS_LOW(desired_us - MARK_EXCESS), DEC);\\n  Serial.print(\\\" <= \\\");\\n  Serial.print(measured_ticks, DEC);\\n  Serial.print(\\\" <= \\\");\\n  Serial.println(TICKS_HIGH(desired_us - MARK_EXCESS), DEC);\\n  return measured_ticks >= TICKS_LOW(desired_us - MARK_EXCESS) && measured_ticks <= TICKS_HIGH(desired_us - MARK_EXCESS);\\n}\\n#endif\\n\\nvoid IRsend::sendNEC(unsigned long data, int nbits)\\n{\\n  enableIROut(38);\\n  mark(NEC_HDR_MARK);\\n  space(NEC_HDR_SPACE);\\n  for (int i = 0; i < nbits; i++) {\\n    if (data & TOPBIT) {\\n      mark(NEC_BIT_MARK);\\n      space(NEC_ONE_SPACE);\\n    } \\n    else {\\n      mark(NEC_BIT_MARK);\\n      space(NEC_ZERO_SPACE);\\n    }\\n    data <<= 1;\\n  }\\n  mark(NEC_BIT_MARK);\\n  space(0);\\n}\\n\\nvoid IRsend::sendSony(unsigned long data, int nbits) {\\n  enableIROut(40);\\n  mark(SONY_HDR_MARK);\\n  space(SONY_HDR_SPACE);\\n  data = data << (32 - nbits);\\n  for (int i = 0; i < nbits; i++) {\\n    if (data & TOPBIT) {\\n      mark(SONY_ONE_MARK);\\n      space(SONY_HDR_SPACE);\\n    } \\n    else {\\n      mark(SONY_ZERO_MARK);\\n      space(SONY_HDR_SPACE);\\n    }\\n    data <<= 1;\\n  }\\n}\\n\\nvoid IRsend::sendRaw(unsigned int buf[], int len, int hz)\\n{\\n  enableIROut(hz);\\n  for (int i = 0; i < len; i++) {\\n    if (i & 1) {\\n      space(buf[i]);\\n    } \\n    else {\\n      mark(buf[i]);\\n    }\\n  }\\n  space(0); // Just to be sure\\n}\\n\\n// Note: first bit must be a one (start bit)\\nvoid IRsend::sendRC5(unsigned long data, int nbits)\\n{\\n  enableIROut(36);\\n  data = data << (32 - nbits);\\n  mark(RC5_T1); // First start bit\\n  space(RC5_T1); // Second start bit\\n  mark(RC5_T1); // Second start bit\\n  for (int i = 0; i < nbits; i++) {\\n    if (data & TOPBIT) {\\n      space(RC5_T1); // 1 is space, then mark\\n      mark(RC5_T1);\\n    } \\n    else {\\n      mark(RC5_T1);\\n      space(RC5_T1);\\n    }\\n    data <<= 1;\\n  }\\n  space(0); // Turn off at end\\n}\\n\\n// Caller needs to take care of flipping the toggle bit\\nvoid IRsend::sendRC6(unsigned long data, int nbits)\\n{\\n  enableIROut(36);\\n  data = data << (32 - nbits);\\n  mark(RC6_HDR_MARK);\\n  space(RC6_HDR_SPACE);\\n  mark(RC6_T1); // start bit\\n  space(RC6_T1);\\n  int t;\\n  for (int i = 0; i < nbits; i++) {\\n    if (i == 3) {\\n      // double-wide trailer bit\\n      t = 2 * RC6_T1;\\n    } \\n    else {\\n      t = RC6_T1;\\n    }\\n    if (data & TOPBIT) {\\n      mark(t);\\n      space(t);\\n    } \\n    else {\\n      space(t);\\n      mark(t);\\n    }\\n\\n    data <<= 1;\\n  }\\n  space(0); // Turn off at end\\n}\\nvoid IRsend::sendPanasonic(unsigned int address, unsigned long data) {\\n    enableIROut(35);\\n    mark(PANASONIC_HDR_MARK);\\n    space(PANASONIC_HDR_SPACE);\\n    \\n    for(int i=0;i<16;i++)\\n    {\\n        mark(PANASONIC_BIT_MARK);\\n        if (address & 0x8000) {\\n            space(PANASONIC_ONE_SPACE);\\n        } else {\\n            space(PANASONIC_ZERO_SPACE);\\n        }\\n        address <<= 1;        \\n    }    \\n    for (int i=0; i < 32; i++) {\\n        mark(PANASONIC_BIT_MARK);\\n        if (data & TOPBIT) {\\n            space(PANASONIC_ONE_SPACE);\\n        } else {\\n            space(PANASONIC_ZERO_SPACE);\\n        }\\n        data <<= 1;\\n    }\\n    mark(PANASONIC_BIT_MARK);\\n    space(0);\\n}\\nvoid IRsend::sendJVC(unsigned long data, int nbits, int repeat)\\n{\\n    enableIROut(38);\\n    data = data << (32 - nbits);\\n    if (!repeat){\\n        mark(JVC_HDR_MARK);\\n        space(JVC_HDR_SPACE); \\n    }\\n    for (int i = 0; i < nbits; i++) {\\n        if (data & TOPBIT) {\\n            mark(JVC_BIT_MARK);\\n            space(JVC_ONE_SPACE); \\n        } \\n        else {\\n            mark(JVC_BIT_MARK);\\n            space(JVC_ZERO_SPACE); \\n        }\\n        data <<= 1;\\n    }\\n    mark(JVC_BIT_MARK);\\n    space(0);\\n}\\nvoid IRsend::mark(int time) {\\n  // Sends an IR mark for the specified number of microseconds.\\n  // The mark output is modulated at the PWM frequency.\\n  TIMER_ENABLE_PWM; // Enable pin 3 PWM output\\n  if (time > 0) delayMicroseconds(time);\\n}\\n\\n/* Leave pin off for time (given in microseconds) */\\nvoid IRsend::space(int time) {\\n  // Sends an IR space for the specified number of microseconds.\\n  // A space is no output, so the PWM output is disabled.\\n  TIMER_DISABLE_PWM; // Disable pin 3 PWM output\\n  if (time > 0) delayMicroseconds(time);\\n}\\n\\nvoid IRsend::enableIROut(int khz) {\\n  // Enables IR output.  The khz value controls the modulation frequency in kilohertz.\\n  // The IR output will be on pin 3 (OC2B).\\n  // This routine is designed for 36-40KHz; if you use it for other values, it's up to you\\n  // to make sure it gives reasonable results.  (Watch out for overflow / underflow / rounding.)\\n  // TIMER2 is used in phase-correct PWM mode, with OCR2A controlling the frequency and OCR2B\\n  // controlling the duty cycle.\\n  // There is no prescaling, so the output frequency is 16MHz / (2 * OCR2A)\\n  // To turn the output on and off, we leave the PWM running, but connect and disconnect the output pin.\\n  // A few hours staring at the ATmega documentation and this will all make sense.\\n  // See my Secrets of Arduino PWM at http://arcfn.com/2009/07/secrets-of-arduino-pwm.html for details.\\n\\n  \\n  // Disable the Timer2 Interrupt (which is used for receiving IR)\\n  TIMER_DISABLE_INTR; //Timer2 Overflow Interrupt\\n  \\n  pinMode(TIMER_PWM_PIN, OUTPUT);\\n  digitalWrite(TIMER_PWM_PIN, LOW); // When not sending PWM, we want it low\\n  \\n  // COM2A = 00: disconnect OC2A\\n  // COM2B = 00: disconnect OC2B; to send signal set to 10: OC2B non-inverted\\n  // WGM2 = 101: phase-correct PWM with OCRA as top\\n  // CS2 = 000: no prescaling\\n  // The top value for the timer.  The modulation frequency will be SYSCLOCK / 2 / OCR2A.\\n  TIMER_CONFIG_KHZ(khz);\\n}\\n\\nIRrecv::IRrecv(int recvpin)\\n{\\n  irparams.recvpin = recvpin;\\n  irparams.blinkflag = 0;\\n}\\n\\n// initialization\\nvoid IRrecv::enableIRIn() {\\n  cli();\\n  // setup pulse clock timer interrupt\\n  //Prescale /8 (16M/8 = 0.5 microseconds per tick)\\n  // Therefore, the timer interval can range from 0.5 to 128 microseconds\\n  // depending on the reset value (255 to 0)\\n  TIMER_CONFIG_NORMAL();\\n\\n  //Timer2 Overflow Interrupt Enable\\n  TIMER_ENABLE_INTR;\\n\\n  TIMER_RESET;\\n\\n  sei();  // enable interrupts\\n\\n  // initialize state machine variables\\n  irparams.rcvstate = STATE_IDLE;\\n  irparams.rawlen = 0;\\n\\n  // set pin modes\\n  pinMode(irparams.recvpin, INPUT);\\n}\\n\\n// enable/disable blinking of pin 13 on IR processing\\nvoid IRrecv::blink13(int blinkflag)\\n{\\n  irparams.blinkflag = blinkflag;\\n  if (blinkflag)\\n    pinMode(BLINKLED, OUTPUT);\\n}\\n\\n// TIMER2 interrupt code to collect raw data.\\n// Widths of alternating SPACE, MARK are recorded in rawbuf.\\n// Recorded in ticks of 50 microseconds.\\n// rawlen counts the number of entries recorded so far.\\n// First entry is the SPACE between transmissions.\\n// As soon as a SPACE gets long, ready is set, state switches to IDLE, timing of SPACE continues.\\n// As soon as first MARK arrives, gap width is recorded, ready is cleared, and new logging starts\\nISR(TIMER_INTR_NAME)\\n{\\n  TIMER_RESET;\\n\\n  uint8_t irdata = (uint8_t)digitalRead(irparams.recvpin);\\n\\n  irparams.timer++; // One more 50us tick\\n  if (irparams.rawlen >= RAWBUF) {\\n    // Buffer overflow\\n    irparams.rcvstate = STATE_STOP;\\n  }\\n  switch(irparams.rcvstate) {\\n  case STATE_IDLE: // In the middle of a gap\\n    if (irdata == MARK) {\\n      if (irparams.timer < GAP_TICKS) {\\n        // Not big enough to be a gap.\\n        irparams.timer = 0;\\n      } \\n      else {\\n        // gap just ended, record duration and start recording transmission\\n        irparams.rawlen = 0;\\n        irparams.rawbuf[irparams.rawlen++] = irparams.timer;\\n        irparams.timer = 0;\\n        irparams.rcvstate = STATE_MARK;\\n      }\\n    }\\n    break;\\n  case STATE_MARK: // timing MARK\\n    if (irdata == SPACE) {   // MARK ended, record time\\n      irparams.rawbuf[irparams.rawlen++] = irparams.timer;\\n      irparams.timer = 0;\\n      irparams.rcvstate = STATE_SPACE;\\n    }\\n    break;\\n  case STATE_SPACE: // timing SPACE\\n    if (irdata == MARK) { // SPACE just ended, record it\\n      irparams.rawbuf[irparams.rawlen++] = irparams.timer;\\n      irparams.timer = 0;\\n      irparams.rcvstate = STATE_MARK;\\n    } \\n    else { // SPACE\\n      if (irparams.timer > GAP_TICKS) {\\n        // big SPACE, indicates gap between codes\\n        // Mark current code as ready for processing\\n        // Switch to STOP\\n        // Don't reset timer; keep counting space width\\n        irparams.rcvstate = STATE_STOP;\\n      } \\n    }\\n    break;\\n  case STATE_STOP: // waiting, measuring gap\\n    if (irdata == MARK) { // reset gap timer\\n      irparams.timer = 0;\\n    }\\n    break;\\n  }\\n\\n  if (irparams.blinkflag) {\\n    if (irdata == MARK) {\\n      BLINKLED_ON();  // turn pin 13 LED on\\n    } \\n    else {\\n      BLINKLED_OFF();  // turn pin 13 LED off\\n    }\\n  }\\n}\\n\\nvoid IRrecv::resume() {\\n  irparams.rcvstate = STATE_IDLE;\\n  irparams.rawlen = 0;\\n}\\n\\n\\n\\n// Decodes the received IR message\\n// Returns 0 if no data ready, 1 if data ready.\\n// Results of decoding are stored in results\\nint IRrecv::decode(decode_results *results) {\\n  results->rawbuf = irparams.rawbuf;\\n  results->rawlen = irparams.rawlen;\\n  if (irparams.rcvstate != STATE_STOP) {\\n    return ERR;\\n  }\\n#ifdef DEBUG\\n  Serial.println(\\\"Attempting NEC decode\\\");\\n#endif\\n  if (decodeNEC(results)) {\\n    return DECODED;\\n  }\\n#ifdef DEBUG\\n  Serial.println(\\\"Attempting Sony decode\\\");\\n#endif\\n  if (decodeSony(results)) {\\n    return DECODED;\\n  }\\n#ifdef DEBUG\\n  Serial.println(\\\"Attempting Sanyo decode\\\");\\n#endif\\n  if (decodeSanyo(results)) {\\n    return DECODED;\\n  }\\n#ifdef DEBUG\\n  Serial.println(\\\"Attempting Mitsubishi decode\\\");\\n#endif\\n  if (decodeMitsubishi(results)) {\\n    return DECODED;\\n  }\\n#ifdef DEBUG\\n  Serial.println(\\\"Attempting RC5 decode\\\");\\n#endif  \\n  if (decodeRC5(results)) {\\n    return DECODED;\\n  }\\n#ifdef DEBUG\\n  Serial.println(\\\"Attempting RC6 decode\\\");\\n#endif \\n  if (decodeRC6(results)) {\\n    return DECODED;\\n  }\\n#ifdef DEBUG\\n    Serial.println(\\\"Attempting Panasonic decode\\\");\\n#endif \\n    if (decodePanasonic(results)) {\\n        return DECODED;\\n    }\\n#ifdef DEBUG\\n    Serial.println(\\\"Attempting JVC decode\\\");\\n#endif \\n    if (decodeJVC(results)) {\\n        return DECODED;\\n    }\\n  // decodeHash returns a hash on any input.\\n  // Thus, it needs to be last in the list.\\n  // If you add any decodes, add them before this.\\n  if (decodeHash(results)) {\\n    return DECODED;\\n  }\\n  // Throw away and start over\\n  resume();\\n  return ERR;\\n}\\n\\n// NECs have a repeat only 4 items long\\nlong IRrecv::decodeNEC(decode_results *results) {\\n  long data = 0;\\n  int offset = 1; // Skip first space\\n  // Initial mark\\n  if (!MATCH_MARK(results->rawbuf[offset], NEC_HDR_MARK)) {\\n    return ERR;\\n  }\\n  offset++;\\n  // Check for repeat\\n  if (irparams.rawlen == 4 &&\\n    MATCH_SPACE(results->rawbuf[offset], NEC_RPT_SPACE) &&\\n    MATCH_MARK(results->rawbuf[offset+1], NEC_BIT_MARK)) {\\n    results->bits = 0;\\n    results->value = REPEAT;\\n    results->decode_type = NEC;\\n    return DECODED;\\n  }\\n  if (irparams.rawlen < 2 * NEC_BITS + 4) {\\n    return ERR;\\n  }\\n  // Initial space  \\n  if (!MATCH_SPACE(results->rawbuf[offset], NEC_HDR_SPACE)) {\\n    return ERR;\\n  }\\n  offset++;\\n  for (int i = 0; i < NEC_BITS; i++) {\\n    if (!MATCH_MARK(results->rawbuf[offset], NEC_BIT_MARK)) {\\n      return ERR;\\n    }\\n    offset++;\\n    if (MATCH_SPACE(results->rawbuf[offset], NEC_ONE_SPACE)) {\\n      data = (data << 1) | 1;\\n    } \\n    else if (MATCH_SPACE(results->rawbuf[offset], NEC_ZERO_SPACE)) {\\n      data <<= 1;\\n    } \\n    else {\\n      return ERR;\\n    }\\n    offset++;\\n  }\\n  // Success\\n  results->bits = NEC_BITS;\\n  results->value = data;\\n  results->decode_type = NEC;\\n  return DECODED;\\n}\\n\\nlong IRrecv::decodeSony(decode_results *results) {\\n  long data = 0;\\n  if (irparams.rawlen < 2 * SONY_BITS + 2) {\\n    return ERR;\\n  }\\n  int offset = 0; // Dont skip first space, check its size\\n\\n  // Some Sony's deliver repeats fast after first\\n  // unfortunately can't spot difference from of repeat from two fast clicks\\n  if (results->rawbuf[offset] < SONY_DOUBLE_SPACE_USECS) {\\n    // Serial.print(\\\"IR Gap found: \\\");\\n    results->bits = 0;\\n    results->value = REPEAT;\\n    results->decode_type = SANYO;\\n    return DECODED;\\n  }\\n  offset++;\\n\\n  // Initial mark\\n  if (!MATCH_MARK(results->rawbuf[offset], SONY_HDR_MARK)) {\\n    return ERR;\\n  }\\n  offset++;\\n\\n  while (offset + 1 < irparams.rawlen) {\\n    if (!MATCH_SPACE(results->rawbuf[offset], SONY_HDR_SPACE)) {\\n      break;\\n    }\\n    offset++;\\n    if (MATCH_MARK(results->rawbuf[offset], SONY_ONE_MARK)) {\\n      data = (data << 1) | 1;\\n    } \\n    else if (MATCH_MARK(results->rawbuf[offset], SONY_ZERO_MARK)) {\\n      data <<= 1;\\n    } \\n    else {\\n      return ERR;\\n    }\\n    offset++;\\n  }\\n\\n  // Success\\n  results->bits = (offset - 1) / 2;\\n  if (results->bits < 12) {\\n    results->bits = 0;\\n    return ERR;\\n  }\\n  results->value = data;\\n  results->decode_type = SONY;\\n  return DECODED;\\n}\\n\\n// I think this is a Sanyo decoder - serial = SA 8650B\\n// Looks like Sony except for timings, 48 chars of data and time/space different\\nlong IRrecv::decodeSanyo(decode_results *results) {\\n  long data = 0;\\n  if (irparams.rawlen < 2 * SANYO_BITS + 2) {\\n    return ERR;\\n  }\\n  int offset = 0; // Skip first space\\n  // Initial space  \\n  /* Put this back in for debugging - note can't use #DEBUG as if Debug on we don't see the repeat cos of the delay\\n  Serial.print(\\\"IR Gap: \\\");\\n  Serial.println( results->rawbuf[offset]);\\n  Serial.println( \\\"test against:\\\");\\n  Serial.println(results->rawbuf[offset]);\\n  */\\n  if (results->rawbuf[offset] < SANYO_DOUBLE_SPACE_USECS) {\\n    // Serial.print(\\\"IR Gap found: \\\");\\n    results->bits = 0;\\n    results->value = REPEAT;\\n    results->decode_type = SANYO;\\n    return DECODED;\\n  }\\n  offset++;\\n\\n  // Initial mark\\n  if (!MATCH_MARK(results->rawbuf[offset], SANYO_HDR_MARK)) {\\n    return ERR;\\n  }\\n  offset++;\\n\\n  // Skip Second Mark\\n  if (!MATCH_MARK(results->rawbuf[offset], SANYO_HDR_MARK)) {\\n    return ERR;\\n  }\\n  offset++;\\n\\n  while (offset + 1 < irparams.rawlen) {\\n    if (!MATCH_SPACE(results->rawbuf[offset], SANYO_HDR_SPACE)) {\\n      break;\\n    }\\n    offset++;\\n    if (MATCH_MARK(results->rawbuf[offset], SANYO_ONE_MARK)) {\\n      data = (data << 1) | 1;\\n    } \\n    else if (MATCH_MARK(results->rawbuf[offset], SANYO_ZERO_MARK)) {\\n      data <<= 1;\\n    } \\n    else {\\n      return ERR;\\n    }\\n    offset++;\\n  }\\n\\n  // Success\\n  results->bits = (offset - 1) / 2;\\n  if (results->bits < 12) {\\n    results->bits = 0;\\n    return ERR;\\n  }\\n  results->value = data;\\n  results->decode_type = SANYO;\\n  return DECODED;\\n}\\n\\n// Looks like Sony except for timings, 48 chars of data and time/space different\\nlong IRrecv::decodeMitsubishi(decode_results *results) {\\n  // Serial.print(\\\"?!? decoding Mitsubishi:\\\");Serial.print(irparams.rawlen); Serial.print(\\\" want \\\"); Serial.println( 2 * MITSUBISHI_BITS + 2);\\n  long data = 0;\\n  if (irparams.rawlen < 2 * MITSUBISHI_BITS + 2) {\\n    return ERR;\\n  }\\n  int offset = 0; // Skip first space\\n  // Initial space  \\n  /* Put this back in for debugging - note can't use #DEBUG as if Debug on we don't see the repeat cos of the delay\\n  Serial.print(\\\"IR Gap: \\\");\\n  Serial.println( results->rawbuf[offset]);\\n  Serial.println( \\\"test against:\\\");\\n  Serial.println(results->rawbuf[offset]);\\n  */\\n  /* Not seeing double keys from Mitsubishi\\n  if (results->rawbuf[offset] < MITSUBISHI_DOUBLE_SPACE_USECS) {\\n    // Serial.print(\\\"IR Gap found: \\\");\\n    results->bits = 0;\\n    results->value = REPEAT;\\n    results->decode_type = MITSUBISHI;\\n    return DECODED;\\n  }\\n  */\\n  offset++;\\n\\n  // Typical\\n  // 14200 7 41 7 42 7 42 7 17 7 17 7 18 7 41 7 18 7 17 7 17 7 18 7 41 8 17 7 17 7 18 7 17 7 \\n\\n  // Initial Space\\n  if (!MATCH_MARK(results->rawbuf[offset], MITSUBISHI_HDR_SPACE)) {\\n    return ERR;\\n  }\\n  offset++;\\n  while (offset + 1 < irparams.rawlen) {\\n    if (MATCH_MARK(results->rawbuf[offset], MITSUBISHI_ONE_MARK)) {\\n      data = (data << 1) | 1;\\n    } \\n    else if (MATCH_MARK(results->rawbuf[offset], MITSUBISHI_ZERO_MARK)) {\\n      data <<= 1;\\n    } \\n    else {\\n      // Serial.println(\\\"A\\\"); Serial.println(offset); Serial.println(results->rawbuf[offset]);\\n      return ERR;\\n    }\\n    offset++;\\n    if (!MATCH_SPACE(results->rawbuf[offset], MITSUBISHI_HDR_SPACE)) {\\n      // Serial.println(\\\"B\\\"); Serial.println(offset); Serial.println(results->rawbuf[offset]);\\n      break;\\n    }\\n    offset++;\\n  }\\n\\n  // Success\\n  results->bits = (offset - 1) / 2;\\n  if (results->bits < MITSUBISHI_BITS) {\\n    results->bits = 0;\\n    return ERR;\\n  }\\n  results->value = data;\\n  results->decode_type = MITSUBISHI;\\n  return DECODED;\\n}\\n\\n\\n// Gets one undecoded level at a time from the raw buffer.\\n// The RC5/6 decoding is easier if the data is broken into time intervals.\\n// E.g. if the buffer has MARK for 2 time intervals and SPACE for 1,\\n// successive calls to getRClevel will return MARK, MARK, SPACE.\\n// offset and used are updated to keep track of the current position.\\n// t1 is the time interval for a single bit in microseconds.\\n// Returns -1 for error (measured time interval is not a multiple of t1).\\nint IRrecv::getRClevel(decode_results *results, int *offset, int *used, int t1) {\\n  if (*offset >= results->rawlen) {\\n    // After end of recorded buffer, assume SPACE.\\n    return SPACE;\\n  }\\n  int width = results->rawbuf[*offset];\\n  int val = ((*offset) % 2) ? MARK : SPACE;\\n  int correction = (val == MARK) ? MARK_EXCESS : - MARK_EXCESS;\\n\\n  int avail;\\n  if (MATCH(width, t1 + correction)) {\\n    avail = 1;\\n  } \\n  else if (MATCH(width, 2*t1 + correction)) {\\n    avail = 2;\\n  } \\n  else if (MATCH(width, 3*t1 + correction)) {\\n    avail = 3;\\n  } \\n  else {\\n    return -1;\\n  }\\n\\n  (*used)++;\\n  if (*used >= avail) {\\n    *used = 0;\\n    (*offset)++;\\n  }\\n#ifdef DEBUG\\n  if (val == MARK) {\\n    Serial.println(\\\"MARK\\\");\\n  } \\n  else {\\n    Serial.println(\\\"SPACE\\\");\\n  }\\n#endif\\n  return val;   \\n}\\n\\nlong IRrecv::decodeRC5(decode_results *results) {\\n  if (irparams.rawlen < MIN_RC5_SAMPLES + 2) {\\n    return ERR;\\n  }\\n  int offset = 1; // Skip gap space\\n  long data = 0;\\n  int used = 0;\\n  // Get start bits\\n  if (getRClevel(results, &offset, &used, RC5_T1) != MARK) return ERR;\\n  if (getRClevel(results, &offset, &used, RC5_T1) != SPACE) return ERR;\\n  if (getRClevel(results, &offset, &used, RC5_T1) != MARK) return ERR;\\n  int nbits;\\n  for (nbits = 0; offset < irparams.rawlen; nbits++) {\\n    int levelA = getRClevel(results, &offset, &used, RC5_T1); \\n    int levelB = getRClevel(results, &offset, &used, RC5_T1);\\n    if (levelA == SPACE && levelB == MARK) {\\n      // 1 bit\\n      data = (data << 1) | 1;\\n    } \\n    else if (levelA == MARK && levelB == SPACE) {\\n      // zero bit\\n      data <<= 1;\\n    } \\n    else {\\n      return ERR;\\n    } \\n  }\\n\\n  // Success\\n  results->bits = nbits;\\n  results->value = data;\\n  results->decode_type = RC5;\\n  return DECODED;\\n}\\n\\nlong IRrecv::decodeRC6(decode_results *results) {\\n  if (results->rawlen < MIN_RC6_SAMPLES) {\\n    return ERR;\\n  }\\n  int offset = 1; // Skip first space\\n  // Initial mark\\n  if (!MATCH_MARK(results->rawbuf[offset], RC6_HDR_MARK)) {\\n    return ERR;\\n  }\\n  offset++;\\n  if (!MATCH_SPACE(results->rawbuf[offset], RC6_HDR_SPACE)) {\\n    return ERR;\\n  }\\n  offset++;\\n  long data = 0;\\n  int used = 0;\\n  // Get start bit (1)\\n  if (getRClevel(results, &offset, &used, RC6_T1) != MARK) return ERR;\\n  if (getRClevel(results, &offset, &used, RC6_T1) != SPACE) return ERR;\\n  int nbits;\\n  for (nbits = 0; offset < results->rawlen; nbits++) {\\n    int levelA, levelB; // Next two levels\\n    levelA = getRClevel(results, &offset, &used, RC6_T1); \\n    if (nbits == 3) {\\n      // T bit is double wide; make sure second half matches\\n      if (levelA != getRClevel(results, &offset, &used, RC6_T1)) return ERR;\\n    } \\n    levelB = getRClevel(results, &offset, &used, RC6_T1);\\n    if (nbits == 3) {\\n      // T bit is double wide; make sure second half matches\\n      if (levelB != getRClevel(results, &offset, &used, RC6_T1)) return ERR;\\n    } \\n    if (levelA == MARK && levelB == SPACE) { // reversed compared to RC5\\n      // 1 bit\\n      data = (data << 1) | 1;\\n    } \\n    else if (levelA == SPACE && levelB == MARK) {\\n      // zero bit\\n      data <<= 1;\\n    } \\n    else {\\n      return ERR; // Error\\n    } \\n  }\\n  // Success\\n  results->bits = nbits;\\n  results->value = data;\\n  results->decode_type = RC6;\\n  return DECODED;\\n}\\nlong IRrecv::decodePanasonic(decode_results *results) {\\n    unsigned long long data = 0;\\n    int offset = 1;\\n    \\n    if (!MATCH_MARK(results->rawbuf[offset], PANASONIC_HDR_MARK)) {\\n        return ERR;\\n    }\\n    offset++;\\n    if (!MATCH_MARK(results->rawbuf[offset], PANASONIC_HDR_SPACE)) {\\n        return ERR;\\n    }\\n    offset++;\\n    \\n    // decode address\\n    for (int i = 0; i < PANASONIC_BITS; i++) {\\n        if (!MATCH_MARK(results->rawbuf[offset++], PANASONIC_BIT_MARK)) {\\n            return ERR;\\n        }\\n        if (MATCH_SPACE(results->rawbuf[offset],PANASONIC_ONE_SPACE)) {\\n            data = (data << 1) | 1;\\n        } else if (MATCH_SPACE(results->rawbuf[offset],PANASONIC_ZERO_SPACE)) {\\n            data <<= 1;\\n        } else {\\n            return ERR;\\n        }\\n        offset++;\\n    }\\n    results->value = (unsigned long)data;\\n    results->panasonicAddress = (unsigned int)(data >> 32);\\n    results->decode_type = PANASONIC;\\n    results->bits = PANASONIC_BITS;\\n    return DECODED;\\n}\\nlong IRrecv::decodeJVC(decode_results *results) {\\n    long data = 0;\\n    int offset = 1; // Skip first space\\n    // Check for repeat\\n    if (irparams.rawlen - 1 == 33 &&\\n        MATCH_MARK(results->rawbuf[offset], JVC_BIT_MARK) &&\\n        MATCH_MARK(results->rawbuf[irparams.rawlen-1], JVC_BIT_MARK)) {\\n        results->bits = 0;\\n        results->value = REPEAT;\\n        results->decode_type = JVC;\\n        return DECODED;\\n    } \\n    // Initial mark\\n    if (!MATCH_MARK(results->rawbuf[offset], JVC_HDR_MARK)) {\\n        return ERR;\\n    }\\n    offset++; \\n    if (irparams.rawlen < 2 * JVC_BITS + 1 ) {\\n        return ERR;\\n    }\\n    // Initial space \\n    if (!MATCH_SPACE(results->rawbuf[offset], JVC_HDR_SPACE)) {\\n        return ERR;\\n    }\\n    offset++;\\n    for (int i = 0; i < JVC_BITS; i++) {\\n        if (!MATCH_MARK(results->rawbuf[offset], JVC_BIT_MARK)) {\\n            return ERR;\\n        }\\n        offset++;\\n        if (MATCH_SPACE(results->rawbuf[offset], JVC_ONE_SPACE)) {\\n            data = (data << 1) | 1;\\n        } \\n        else if (MATCH_SPACE(results->rawbuf[offset], JVC_ZERO_SPACE)) {\\n            data <<= 1;\\n        } \\n        else {\\n            return ERR;\\n        }\\n        offset++;\\n    }\\n    //Stop bit\\n    if (!MATCH_MARK(results->rawbuf[offset], JVC_BIT_MARK)){\\n        return ERR;\\n    }\\n    // Success\\n    results->bits = JVC_BITS;\\n    results->value = data;\\n    results->decode_type = JVC;\\n    return DECODED;\\n}\\n\\n/* -----------------------------------------------------------------------\\n * hashdecode - decode an arbitrary IR code.\\n * Instead of decoding using a standard encoding scheme\\n * (e.g. Sony, NEC, RC5), the code is hashed to a 32-bit value.\\n *\\n * The algorithm: look at the sequence of MARK signals, and see if each one\\n * is shorter (0), the same length (1), or longer (2) than the previous.\\n * Do the same with the SPACE signals.  Hszh the resulting sequence of 0's,\\n * 1's, and 2's to a 32-bit value.  This will give a unique value for each\\n * different code (probably), for most code systems.\\n *\\n * http://arcfn.com/2010/01/using-arbitrary-remotes-with-arduino.html\\n */\\n\\n// Compare two tick values, returning 0 if newval is shorter,\\n// 1 if newval is equal, and 2 if newval is longer\\n// Use a tolerance of 20%\\nint IRrecv::compare(unsigned int oldval, unsigned int newval) {\\n  if (newval < oldval * .8) {\\n    return 0;\\n  } \\n  else if (oldval < newval * .8) {\\n    return 2;\\n  } \\n  else {\\n    return 1;\\n  }\\n}\\n\\n// Use FNV hash algorithm: http://isthe.com/chongo/tech/comp/fnv/#FNV-param\\n#define FNV_PRIME_32 16777619\\n#define FNV_BASIS_32 2166136261\\n\\n/* Converts the raw code values into a 32-bit hash code.\\n * Hopefully this code is unique for each button.\\n * This isn't a \\\"real\\\" decoding, just an arbitrary value.\\n */\\nlong IRrecv::decodeHash(decode_results *results) {\\n  // Require at least 6 samples to prevent triggering on noise\\n  if (results->rawlen < 6) {\\n    return ERR;\\n  }\\n  long hash = FNV_BASIS_32;\\n  for (int i = 1; i+2 < results->rawlen; i++) {\\n    int value =  compare(results->rawbuf[i], results->rawbuf[i+2]);\\n    // Add value into the hash\\n    hash = (hash * FNV_PRIME_32) ^ value;\\n  }\\n  results->value = hash;\\n  results->bits = 32;\\n  results->decode_type = UNKNOWN;\\n  return DECODED;\\n}\\n\\n/* Sharp and DISH support by Todd Treece ( http://unionbridge.org/design/ircommand )\\n\\nThe Dish send function needs to be repeated 4 times, and the Sharp function\\nhas the necessary repeat built in because of the need to invert the signal.\\n\\nSharp protocol documentation:\\nhttp://www.sbprojects.com/knowledge/ir/sharp.htm\\n\\nHere are the LIRC files that I found that seem to match the remote codes\\nfrom the oscilloscope:\\n\\nSharp LCD TV:\\nhttp://lirc.sourceforge.net/remotes/sharp/GA538WJSA\\n\\nDISH NETWORK (echostar 301):\\nhttp://lirc.sourceforge.net/remotes/echostar/301_501_3100_5100_58xx_59xx\\n\\nFor the DISH codes, only send the last for characters of the hex.\\ni.e. use 0x1C10 instead of 0x0000000000001C10 which is listed in the\\nlinked LIRC file.\\n*/\\n\\nvoid IRsend::sendSharp(unsigned long data, int nbits) {\\n  unsigned long invertdata = data ^ SHARP_TOGGLE_MASK;\\n  enableIROut(38);\\n  for (int i = 0; i < nbits; i++) {\\n    if (data & 0x4000) {\\n      mark(SHARP_BIT_MARK);\\n      space(SHARP_ONE_SPACE);\\n    }\\n    else {\\n      mark(SHARP_BIT_MARK);\\n      space(SHARP_ZERO_SPACE);\\n    }\\n    data <<= 1;\\n  }\\n  \\n  mark(SHARP_BIT_MARK);\\n  space(SHARP_ZERO_SPACE);\\n  delay(46);\\n  for (int i = 0; i < nbits; i++) {\\n    if (invertdata & 0x4000) {\\n      mark(SHARP_BIT_MARK);\\n      space(SHARP_ONE_SPACE);\\n    }\\n    else {\\n      mark(SHARP_BIT_MARK);\\n      space(SHARP_ZERO_SPACE);\\n    }\\n    invertdata <<= 1;\\n  }\\n  mark(SHARP_BIT_MARK);\\n  space(SHARP_ZERO_SPACE);\\n  delay(46);\\n}\\n\\nvoid IRsend::sendDISH(unsigned long data, int nbits)\\n{\\n  enableIROut(56);\\n  mark(DISH_HDR_MARK);\\n  space(DISH_HDR_SPACE);\\n  for (int i = 0; i < nbits; i++) {\\n    if (data & DISH_TOP_BIT) {\\n      mark(DISH_BIT_MARK);\\n      space(DISH_ONE_SPACE);\\n    }\\n    else {\\n      mark(DISH_BIT_MARK);\\n      space(DISH_ZERO_SPACE);\\n    }\\n    data <<= 1;\\n  }\\n}\\n\" }",
    "{ filename: \"src/IRremote.h\", code: \"/*\\n * IRremote\\n * Version 0.1 July, 2009\\n * Copyright 2009 Ken Shirriff\\n * For details, see http://arcfn.com/2009/08/multi-protocol-infrared-remote-library.htm http://arcfn.com\\n * Edited by Mitra to add new controller SANYO\\n *\\n * Interrupt code based on NECIRrcv by Joe Knapp\\n * http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1210243556\\n * Also influenced by http://zovirl.com/2008/11/12/building-a-universal-remote-with-an-arduino/\\n *\\n * JVC and Panasonic protocol added by Kristian Lauszus (Thanks to zenwheel and other people at the original blog post)\\n */\\n\\n#ifndef IRremote_h\\n#define IRremote_h\\n\\n// The following are compile-time library options.\\n// If you change them, recompile the library.\\n// If DEBUG is defined, a lot of debugging output will be printed during decoding.\\n// TEST must be defined for the IRtest unittests to work.  It will make some\\n// methods virtual, which will be slightly slower, which is why it is optional.\\n// #define DEBUG\\n// #define TEST\\n\\n// Results returned from the decoder\\nclass decode_results {\\npublic:\\n  int decode_type; // NEC, SONY, RC5, UNKNOWN\\n  unsigned int panasonicAddress; // This is only used for decoding Panasonic data\\n  unsigned long value; // Decoded value\\n  int bits; // Number of bits in decoded value\\n  volatile unsigned int *rawbuf; // Raw intervals in .5 us ticks\\n  int rawlen; // Number of records in rawbuf.\\n};\\n\\n// Values for decode_type\\n#define NEC 1\\n#define SONY 2\\n#define RC5 3\\n#define RC6 4\\n#define DISH 5\\n#define SHARP 6\\n#define PANASONIC 7\\n#define JVC 8\\n#define SANYO 9\\n#define MITSUBISHI 10\\n#define UNKNOWN -1\\n\\n// Decoded value for NEC when a repeat code is received\\n#define REPEAT 0xffffffff\\n\\n// main class for receiving IR\\nclass IRrecv\\n{\\npublic:\\n  IRrecv(int recvpin);\\n  void blink13(int blinkflag);\\n  int decode(decode_results *results);\\n  void enableIRIn();\\n  void resume();\\nprivate:\\n  // These are called by decode\\n  int getRClevel(decode_results *results, int *offset, int *used, int t1);\\n  long decodeNEC(decode_results *results);\\n  long decodeSony(decode_results *results);\\n  long decodeSanyo(decode_results *results);\\n  long decodeMitsubishi(decode_results *results);\\n  long decodeRC5(decode_results *results);\\n  long decodeRC6(decode_results *results);\\n  long decodePanasonic(decode_results *results);\\n  long decodeJVC(decode_results *results);\\n  long decodeHash(decode_results *results);\\n  int compare(unsigned int oldval, unsigned int newval);\\n\\n} \\n;\\n\\n// Only used for testing; can remove virtual for shorter code\\n#ifdef TEST\\n#define VIRTUAL virtual\\n#else\\n#define VIRTUAL\\n#endif\\n\\nclass IRsend\\n{\\npublic:\\n  IRsend() {}\\n  void sendNEC(unsigned long data, int nbits);\\n  void sendSony(unsigned long data, int nbits);\\n  // Neither Sanyo nor Mitsubishi send is implemented yet\\n  //  void sendSanyo(unsigned long data, int nbits);\\n  //  void sendMitsubishi(unsigned long data, int nbits);\\n  void sendRaw(unsigned int buf[], int len, int hz);\\n  void sendRC5(unsigned long data, int nbits);\\n  void sendRC6(unsigned long data, int nbits);\\n  void sendDISH(unsigned long data, int nbits);\\n  void sendSharp(unsigned long data, int nbits);\\n  void sendPanasonic(unsigned int address, unsigned long data);\\n  void sendJVC(unsigned long data, int nbits, int repeat); // *Note instead of sending the REPEAT constant if you want the JVC repeat signal sent, send the original code value and change the repeat argument from 0 to 1. JVC protocol repeats by skipping the header NOT by sending a separate code value like NEC does.\\n  // private:\\n  void enableIROut(int khz);\\n  VIRTUAL void mark(int usec);\\n  VIRTUAL void space(int usec);\\n}\\n;\\n\\n// Some useful constants\\n\\n#define USECPERTICK 50  // microseconds per clock interrupt tick\\n#define RAWBUF 100 // Length of raw duration buffer\\n\\n// Marks tend to be 100us too long, and spaces 100us too short\\n// when received due to sensor lag.\\n#define MARK_EXCESS 100\\n\\n#endif\\n\" }",
    "{ filename: \"src/oz.h\", code: \"#ifndef arbot_oz\\r\\n\\r\\nint arbotPort[6][2] ={{ 2, 3 }, { 4, 5 }, {  6, 7 }, { A0, A1 },{ A2, A3 },{ A6, A7 }};\\r\\nint arbotMotor[2][3]={{11,12,13},{10,8,9}};\\r\\nint motorAnahtar=0;\\r\\nint saniye=0;\\r\\n#endif\" }",
    "{ filename: \"src/IRremoteInt.h\", code: \"/*\\n * IRremote\\n * Version 0.1 July, 2009\\n * Copyright 2009 Ken Shirriff\\n * For details, see http://arcfn.com/2009/08/multi-protocol-infrared-remote-library.html\\n *\\n * Modified by Paul Stoffregen <paul@pjrc.com> to support other boards and timers\\n *\\n * Interrupt code based on NECIRrcv by Joe Knapp\\n * http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1210243556\\n * Also influenced by http://zovirl.com/2008/11/12/building-a-universal-remote-with-an-arduino/\\n *\\n * JVC and Panasonic protocol added by Kristian Lauszus (Thanks to zenwheel and other people at the original blog post)\\n */\\n\\n#ifndef IRremoteint_h\\n#define IRremoteint_h\\n\\n#if defined(ARDUINO) && ARDUINO >= 100\\n#include <Arduino.h>\\n#else\\n#include <WProgram.h>\\n#endif\\n\\n// define which timer to use\\n//\\n// Uncomment the timer you wish to use on your board.  If you\\n// are using another library which uses timer2, you have options\\n// to switch IRremote to use a different timer.\\n\\n// Arduino Mega\\n#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n  //#define IR_USE_TIMER1   // tx = pin 11\\n  //#define IR_USE_TIMER2   //   tx = pin 3\\n  #define IR_USE_TIMER3   // tx = pin 5\\n  //#define IR_USE_TIMER4   // tx = pin 6\\n  //#define IR_USE_TIMER5   // tx = pin 46\\n\\n// Teensy 1.0\\n#elif defined(__AVR_AT90USB162__)\\n  #define IR_USE_TIMER1     // tx = pin 17\\n\\n// Teensy 2.0\\n#elif defined(__AVR_ATmega32U4__)\\n  //#define IR_USE_TIMER1   // tx = pin 14\\n  //#define IR_USE_TIMER3   // tx = pin 9\\n  #define IR_USE_TIMER4_HS  // tx = pin 10\\n\\n// Teensy 3.0\\n#elif defined(__MK20DX128__) || defined(__MK20DX256__)\\n  #define IR_USE_TIMER_CMT  // tx = pin 5\\n\\n// Teensy++ 1.0 & 2.0\\n#elif defined(__AVR_AT90USB646__) || defined(__AVR_AT90USB1286__)\\n  //#define IR_USE_TIMER1   // tx = pin 25\\n  #define IR_USE_TIMER2     // tx = pin 1\\n  //#define IR_USE_TIMER3   // tx = pin 16\\n\\n// Sanguino\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n  //#define IR_USE_TIMER1   // tx = pin 13\\n  #define IR_USE_TIMER2     // tx = pin 14\\n\\n// Atmega8\\n#elif defined(__AVR_ATmega8P__) || defined(__AVR_ATmega8__)\\n  #define IR_USE_TIMER1   // tx = pin 9\\n\\n// Arduino Duemilanove, Diecimila, LilyPad, Mini, Fio, etc\\n#else\\n  #define IR_USE_TIMER1    tx = pin 9\\n  //#define IR_USE_TIMER2      tx = pin 3\\n#endif\\n\\n\\n\\n#ifdef F_CPU\\n#define SYSCLOCK F_CPU     // main Arduino clock\\n#else\\n#define SYSCLOCK 16000000  // main Arduino clock\\n#endif\\n\\n#define ERR 0\\n#define DECODED 1\\n\\n\\n// defines for setting and clearing register bits\\n#ifndef cbi\\n#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))\\n#endif\\n#ifndef sbi\\n#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))\\n#endif\\n\\n// Pulse parms are *50-100 for the Mark and *50+100 for the space\\n// First MARK is the one after the long gap\\n// pulse parameters in usec\\n#define NEC_HDR_MARK\\t9000\\n#define NEC_HDR_SPACE\\t4500\\n#define NEC_BIT_MARK\\t560\\n#define NEC_ONE_SPACE\\t1600\\n#define NEC_ZERO_SPACE\\t560\\n#define NEC_RPT_SPACE\\t2250\\n\\n#define SONY_HDR_MARK\\t2400\\n#define SONY_HDR_SPACE\\t600\\n#define SONY_ONE_MARK\\t1200\\n#define SONY_ZERO_MARK\\t600\\n#define SONY_RPT_LENGTH 45000\\n#define SONY_DOUBLE_SPACE_USECS  500  // usually ssee 713 - not using ticks as get number wrapround\\n\\n// SA 8650B\\n#define SANYO_HDR_MARK\\t3500  // seen range 3500\\n#define SANYO_HDR_SPACE\\t950 //  seen 950\\n#define SANYO_ONE_MARK\\t2400 // seen 2400  \\n#define SANYO_ZERO_MARK 700 //  seen 700\\n#define SANYO_DOUBLE_SPACE_USECS  800  // usually ssee 713 - not using ticks as get number wrapround\\n#define SANYO_RPT_LENGTH 45000\\n\\n// Mitsubishi RM 75501\\n// 14200 7 41 7 42 7 42 7 17 7 17 7 18 7 41 7 18 7 17 7 17 7 18 7 41 8 17 7 17 7 18 7 17 7 \\n\\n// #define MITSUBISHI_HDR_MARK\\t250  // seen range 3500\\n#define MITSUBISHI_HDR_SPACE\\t350 //  7*50+100\\n#define MITSUBISHI_ONE_MARK\\t1950 // 41*50-100\\n#define MITSUBISHI_ZERO_MARK  750 // 17*50-100\\n// #define MITSUBISHI_DOUBLE_SPACE_USECS  800  // usually ssee 713 - not using ticks as get number wrapround\\n// #define MITSUBISHI_RPT_LENGTH 45000\\n\\n\\n#define RC5_T1\\t\\t889\\n#define RC5_RPT_LENGTH\\t46000\\n\\n#define RC6_HDR_MARK\\t2666\\n#define RC6_HDR_SPACE\\t889\\n#define RC6_T1\\t\\t444\\n#define RC6_RPT_LENGTH\\t46000\\n\\n#define SHARP_BIT_MARK 245\\n#define SHARP_ONE_SPACE 1805\\n#define SHARP_ZERO_SPACE 795\\n#define SHARP_GAP 600000\\n#define SHARP_TOGGLE_MASK 0x3FF\\n#define SHARP_RPT_SPACE 3000\\n\\n#define DISH_HDR_MARK 400\\n#define DISH_HDR_SPACE 6100\\n#define DISH_BIT_MARK 400\\n#define DISH_ONE_SPACE 1700\\n#define DISH_ZERO_SPACE 2800\\n#define DISH_RPT_SPACE 6200\\n#define DISH_TOP_BIT 0x8000\\n\\n#define PANASONIC_HDR_MARK 3502\\n#define PANASONIC_HDR_SPACE 1750\\n#define PANASONIC_BIT_MARK 502\\n#define PANASONIC_ONE_SPACE 1244\\n#define PANASONIC_ZERO_SPACE 400\\n\\n#define JVC_HDR_MARK 8000\\n#define JVC_HDR_SPACE 4000\\n#define JVC_BIT_MARK 600\\n#define JVC_ONE_SPACE 1600\\n#define JVC_ZERO_SPACE 550\\n#define JVC_RPT_LENGTH 60000\\n\\n#define SHARP_BITS 15\\n#define DISH_BITS 16\\n\\n#define TOLERANCE 25  // percent tolerance in measurements\\n#define LTOL (1.0 - TOLERANCE/100.) \\n#define UTOL (1.0 + TOLERANCE/100.) \\n\\n#define _GAP 5000 // Minimum map between transmissions\\n#define GAP_TICKS (_GAP/USECPERTICK)\\n\\n#define TICKS_LOW(us) (int) (((us)*LTOL/USECPERTICK))\\n#define TICKS_HIGH(us) (int) (((us)*UTOL/USECPERTICK + 1))\\n\\n#ifndef DEBUG\\nint MATCH(int measured, int desired) {return measured >= TICKS_LOW(desired) && measured <= TICKS_HIGH(desired);}\\nint MATCH_MARK(int measured_ticks, int desired_us) {return MATCH(measured_ticks, (desired_us + MARK_EXCESS));}\\nint MATCH_SPACE(int measured_ticks, int desired_us) {return MATCH(measured_ticks, (desired_us - MARK_EXCESS));}\\n// Debugging versions are in IRremote.cpp\\n#endif\\n\\n// receiver states\\n#define STATE_IDLE     2\\n#define STATE_MARK     3\\n#define STATE_SPACE    4\\n#define STATE_STOP     5\\n\\n// information for the interrupt handler\\ntypedef struct {\\n  uint8_t recvpin;           // pin for IR data from detector\\n  uint8_t rcvstate;          // state machine\\n  uint8_t blinkflag;         // TRUE to enable blinking of pin 13 on IR processing\\n  unsigned int timer;     // state timer, counts 50uS ticks.\\n  unsigned int rawbuf[RAWBUF]; // raw data\\n  uint8_t rawlen;         // counter of entries in rawbuf\\n} \\nirparams_t;\\n\\n// Defined in IRremote.cpp\\nextern volatile irparams_t irparams;\\n\\n// IR detector output is active low\\n#define MARK  0\\n#define SPACE 1\\n\\n#define TOPBIT 0x80000000\\n\\n#define NEC_BITS 32\\n#define SONY_BITS 12\\n#define SANYO_BITS 12\\n#define MITSUBISHI_BITS 16\\n#define MIN_RC5_SAMPLES 11\\n#define MIN_RC6_SAMPLES 1\\n#define PANASONIC_BITS 48\\n#define JVC_BITS 16\\n\\n\\n\\n\\n// defines for timer2 (8 bits)\\n#if defined(IR_USE_TIMER2)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR2A |= _BV(COM2B1))\\n#define TIMER_DISABLE_PWM    (TCCR2A &= ~(_BV(COM2B1)))\\n#define TIMER_ENABLE_INTR    (TIMSK2 = _BV(OCIE2A))\\n#define TIMER_DISABLE_INTR   (TIMSK2 = 0)\\n#define TIMER_INTR_NAME      TIMER2_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint8_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR2A = _BV(WGM20); \\\\\\n  TCCR2B = _BV(WGM22) | _BV(CS20); \\\\\\n  OCR2A = pwmval; \\\\\\n  OCR2B = pwmval / 3; \\\\\\n})\\n#define TIMER_COUNT_TOP      (SYSCLOCK * USECPERTICK / 1000000)\\n#if (TIMER_COUNT_TOP < 256)\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR2A = _BV(WGM21); \\\\\\n  TCCR2B = _BV(CS20); \\\\\\n  OCR2A = TIMER_COUNT_TOP; \\\\\\n  TCNT2 = 0; \\\\\\n})\\n#else\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR2A = _BV(WGM21); \\\\\\n  TCCR2B = _BV(CS21); \\\\\\n  OCR2A = TIMER_COUNT_TOP / 8; \\\\\\n  TCNT2 = 0; \\\\\\n})\\n#endif\\n#if defined(CORE_OC2B_PIN)\\n#define TIMER_PWM_PIN        CORE_OC2B_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        9  /* Arduino Mega */\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n#define TIMER_PWM_PIN        14 /* Sanguino */\\n#else\\n#define TIMER_PWM_PIN        3  /* Arduino Duemilanove, Diecimila, LilyPad, etc */\\n#endif\\n\\n\\n// defines for timer1 (16 bits)\\n#elif defined(IR_USE_TIMER1)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR1A |= _BV(COM1A1))\\n#define TIMER_DISABLE_PWM    (TCCR1A &= ~(_BV(COM1A1)))\\n#if defined(__AVR_ATmega8P__) || defined(__AVR_ATmega8__)\\n  #define TIMER_ENABLE_INTR    (TIMSK = _BV(OCIE1A))\\n  #define TIMER_DISABLE_INTR   (TIMSK = 0)\\n#else\\n  #define TIMER_ENABLE_INTR    (TIMSK1 = _BV(OCIE1A))\\n  #define TIMER_DISABLE_INTR   (TIMSK1 = 0)\\n#endif\\n#define TIMER_INTR_NAME      TIMER1_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR1A = _BV(WGM11); \\\\\\n  TCCR1B = _BV(WGM13) | _BV(CS10); \\\\\\n  ICR1 = pwmval; \\\\\\n  OCR1A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR1A = 0; \\\\\\n  TCCR1B = _BV(WGM12) | _BV(CS10); \\\\\\n  OCR1A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT1 = 0; \\\\\\n})\\n#if defined(CORE_OC1A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC1A_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        11  /* Arduino Mega */\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n#define TIMER_PWM_PIN        13 /* Sanguino */\\n#else\\n#define TIMER_PWM_PIN        9  /* Arduino Duemilanove, Diecimila, LilyPad, etc */\\n#endif\\n\\n\\n// defines for timer3 (16 bits)\\n#elif defined(IR_USE_TIMER3)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR3A |= _BV(COM3A1))\\n#define TIMER_DISABLE_PWM    (TCCR3A &= ~(_BV(COM3A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK3 = _BV(OCIE3A))\\n#define TIMER_DISABLE_INTR   (TIMSK3 = 0)\\n#define TIMER_INTR_NAME      TIMER3_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR3A = _BV(WGM31); \\\\\\n  TCCR3B = _BV(WGM33) | _BV(CS30); \\\\\\n  ICR3 = pwmval; \\\\\\n  OCR3A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR3A = 0; \\\\\\n  TCCR3B = _BV(WGM32) | _BV(CS30); \\\\\\n  OCR3A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT3 = 0; \\\\\\n})\\n#if defined(CORE_OC3A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC3A_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        5  /* Arduino Mega */\\n#else\\n#error \\\"Please add OC3A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for timer4 (10 bits, high speed option)\\n#elif defined(IR_USE_TIMER4_HS)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR4A |= _BV(COM4A1))\\n#define TIMER_DISABLE_PWM    (TCCR4A &= ~(_BV(COM4A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK4 = _BV(TOIE4))\\n#define TIMER_DISABLE_INTR   (TIMSK4 = 0)\\n#define TIMER_INTR_NAME      TIMER4_OVF_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR4A = (1<<PWM4A); \\\\\\n  TCCR4B = _BV(CS40); \\\\\\n  TCCR4C = 0; \\\\\\n  TCCR4D = (1<<WGM40); \\\\\\n  TCCR4E = 0; \\\\\\n  TC4H = pwmval >> 8; \\\\\\n  OCR4C = pwmval; \\\\\\n  TC4H = (pwmval / 3) >> 8; \\\\\\n  OCR4A = (pwmval / 3) & 255; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR4A = 0; \\\\\\n  TCCR4B = _BV(CS40); \\\\\\n  TCCR4C = 0; \\\\\\n  TCCR4D = 0; \\\\\\n  TCCR4E = 0; \\\\\\n  TC4H = (SYSCLOCK * USECPERTICK / 1000000) >> 8; \\\\\\n  OCR4C = (SYSCLOCK * USECPERTICK / 1000000) & 255; \\\\\\n  TC4H = 0; \\\\\\n  TCNT4 = 0; \\\\\\n})\\n#if defined(CORE_OC4A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC4A_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega32U4__)\\n#define TIMER_PWM_PIN        13  /* Leonardo */\\n#else\\n#error \\\"Please add OC4A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for timer4 (16 bits)\\n#elif defined(IR_USE_TIMER4)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR4A |= _BV(COM4A1))\\n#define TIMER_DISABLE_PWM    (TCCR4A &= ~(_BV(COM4A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK4 = _BV(OCIE4A))\\n#define TIMER_DISABLE_INTR   (TIMSK4 = 0)\\n#define TIMER_INTR_NAME      TIMER4_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR4A = _BV(WGM41); \\\\\\n  TCCR4B = _BV(WGM43) | _BV(CS40); \\\\\\n  ICR4 = pwmval; \\\\\\n  OCR4A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR4A = 0; \\\\\\n  TCCR4B = _BV(WGM42) | _BV(CS40); \\\\\\n  OCR4A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT4 = 0; \\\\\\n})\\n#if defined(CORE_OC4A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC4A_PIN\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        6  /* Arduino Mega */\\n#else\\n#error \\\"Please add OC4A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for timer5 (16 bits)\\n#elif defined(IR_USE_TIMER5)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR5A |= _BV(COM5A1))\\n#define TIMER_DISABLE_PWM    (TCCR5A &= ~(_BV(COM5A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK5 = _BV(OCIE5A))\\n#define TIMER_DISABLE_INTR   (TIMSK5 = 0)\\n#define TIMER_INTR_NAME      TIMER5_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR5A = _BV(WGM51); \\\\\\n  TCCR5B = _BV(WGM53) | _BV(CS50); \\\\\\n  ICR5 = pwmval; \\\\\\n  OCR5A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR5A = 0; \\\\\\n  TCCR5B = _BV(WGM52) | _BV(CS50); \\\\\\n  OCR5A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT5 = 0; \\\\\\n})\\n#if defined(CORE_OC5A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC5A_PIN\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        46  /* Arduino Mega */\\n#else\\n#error \\\"Please add OC5A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for special carrier modulator timer\\n#elif defined(IR_USE_TIMER_CMT)\\n#define TIMER_RESET ({\\t\\t\\t\\\\\\n\\tuint8_t tmp = CMT_MSC;\\t\\t\\\\\\n\\tCMT_CMD2 = 30;\\t\\t\\t\\\\\\n})\\n#define TIMER_ENABLE_PWM     CORE_PIN5_CONFIG = PORT_PCR_MUX(2)|PORT_PCR_DSE|PORT_PCR_SRE\\n#define TIMER_DISABLE_PWM    CORE_PIN5_CONFIG = PORT_PCR_MUX(1)|PORT_PCR_DSE|PORT_PCR_SRE\\n#define TIMER_ENABLE_INTR    NVIC_ENABLE_IRQ(IRQ_CMT)\\n#define TIMER_DISABLE_INTR   NVIC_DISABLE_IRQ(IRQ_CMT)\\n#define TIMER_INTR_NAME      cmt_isr\\n#ifdef ISR\\n#undef ISR\\n#endif\\n#define ISR(f) void f(void)\\n#if F_BUS == 48000000\\n#define CMT_PPS_VAL 5\\n#else\\n#define CMT_PPS_VAL 2\\n#endif\\n#define TIMER_CONFIG_KHZ(val) ({ \\t\\\\\\n\\tSIM_SCGC4 |= SIM_SCGC4_CMT;\\t\\\\\\n\\tSIM_SOPT2 |= SIM_SOPT2_PTD7PAD;\\t\\\\\\n\\tCMT_PPS = CMT_PPS_VAL;\\t\\t\\\\\\n\\tCMT_CGH1 = 2667 / val;\\t\\t\\\\\\n\\tCMT_CGL1 = 5333 / val;\\t\\t\\\\\\n\\tCMT_CMD1 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD2 = 30;\\t\\t\\t\\\\\\n\\tCMT_CMD3 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD4 = 0;\\t\\t\\t\\\\\\n\\tCMT_OC = 0x60;\\t\\t\\t\\\\\\n\\tCMT_MSC = 0x01;\\t\\t\\t\\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({\\t\\\\\\n\\tSIM_SCGC4 |= SIM_SCGC4_CMT;\\t\\\\\\n\\tCMT_PPS = CMT_PPS_VAL;\\t\\t\\\\\\n\\tCMT_CGH1 = 1;\\t\\t\\t\\\\\\n\\tCMT_CGL1 = 1;\\t\\t\\t\\\\\\n\\tCMT_CMD1 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD2 = 30;\\t\\t\\t\\\\\\n\\tCMT_CMD3 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD4 = 19;\\t\\t\\t\\\\\\n\\tCMT_OC = 0;\\t\\t\\t\\\\\\n\\tCMT_MSC = 0x03;\\t\\t\\t\\\\\\n})\\n#define TIMER_PWM_PIN        5\\n\\n\\n#else // unknown timer\\n#error \\\"Internal code configuration error, no known IR_USE_TIMER# defined\\\\n\\\"\\n#endif\\n\\n\\n// defines for blinking the LED\\n#if defined(CORE_LED0_PIN)\\n#define BLINKLED       CORE_LED0_PIN\\n#define BLINKLED_ON()  (digitalWrite(CORE_LED0_PIN, HIGH))\\n#define BLINKLED_OFF() (digitalWrite(CORE_LED0_PIN, LOW))\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define BLINKLED       13\\n#define BLINKLED_ON()  (PORTB |= B10000000)\\n#define BLINKLED_OFF() (PORTB &= B01111111)\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n#define BLINKLED       0\\n#define BLINKLED_ON()  (PORTD |= B00000001)\\n#define BLINKLED_OFF() (PORTD &= B11111110)\\n#else\\n#define BLINKLED       13\\n#define BLINKLED_ON()  (PORTB |= B00100000)\\n#define BLINKLED_OFF() (PORTB &= B11011111)\\n#endif\\n\\n#endif\\n\" }",
    "{ filename: \"src/IRremoteInt.h\", code: \"/*\\n * IRremote\\n * Version 0.1 July, 2009\\n * Copyright 2009 Ken Shirriff\\n * For details, see http://arcfn.com/2009/08/multi-protocol-infrared-remote-library.html\\n *\\n * Modified by Paul Stoffregen <paul@pjrc.com> to support other boards and timers\\n *\\n * Interrupt code based on NECIRrcv by Joe Knapp\\n * http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1210243556\\n * Also influenced by http://zovirl.com/2008/11/12/building-a-universal-remote-with-an-arduino/\\n *\\n * JVC and Panasonic protocol added by Kristian Lauszus (Thanks to zenwheel and other people at the original blog post)\\n */\\n\\n#ifndef IRremoteint_h\\n#define IRremoteint_h\\n\\n#if defined(ARDUINO) && ARDUINO >= 100\\n#include <Arduino.h>\\n#else\\n#include <WProgram.h>\\n#endif\\n\\n// define which timer to use\\n//\\n// Uncomment the timer you wish to use on your board.  If you\\n// are using another library which uses timer2, you have options\\n// to switch IRremote to use a different timer.\\n\\n// Arduino Mega\\n#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n  //#define IR_USE_TIMER1   // tx = pin 11\\n  //#define IR_USE_TIMER2     // tx = pin 9\\n  #define IR_USE_TIMER3   // tx = pin 5\\n  //#define IR_USE_TIMER4   // tx = pin 6\\n  //#define IR_USE_TIMER5   // tx = pin 46\\n\\n// Teensy 1.0\\n#elif defined(__AVR_AT90USB162__)\\n  #define IR_USE_TIMER1     // tx = pin 17\\n\\n// Teensy 2.0\\n#elif defined(__AVR_ATmega32U4__)\\n  //#define IR_USE_TIMER1   // tx = pin 14\\n  //#define IR_USE_TIMER3   // tx = pin 9\\n  #define IR_USE_TIMER4_HS  // tx = pin 10\\n\\n// Teensy 3.0\\n#elif defined(__MK20DX128__) || defined(__MK20DX256__)\\n  #define IR_USE_TIMER_CMT  // tx = pin 5\\n\\n// Teensy++ 1.0 & 2.0\\n#elif defined(__AVR_AT90USB646__) || defined(__AVR_AT90USB1286__)\\n  //#define IR_USE_TIMER1   // tx = pin 25\\n  #define IR_USE_TIMER2     // tx = pin 1\\n  //#define IR_USE_TIMER3   // tx = pin 16\\n\\n// Sanguino\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n  //#define IR_USE_TIMER1   // tx = pin 13\\n  #define IR_USE_TIMER2     // tx = pin 14\\n\\n// Atmega8\\n#elif defined(__AVR_ATmega8P__) || defined(__AVR_ATmega8__)\\n  #define IR_USE_TIMER1   // tx = pin 9\\n\\n// Arduino Duemilanove, Diecimila, LilyPad, Mini, Fio, etc\\n#else\\n  //#define IR_USE_TIMER1   // tx = pin 9\\n  #define IR_USE_TIMER2     // tx = pin 3\\n#endif\\n\\n\\n\\n#ifdef F_CPU\\n#define SYSCLOCK F_CPU     // main Arduino clock\\n#else\\n#define SYSCLOCK 16000000  // main Arduino clock\\n#endif\\n\\n#define ERR 0\\n#define DECODED 1\\n\\n\\n// defines for setting and clearing register bits\\n#ifndef cbi\\n#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))\\n#endif\\n#ifndef sbi\\n#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))\\n#endif\\n\\n// Pulse parms are *50-100 for the Mark and *50+100 for the space\\n// First MARK is the one after the long gap\\n// pulse parameters in usec\\n#define NEC_HDR_MARK\\t9000\\n#define NEC_HDR_SPACE\\t4500\\n#define NEC_BIT_MARK\\t560\\n#define NEC_ONE_SPACE\\t1600\\n#define NEC_ZERO_SPACE\\t560\\n#define NEC_RPT_SPACE\\t2250\\n\\n#define SONY_HDR_MARK\\t2400\\n#define SONY_HDR_SPACE\\t600\\n#define SONY_ONE_MARK\\t1200\\n#define SONY_ZERO_MARK\\t600\\n#define SONY_RPT_LENGTH 45000\\n#define SONY_DOUBLE_SPACE_USECS  500  // usually ssee 713 - not using ticks as get number wrapround\\n\\n// SA 8650B\\n#define SANYO_HDR_MARK\\t3500  // seen range 3500\\n#define SANYO_HDR_SPACE\\t950 //  seen 950\\n#define SANYO_ONE_MARK\\t2400 // seen 2400  \\n#define SANYO_ZERO_MARK 700 //  seen 700\\n#define SANYO_DOUBLE_SPACE_USECS  800  // usually ssee 713 - not using ticks as get number wrapround\\n#define SANYO_RPT_LENGTH 45000\\n\\n// Mitsubishi RM 75501\\n// 14200 7 41 7 42 7 42 7 17 7 17 7 18 7 41 7 18 7 17 7 17 7 18 7 41 8 17 7 17 7 18 7 17 7 \\n\\n// #define MITSUBISHI_HDR_MARK\\t250  // seen range 3500\\n#define MITSUBISHI_HDR_SPACE\\t350 //  7*50+100\\n#define MITSUBISHI_ONE_MARK\\t1950 // 41*50-100\\n#define MITSUBISHI_ZERO_MARK  750 // 17*50-100\\n// #define MITSUBISHI_DOUBLE_SPACE_USECS  800  // usually ssee 713 - not using ticks as get number wrapround\\n// #define MITSUBISHI_RPT_LENGTH 45000\\n\\n\\n#define RC5_T1\\t\\t889\\n#define RC5_RPT_LENGTH\\t46000\\n\\n#define RC6_HDR_MARK\\t2666\\n#define RC6_HDR_SPACE\\t889\\n#define RC6_T1\\t\\t444\\n#define RC6_RPT_LENGTH\\t46000\\n\\n#define SHARP_BIT_MARK 245\\n#define SHARP_ONE_SPACE 1805\\n#define SHARP_ZERO_SPACE 795\\n#define SHARP_GAP 600000\\n#define SHARP_TOGGLE_MASK 0x3FF\\n#define SHARP_RPT_SPACE 3000\\n\\n#define DISH_HDR_MARK 400\\n#define DISH_HDR_SPACE 6100\\n#define DISH_BIT_MARK 400\\n#define DISH_ONE_SPACE 1700\\n#define DISH_ZERO_SPACE 2800\\n#define DISH_RPT_SPACE 6200\\n#define DISH_TOP_BIT 0x8000\\n\\n#define PANASONIC_HDR_MARK 3502\\n#define PANASONIC_HDR_SPACE 1750\\n#define PANASONIC_BIT_MARK 502\\n#define PANASONIC_ONE_SPACE 1244\\n#define PANASONIC_ZERO_SPACE 400\\n\\n#define JVC_HDR_MARK 8000\\n#define JVC_HDR_SPACE 4000\\n#define JVC_BIT_MARK 600\\n#define JVC_ONE_SPACE 1600\\n#define JVC_ZERO_SPACE 550\\n#define JVC_RPT_LENGTH 60000\\n\\n#define SHARP_BITS 15\\n#define DISH_BITS 16\\n\\n#define TOLERANCE 25  // percent tolerance in measurements\\n#define LTOL (1.0 - TOLERANCE/100.) \\n#define UTOL (1.0 + TOLERANCE/100.) \\n\\n#define _GAP 5000 // Minimum map between transmissions\\n#define GAP_TICKS (_GAP/USECPERTICK)\\n\\n#define TICKS_LOW(us) (int) (((us)*LTOL/USECPERTICK))\\n#define TICKS_HIGH(us) (int) (((us)*UTOL/USECPERTICK + 1))\\n\\n#ifndef DEBUG\\nint MATCH(int measured, int desired) {return measured >= TICKS_LOW(desired) && measured <= TICKS_HIGH(desired);}\\nint MATCH_MARK(int measured_ticks, int desired_us) {return MATCH(measured_ticks, (desired_us + MARK_EXCESS));}\\nint MATCH_SPACE(int measured_ticks, int desired_us) {return MATCH(measured_ticks, (desired_us - MARK_EXCESS));}\\n// Debugging versions are in IRremote.cpp\\n#endif\\n\\n// receiver states\\n#define STATE_IDLE     2\\n#define STATE_MARK     3\\n#define STATE_SPACE    4\\n#define STATE_STOP     5\\n\\n// information for the interrupt handler\\ntypedef struct {\\n  uint8_t recvpin;           // pin for IR data from detector\\n  uint8_t rcvstate;          // state machine\\n  uint8_t blinkflag;         // TRUE to enable blinking of pin 13 on IR processing\\n  unsigned int timer;     // state timer, counts 50uS ticks.\\n  unsigned int rawbuf[RAWBUF]; // raw data\\n  uint8_t rawlen;         // counter of entries in rawbuf\\n} \\nirparams_t;\\n\\n// Defined in IRremote.cpp\\nextern volatile irparams_t irparams;\\n\\n// IR detector output is active low\\n#define MARK  0\\n#define SPACE 1\\n\\n#define TOPBIT 0x80000000\\n\\n#define NEC_BITS 32\\n#define SONY_BITS 12\\n#define SANYO_BITS 12\\n#define MITSUBISHI_BITS 16\\n#define MIN_RC5_SAMPLES 11\\n#define MIN_RC6_SAMPLES 1\\n#define PANASONIC_BITS 48\\n#define JVC_BITS 16\\n\\n\\n\\n\\n// defines for timer2 (8 bits)\\n#if defined(IR_USE_TIMER2)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR2A |= _BV(COM2B1))\\n#define TIMER_DISABLE_PWM    (TCCR2A &= ~(_BV(COM2B1)))\\n#define TIMER_ENABLE_INTR    (TIMSK2 = _BV(OCIE2A))\\n#define TIMER_DISABLE_INTR   (TIMSK2 = 0)\\n#define TIMER_INTR_NAME      TIMER2_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint8_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR2A = _BV(WGM20); \\\\\\n  TCCR2B = _BV(WGM22) | _BV(CS20); \\\\\\n  OCR2A = pwmval; \\\\\\n  OCR2B = pwmval / 3; \\\\\\n})\\n#define TIMER_COUNT_TOP      (SYSCLOCK * USECPERTICK / 1000000)\\n#if (TIMER_COUNT_TOP < 256)\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR2A = _BV(WGM21); \\\\\\n  TCCR2B = _BV(CS20); \\\\\\n  OCR2A = TIMER_COUNT_TOP; \\\\\\n  TCNT2 = 0; \\\\\\n})\\n#else\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR2A = _BV(WGM21); \\\\\\n  TCCR2B = _BV(CS21); \\\\\\n  OCR2A = TIMER_COUNT_TOP / 8; \\\\\\n  TCNT2 = 0; \\\\\\n})\\n#endif\\n#if defined(CORE_OC2B_PIN)\\n#define TIMER_PWM_PIN        CORE_OC2B_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        9  /* Arduino Mega */\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n#define TIMER_PWM_PIN        14 /* Sanguino */\\n#else\\n#define TIMER_PWM_PIN        3  /* Arduino Duemilanove, Diecimila, LilyPad, etc */\\n#endif\\n\\n\\n// defines for timer1 (16 bits)\\n#elif defined(IR_USE_TIMER1)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR1A |= _BV(COM1A1))\\n#define TIMER_DISABLE_PWM    (TCCR1A &= ~(_BV(COM1A1)))\\n#if defined(__AVR_ATmega8P__) || defined(__AVR_ATmega8__)\\n  #define TIMER_ENABLE_INTR    (TIMSK = _BV(OCIE1A))\\n  #define TIMER_DISABLE_INTR   (TIMSK = 0)\\n#else\\n  #define TIMER_ENABLE_INTR    (TIMSK1 = _BV(OCIE1A))\\n  #define TIMER_DISABLE_INTR   (TIMSK1 = 0)\\n#endif\\n#define TIMER_INTR_NAME      TIMER1_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR1A = _BV(WGM11); \\\\\\n  TCCR1B = _BV(WGM13) | _BV(CS10); \\\\\\n  ICR1 = pwmval; \\\\\\n  OCR1A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR1A = 0; \\\\\\n  TCCR1B = _BV(WGM12) | _BV(CS10); \\\\\\n  OCR1A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT1 = 0; \\\\\\n})\\n#if defined(CORE_OC1A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC1A_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        11  /* Arduino Mega */\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n#define TIMER_PWM_PIN        13 /* Sanguino */\\n#else\\n#define TIMER_PWM_PIN        9  /* Arduino Duemilanove, Diecimila, LilyPad, etc */\\n#endif\\n\\n\\n// defines for timer3 (16 bits)\\n#elif defined(IR_USE_TIMER3)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR3A |= _BV(COM3A1))\\n#define TIMER_DISABLE_PWM    (TCCR3A &= ~(_BV(COM3A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK3 = _BV(OCIE3A))\\n#define TIMER_DISABLE_INTR   (TIMSK3 = 0)\\n#define TIMER_INTR_NAME      TIMER3_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR3A = _BV(WGM31); \\\\\\n  TCCR3B = _BV(WGM33) | _BV(CS30); \\\\\\n  ICR3 = pwmval; \\\\\\n  OCR3A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR3A = 0; \\\\\\n  TCCR3B = _BV(WGM32) | _BV(CS30); \\\\\\n  OCR3A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT3 = 0; \\\\\\n})\\n#if defined(CORE_OC3A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC3A_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        5  /* Arduino Mega */\\n#else\\n#error \\\"Please add OC3A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for timer4 (10 bits, high speed option)\\n#elif defined(IR_USE_TIMER4_HS)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR4A |= _BV(COM4A1))\\n#define TIMER_DISABLE_PWM    (TCCR4A &= ~(_BV(COM4A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK4 = _BV(TOIE4))\\n#define TIMER_DISABLE_INTR   (TIMSK4 = 0)\\n#define TIMER_INTR_NAME      TIMER4_OVF_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR4A = (1<<PWM4A); \\\\\\n  TCCR4B = _BV(CS40); \\\\\\n  TCCR4C = 0; \\\\\\n  TCCR4D = (1<<WGM40); \\\\\\n  TCCR4E = 0; \\\\\\n  TC4H = pwmval >> 8; \\\\\\n  OCR4C = pwmval; \\\\\\n  TC4H = (pwmval / 3) >> 8; \\\\\\n  OCR4A = (pwmval / 3) & 255; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR4A = 0; \\\\\\n  TCCR4B = _BV(CS40); \\\\\\n  TCCR4C = 0; \\\\\\n  TCCR4D = 0; \\\\\\n  TCCR4E = 0; \\\\\\n  TC4H = (SYSCLOCK * USECPERTICK / 1000000) >> 8; \\\\\\n  OCR4C = (SYSCLOCK * USECPERTICK / 1000000) & 255; \\\\\\n  TC4H = 0; \\\\\\n  TCNT4 = 0; \\\\\\n})\\n#if defined(CORE_OC4A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC4A_PIN  /* Teensy */\\n#elif defined(__AVR_ATmega32U4__)\\n#define TIMER_PWM_PIN        13  /* Leonardo */\\n#else\\n#error \\\"Please add OC4A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for timer4 (16 bits)\\n#elif defined(IR_USE_TIMER4)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR4A |= _BV(COM4A1))\\n#define TIMER_DISABLE_PWM    (TCCR4A &= ~(_BV(COM4A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK4 = _BV(OCIE4A))\\n#define TIMER_DISABLE_INTR   (TIMSK4 = 0)\\n#define TIMER_INTR_NAME      TIMER4_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR4A = _BV(WGM41); \\\\\\n  TCCR4B = _BV(WGM43) | _BV(CS40); \\\\\\n  ICR4 = pwmval; \\\\\\n  OCR4A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR4A = 0; \\\\\\n  TCCR4B = _BV(WGM42) | _BV(CS40); \\\\\\n  OCR4A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT4 = 0; \\\\\\n})\\n#if defined(CORE_OC4A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC4A_PIN\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        6  /* Arduino Mega */\\n#else\\n#error \\\"Please add OC4A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for timer5 (16 bits)\\n#elif defined(IR_USE_TIMER5)\\n#define TIMER_RESET\\n#define TIMER_ENABLE_PWM     (TCCR5A |= _BV(COM5A1))\\n#define TIMER_DISABLE_PWM    (TCCR5A &= ~(_BV(COM5A1)))\\n#define TIMER_ENABLE_INTR    (TIMSK5 = _BV(OCIE5A))\\n#define TIMER_DISABLE_INTR   (TIMSK5 = 0)\\n#define TIMER_INTR_NAME      TIMER5_COMPA_vect\\n#define TIMER_CONFIG_KHZ(val) ({ \\\\\\n  const uint16_t pwmval = SYSCLOCK / 2000 / (val); \\\\\\n  TCCR5A = _BV(WGM51); \\\\\\n  TCCR5B = _BV(WGM53) | _BV(CS50); \\\\\\n  ICR5 = pwmval; \\\\\\n  OCR5A = pwmval / 3; \\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({ \\\\\\n  TCCR5A = 0; \\\\\\n  TCCR5B = _BV(WGM52) | _BV(CS50); \\\\\\n  OCR5A = SYSCLOCK * USECPERTICK / 1000000; \\\\\\n  TCNT5 = 0; \\\\\\n})\\n#if defined(CORE_OC5A_PIN)\\n#define TIMER_PWM_PIN        CORE_OC5A_PIN\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define TIMER_PWM_PIN        46  /* Arduino Mega */\\n#else\\n#error \\\"Please add OC5A pin number here\\\\n\\\"\\n#endif\\n\\n\\n// defines for special carrier modulator timer\\n#elif defined(IR_USE_TIMER_CMT)\\n#define TIMER_RESET ({\\t\\t\\t\\\\\\n\\tuint8_t tmp = CMT_MSC;\\t\\t\\\\\\n\\tCMT_CMD2 = 30;\\t\\t\\t\\\\\\n})\\n#define TIMER_ENABLE_PWM     CORE_PIN5_CONFIG = PORT_PCR_MUX(2)|PORT_PCR_DSE|PORT_PCR_SRE\\n#define TIMER_DISABLE_PWM    CORE_PIN5_CONFIG = PORT_PCR_MUX(1)|PORT_PCR_DSE|PORT_PCR_SRE\\n#define TIMER_ENABLE_INTR    NVIC_ENABLE_IRQ(IRQ_CMT)\\n#define TIMER_DISABLE_INTR   NVIC_DISABLE_IRQ(IRQ_CMT)\\n#define TIMER_INTR_NAME      cmt_isr\\n#ifdef ISR\\n#undef ISR\\n#endif\\n#define ISR(f) void f(void)\\n#if F_BUS == 48000000\\n#define CMT_PPS_VAL 5\\n#else\\n#define CMT_PPS_VAL 2\\n#endif\\n#define TIMER_CONFIG_KHZ(val) ({ \\t\\\\\\n\\tSIM_SCGC4 |= SIM_SCGC4_CMT;\\t\\\\\\n\\tSIM_SOPT2 |= SIM_SOPT2_PTD7PAD;\\t\\\\\\n\\tCMT_PPS = CMT_PPS_VAL;\\t\\t\\\\\\n\\tCMT_CGH1 = 2667 / val;\\t\\t\\\\\\n\\tCMT_CGL1 = 5333 / val;\\t\\t\\\\\\n\\tCMT_CMD1 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD2 = 30;\\t\\t\\t\\\\\\n\\tCMT_CMD3 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD4 = 0;\\t\\t\\t\\\\\\n\\tCMT_OC = 0x60;\\t\\t\\t\\\\\\n\\tCMT_MSC = 0x01;\\t\\t\\t\\\\\\n})\\n#define TIMER_CONFIG_NORMAL() ({\\t\\\\\\n\\tSIM_SCGC4 |= SIM_SCGC4_CMT;\\t\\\\\\n\\tCMT_PPS = CMT_PPS_VAL;\\t\\t\\\\\\n\\tCMT_CGH1 = 1;\\t\\t\\t\\\\\\n\\tCMT_CGL1 = 1;\\t\\t\\t\\\\\\n\\tCMT_CMD1 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD2 = 30;\\t\\t\\t\\\\\\n\\tCMT_CMD3 = 0;\\t\\t\\t\\\\\\n\\tCMT_CMD4 = 19;\\t\\t\\t\\\\\\n\\tCMT_OC = 0;\\t\\t\\t\\\\\\n\\tCMT_MSC = 0x03;\\t\\t\\t\\\\\\n})\\n#define TIMER_PWM_PIN        5\\n\\n\\n#else // unknown timer\\n#error \\\"Internal code configuration error, no known IR_USE_TIMER# defined\\\\n\\\"\\n#endif\\n\\n\\n// defines for blinking the LED\\n#if defined(CORE_LED0_PIN)\\n#define BLINKLED       CORE_LED0_PIN\\n#define BLINKLED_ON()  (digitalWrite(CORE_LED0_PIN, HIGH))\\n#define BLINKLED_OFF() (digitalWrite(CORE_LED0_PIN, LOW))\\n#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)\\n#define BLINKLED       13\\n#define BLINKLED_ON()  (PORTB |= B10000000)\\n#define BLINKLED_OFF() (PORTB &= B01111111)\\n#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)\\n#define BLINKLED       0\\n#define BLINKLED_ON()  (PORTD |= B00000001)\\n#define BLINKLED_OFF() (PORTD &= B11111110)\\n#else\\n#define BLINKLED       13\\n#define BLINKLED_ON()  (PORTB |= B00100000)\\n#define BLINKLED_OFF() (PORTB &= B11011111)\\n#endif\\n\\n#endif\\n\" }",
    "{ filename: \"src/TimerFreeTone.cpp\", code: \"// ---------------------------------------------------------------------------\\r\\n// Created by Tim Eckel - teckel@leethost.com\\r\\n// Copyright 2016 License: GNU GPL v3 http://www.gnu.org/licenses/gpl-3.0.html\\r\\n//\\r\\n// See \\\"TimerFreeTone.h\\\" for purpose, syntax, version history, links, and more.\\r\\n// ---------------------------------------------------------------------------\\r\\n\\r\\n#include \\\"TimerFreeTone.h\\\"\\r\\n\\r\\nuint8_t _tft_volume[] = { 255, 200, 150, 125, 100, 87, 50, 33, 22, 2 }; // Duty for linear volume control.\\r\\n\\r\\nvoid TimerFreeTone(uint8_t pin, unsigned long frequency, unsigned int duration, uint8_t volume) {\\r\\n\\tif (frequency == 0 || volume == 0) { // If frequency or volume are zero, just wait duration and exit.\\r\\n\\t\\tdelay(duration);\\r\\n\\t\\treturn;\\r\\n\\t} \\r\\n\\tfrequency = 1000000 / frequency;                              // Calculate the square wave length (in microseconds).\\r\\n\\tuint32_t duty = frequency / _tft_volume[min(volume, 10) - 1]; // Calculate the duty cycle (volume).\\r\\n#ifdef __AVR__\\r\\n\\tuint8_t pinBit = digitalPinToBitMask(pin);                                  // Get the bitmask for the pin.\\r\\n\\tvolatile uint8_t *pinOutput = (uint8_t *) portOutputRegister(digitalPinToPort(pin)); // Get the port register for the pin.\\r\\n\\tvolatile uint8_t *portMode = (uint8_t *) portModeRegister(digitalPinToPort(pin));    // Get the port mode register for the pin.\\r\\n\\t*portMode |= pinBit;                                                        // Set pin to output mode.\\r\\n#else\\r\\n\\tpinMode(pin, OUTPUT);                                                       // Set pin to output mode.\\r\\n#endif\\r\\n\\r\\n\\tuint32_t startTime = millis();           // Starting time of note.\\r\\n\\twhile(millis() - startTime < duration) { // Loop for the duration.\\r\\n\\t#ifdef __AVR__\\r\\n\\t\\t*pinOutput |= pinBit;    // Set pin high.\\r\\n\\t\\tdelayMicroseconds(duty); // Square wave duration (how long to leave pin high).\\r\\n\\t\\t*pinOutput &= ~pinBit;   // Set pin low.\\r\\n\\t#else\\r\\n\\t\\tdigitalWrite(pin,HIGH);  // Set pin high.\\r\\n\\t\\tdelayMicroseconds(duty); // Square wave duration (how long to leave pin high).\\r\\n\\t\\tdigitalWrite(pin,LOW);   // Set pin low.\\r\\n\\t#endif\\r\\n\\t\\tdelayMicroseconds(frequency - duty); // Square wave duration (how long to leave pin low).\\r\\n\\t}\\r\\n}\" }",
    "{ filename: \"src/TimerFreeTone.h\", code: \"// ---------------------------------------------------------------------------\\r\\n// TimerFreeTone Library - v1.5 - 09/12/2016\\r\\n//\\r\\n// AUTHOR/LICENSE:\\r\\n// Created by Tim Eckel - teckel@leethost.com\\r\\n// Copyright 2016 License: GNU GPL v3 http://www.gnu.org/licenses/gpl-3.0.html\\r\\n//\\r\\n// LINKS:\\r\\n// Project home: https://bitbucket.org/teckel12/arduino-timer-free-tone/wiki/Home\\r\\n// Blog: http://forum.arduino.cc/index.php?topic=235774.0\\r\\n//\\r\\n// DISCLAIMER:\\r\\n// This software is furnished \\\"as is\\\", without technical support, and with no \\r\\n// warranty, express or implied, as to its usefulness for any purpose.\\r\\n//\\r\\n// PURPOSE:\\r\\n// Doesn't use timers which frees up conflicts with other libraries. Compatible\\r\\n// with all ATmega, ATtiny and ARM-based microcontrollers. About 1,500 bytes\\r\\n// smaller binary sketch size than the standard tone library. Exclusive use of\\r\\n// port registers for AVR-based microcontrollers for fastest and smallest code.\\r\\n// Close to a plug-in replacement for the standard Tone library.\\r\\n//\\r\\n// SYNTAX:\\r\\n//   TimerFreeTone( pin, frequency, duration [, volume ] ) - Play a note on pin at frequency in Hz for duration in milliseconds.\\r\\n//     Parameters:\\r\\n//       * pin        - Pin speaker is wired to (other wire to ground, be sure to add an inline 100 ohm resistor).\\r\\n//       * frequency  - Play the specified frequency (should work fairly well in the 100 to 15000 Hz range).\\r\\n//       * duration   - Set the duration to play in milliseconds. Range: 0 to 65535 (65.5 seconds).\\r\\n//       * volume     - Optionally set the tone volume level (from 1 to 10), defaults to full volume (10).\\r\\n//\\r\\n// HISTORY:\\r\\n// 09/12/2016 v1.5 - Fixed problem with latest release of the Arduino IDE which\\r\\n// caused the library to totally stop functioning. Adjusted note duration to\\r\\n// not fail on timer rollover. Now delays for note duration when frequency or\\r\\n// volume are zero.\\r\\n//\\r\\n// 08/05/2016 v1.4 - Added optional volume parameter.\\r\\n//\\r\\n// 07/23/2016 v1.3 - Fixed problem with long tone play durations. Changed the\\r\\n// way the note duration is calculated from a suggestion by Paul Stoffregen\\r\\n// (http://www.pjrc.com/teensy/).\\r\\n//\\r\\n// 01/14/2015 v1.2 - Calculates duration differently for higher tone accuracy\\r\\n// and smaller code size.\\r\\n//\\r\\n// 04/30/2014 v1.1 - Automatically sets mode of pin to OUTPUT as does the\\r\\n// standard Tone library. Sets pinOutput variable to volatile to work with\\r\\n// certain microcontrollers. Removed overhead parameter and calculation, fairly\\r\\n// accurate anyway at audible frequencies. Even smaller binary sketch size.\\r\\n//\\r\\n// 04/25/2014 v1.0 - Initial release.\\r\\n//\\r\\n// ---------------------------------------------------------------------------\\r\\n\\r\\n#ifndef TimerFreeTone_h\\r\\n\\t#define TimerFreeTone_h\\r\\n\\r\\n  \\t#if defined(ARDUINO) && ARDUINO >= 100\\r\\n    \\t#include <Arduino.h>\\r\\n  \\t#else\\r\\n    \\t#include <WProgram.h>\\r\\n\\t\\t#include <pins_arduino.h>\\r\\n\\t#endif\\r\\n  \\r\\n\\tvoid TimerFreeTone(uint8_t pin, unsigned long frequency, unsigned int duration, uint8_t volume = 10);\\r\\n#endif\" }"
];